//? cpmf.h
//? C+- by Ulrich Mutze. Status of work 2022-05-06.
//? Copyright (c) 2022 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//? 
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//? 
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for 
//? more details.


#ifndef CPM_F_H_
#define CPM_F_H_
/*
   Purpose: see cpmfl.h

*/
#include <cpmv.h>
   // includes <cpmfl.h>, where the essential template F<X,Y>
   // is defined

namespace CpmFunctions{

   using CpmArrays::V;
   using CpmArrays::IvZ;

   template <class X, class Y>
   V<Y> apply(F<X,Y>  const& f, V<X>  const& xs)
      // action on sequences: applying f to xs gives the return value of
      // the present function
   {
      IvZ d=xs.dom(); // general indexing
      V<Y> res(d);
      for (Z i=xs.b();i<=xs.e();i++) res[i]=f(xs[i]);
      return res;
   }

namespace aux{
#ifdef CPM_Fn
template
   <class X, class P1, class P2, class P3, class P4, class P5, class Y>
class Par5FncObj : public FncObj<X,Y>{

   const P1 p1;
   const P2 p2;
   const P3 p3;
   const P4 p4;
   const P5 p5;
   Y (* const fp)
      (X const&, P1 const&, P2 const&, P3 const&, P4 const&, P5 const&);
public:
   Y operator()(X const& x)const
      { return (*fp)(x,p1,p2,p3,p4,p5);}
   Par5FncObj(
      Y (*g)(X const&,P1 const&,P2 const&,P3 const&,P4 const&,P5 const&),
      P1 const& q1,P2 const& q2,P3 const& q3,P4 const& q4, P5 const& q5):
      p1(q1),p2(q2),p3(q3),p4(q4),p5(q5),fp(g){}
};

template
   <class X, class P1, class P2, class P3, class P4,
   class P5, class P6, class Y>
class Par6FncObj : public FncObj<X,Y>{

   const P1 p1;
   const P2 p2;
   const P3 p3;
   const P4 p4;
   const P5 p5;
   const P6 p6;
   Y (* const fp)(X const&, P1 const&, P2 const&, P3 const&,
      P4 const&, P5 const&, P6 const&);
public:
   Y operator()(X const& x)const
      { return (*fp)(x,p1,p2,p3,p4,p5,p6);}
   Par6FncObj(
      Y (*g)(X const&,P1 const&,P2 const&,P3 const&,P4 const&,
         P5 const&, P6 const&),
         P1 const& q1,P2 const& q2,P3 const& q3,
         P4 const& q4, P5 const& q5, P6 const& q6):
         p1(q1),p2(q2),p3(q3),p4(q4),p5(q5),p6(q6),fp(g){}
};

#endif // CPM_Fn

} // aux

// For F5 and F6, we do not introduce the corresponding F5_1,...F6_6
// which would enable using the parameters as function arguments. Having
// this capability for up to four parameters is sufficient. See cpmfl.h for // // this matter.

#ifdef CPM_Fn
////////////////////// class F5<> //////////////////////////////////////

template <class X, class P1, class P2, class P3, class P4,
   class P5, class Y>
class F5{ // functions with five parameters
    const P1 p1;
    const P2 p2;
    const P3 p3;
    const P4 p4;
    const P5 p5;
    typedef F5<X,P1,P2,P3,P4,P5,Y> Type;
    CPM_INVAR(F5)
public:
    F5( P1 const& p1_,P2 const& p2_,P3 const& p3_,P4 const& p4_,
       P5 const& p5_):
    p1(p1_),p2(p2_),p3(p3_),p4(p4_),p5(p5_){}
    F<X,Y> operator()(Y (*f)(X const&,P1 const&,P2 const&,
       P3 const&,P4 const&,P5 const&))const;
};

template <class X, class P1, class P2, class P3, class P4,
   class P5,  class Y>

F<X,Y> F5<X,P1,P2,P3,P4,P5,Y>::operator()(Y (*f)(X const&,P1 const&,
   P2 const&, P3 const&,P4 const&,P5 const&))const
{
   using namespace aux;
   return
   F<X,Y>(new Par5FncObj<X,P1,P2,P3,P4,P5,Y>(f,p1,p2,p3,p4,p5));
}

////////////////////// class F6<> //////////////////////////////////////

template <class X, class P1, class P2, class P3, class P4,
   class P5, class P6, class Y>
class F6{ // functions with six parameters
    const P1 p1;
    const P2 p2;
    const P3 p3;
    const P4 p4;
    const P5 p5;
    const P6 p6;
    typedef F6<X,P1,P2,P3,P4,P5,P6,Y> Type;
    CPM_INVAR(F6)
public:
    F6( P1 const& p1_,P2 const& p2_,P3 const& p3_,P4 const& p4_,
       P5 const& p5_, P6 const& p6_):
    p1(p1_),p2(p2_),p3(p3_),p4(p4_),p5(p5_),p6(p6_){}
    F<X,Y> operator()(Y (*f)(X const&,P1 const&,P2 const&,
       P3 const&,P4 const&,P5 const&, P6 const&))const;
};

template <class X, class P1, class P2, class P3, class P4,
   class P5, class P6, class Y>

F<X,Y> F6<X,P1,P2,P3,P4,P5,P6,Y>::operator()(Y (*f)(X const&,P1 const&,
   P2 const&, P3 const&,P4 const&,P5 const&, P6 const&))const
{
   using namespace aux;
   return
   F<X,Y>(new Par6FncObj<X,P1,P2,P3,P4,P5,P6,Y>
      (f,p1,p2,p3,p4,p5,p6));
}

#endif // CPM_Fn

namespace aux{

template <class X1, class X2, class Y1, class Y2>
class cart: public FncObj<cpmX2<X1,X2>,cpmX2<Y1,Y2> >{ // CART stands for
    // Cartesian

   const F<X1,Y1> f1;
   const F<X2,Y2> f2;

public:
   cpmX2<Y1,Y2> operator()(const cpmX2<X1,X2>& x)const
   { return cpmX2<Y1,Y2>(f1(x.first),f2(x.second));}

   cart(const F<X1,Y1>& f1_,const F<X2,Y2>& f2_):f1(f1_),f2(f2_){}
};

// pairing of functions which are defined on the same class

//////////////////////// class cart1<> /////////////////////////////////

template <class X, class Y1, class Y2>
class cart1: public FncObj<X,cpmX2<Y1,Y2> >{

   const F<X,Y1> f1;
   const F<X,Y2> f2;
public:
   cpmX2<Y1,Y2> operator()(X const& x)const
   { return cpmX2<Y1,Y2>(f1(x),f2(x));}

   cart1(const F<X,Y1>& f1_,const F<X,Y2>& f2_):f1(f1_),f2(f2_){}
};

} // aux

///////////////////// function pairOf<> ////////////////////////////////

template <class X1, class X2, class Y1, class Y2>
F<cpmX2<X1,X2>,cpmX2<Y1,Y2> > pairOf(const F<X1,Y1>& f1,
   const F<X2,Y2>& f2)
   // Cartesian product of functions:
   // cpmX2(f1,f2)(x1,x2)=(f1(x1),f2(x2)).
   // name changed from pair to pairOf to avoid clash with std name
   // (99-5-31)
{
   using namespace aux;
   return F<cpmX2<X1,X2>,cpmX2<Y1,Y2> >(new cart<X1,X2,Y1,Y2>(f1,f2));
}

//////////////////////// operator&&<> //////////////////////////////////

template <class X, class Y1, class Y2>
F<X,cpmX2<Y1,Y2> > operator &&(const F<X,Y1>& f1,const F<X,Y2>& f2)
   // Cartesian product of functions which are defined on the same class.
    // (f1&&f2)(x)=(f1(x),f2(x)).
    // Elegant way for flexible definition of binary operators by
    // concatenation
    // with a mapping cpmX2<Y1,Y2> ---> Y3
{
   using namespace aux;
   return F<X,cpmX2<Y1,Y2> >(new cart1<X,Y1,Y2>(f1,f2));
}

// 'changing the number of variables from 2 to 1'

// Bind1 and Bind2 as classes of the same design as FuncPar.
// Assume:
// class X1;
// class X2;
// class Y;
// F<cpmX2<X1,X2>,Y> f(...);
// X1 x1=...;
// F<X2,Y> f1=Bind1<X1,X2,Y>(x1)(f);
// X2 x2=...;
// Y ya=f(cpmX2<X1,X2>(x1,x2));
// Y yb=f1(x2);
// then ya==yb

namespace aux{

template <class X1, class X2, class Y>
class bind1: public FncObj<X2,Y>{
   // bind1 stands for bind first variable
   const X1 x1;
   const F<cpmX2<X1,X2>,Y> f;
public:
   Y operator()(const X2& x2)const{ return f( cpmX2<X1,X2>(x1,x2));}
   bind1( const F<cpmX2<X1,X2>,Y> & f_, const X1& x1_):f(f_),x1(x1_){}
};

template <class X1, class X2, class Y>
class bind2: public FncObj<X1,Y>{
   // bind2 stands for bind second variable
   const X2 x2;
   const F< cpmX2<X1,X2>,Y>  f;
public:
   Y operator()(const X1& x1)const{ return f( cpmX2<X1,X2>(x1,x2));}
   bind2( const F< cpmX2<X1,X2>,Y>& f_, const X2& x2_):f(f_),x2(x2_){}
};

} // aux

///////////////////////// class Bind1<> ////////////////////////////////

template <class X1, class X2, class Y>
class Bind1 { // binding the first parameter
    const X1 x1;
public:
    Bind1(const X1& x1_):x1(x1_){}
    F<X2,Y> operator()(const F<cpmX2<X1,X2>,Y>& f)const
    { return F<X2,Y>(new aux::bind1<X1,X2,Y>(f,x1));}
};

///////////////////////// class Bind2<> ////////////////////////////////

template <class X1, class X2, class Y>
class Bind2 { // binding the second parameter
    const X2 x2;
public:
    Bind2(const X2& x2_):x2(x2_){}
    F<X1,Y> operator()(const F<cpmX2<X1,X2>,Y>& f)const
    { return F<X1,Y>(new aux::bind2<X1,X2,Y>(f,x2));}
};

// range and domain conversion

namespace aux{

template <class X1, class X2, class Y>
class convertDomain: public FncObj<X2,Y>{
   // means to transform a F<X,Y> in a F<X',Y> if there is an
   // automatic conversion from X to X'

   const F<X1,Y> f;
public:
   Y operator()(const X2& x)const{ return f(x);}
   convertDomain( const F<X1,Y>& f_):f(f_){}
};

template <class X, class Y1, class Y2>
class convertRange: public FncObj<X,Y2>{
   // means to transform a F<X,Y> in a F<X,Y'> if there is an
   // automatic conversion from Y to Y'

   const F<X,Y1> f;

public:
   Y2 operator()(X const& x)const{ return f(x);}
   convertRange( const F<X,Y1>& f_):f(f_){}
};

} // auxiliatry

/////////////////////// class ConvertDomain<> //////////////////////////

template <class X1, class X2, class Y>
// usage e.g. :
//   F<R,C> f=.....
//   ConvertDomain<R,N,C> converter;
//   F<N,C> f_N=converter(f);
// The same converter object can be used for converting other functions
// of the same domain and range classes.
class ConvertDomain{ // converting the function domain
public:
   ConvertDomain(void){}
   F<X2,Y> operator()(const F<X1,Y>& f)const
      // transforms a F<X1,Y> in a F<X2,Y> if there is an
      // automatic conversion from X1 to X2.
   {
      return F<X2,Y>(new aux::convertDomain<X1,X2,Y>(f));
   }
};

//////////////////////// class ConvertRange ////////////////////////////

template <class X, class Y1, class Y2>
// Usage:
//   F<R,C> f=.....
//   ConvertRange<R,C,Cd> converter;
//   F<R,Cd> f_Cd=converter(f);
// The same converter object can be used for converting other functions
// of the same domain and range classes.
class ConvertRange{ // converting the function range
public:
   ConvertRange(void){}
   F<X,Y2> operator()(const F<X,Y1>& f)const
      // transforms a F<X,Y1> in a F<X,Y2> if there is an
      // automatic conversion from Y1 to Y2.
   {
      return F<X,Y2>(new aux::convertRange<X,Y1,Y2>(f));
   }
};

/// functions of more than one variable

namespace aux{

template <class Y1, class Y2, class Y3>
class Arg2FncObj : public FncObj<cpmX2<Y1,Y2>, Y3>{

   const Y3 (*fp)(const Y1&, const Y2& );

public:
   Y3 operator()(const cpmX2<Y1,Y2>& x)const
   { return fp(x.c1(),x.c2());}

   Arg2FncObj( Y3 (*g)(const Y1&, const Y2&)):fp(g){}
};

} // aux

#ifdef CPM_Fn
//////////////////// class F_2<> ///////////////////////////////////////

template <class Y1, class Y2, class Y3>
class F_2{ // functions of two variables

   const F<cpmX2<Y1,Y2>,Y3> f;

public:

   F_2(Y3 (*f_)(const Y1&, const Y2&)):
      f(new aux::Arg2FncObj<Y1,Y2,Y3>(f_)){}
      // construction from function pointer

   F_2(const F<CpmArrays::X2<Y1,Y2>,Y3>& f_):f(f_){}
      // construction from 'smart function object'

   F<CpmArrays::X2<Y1,Y2>,Y3> operator()(void)const{ return f;}
      // evaluation to 'smart function object'

   Y3 operator()(const Y1& y1, const Y2& y2)const
      // evaluation to value
   { return f(CpmArrays::X2<Y1,Y2>(y1,y2));}

   F<Y2,Y3> c1(const Y1& y1)const
      // evaluation to a 'smart function object'
      // representing a function of one variable;
      // obtained by binding the first of the two variables
   { return Bind1<Y1,Y2,Y3>(y1)(f);}

    F<Y1,Y3> c2(const Y2& y2)const
     // ...  binding the second of the two variables
   { return Bind2<Y1,Y2,Y3>(y2)(f);}

};

#endif // CPM_Fn

// define a function by selecting one function value from two functions

namespace aux{

template <class X, class Y>
class sel: public FncObj<X,Y>{

   const F<X,Y>  f1;
   const F<X,Y>  f2;
   const F<X,Z>  s;

public:

   sel(const F<X,Y> & f1_, F<X,Y>  const& f2_, const F<X,Z>& s_):
      f1(f1_),f2(f2_),s(s_){}

   Y operator()(X const& )const;
};

template <class X, class Y>
Y sel<X,Y>::operator()(X const& x)const
{
   Z i=s(x);
   if (i==1) return f1(x);
   else if (i==2) return f2(x);
   else {
      cpmerror
         ("select::operator(): unvalid value of selection function");
      return f1(x); // never done
   }
}

//////////////////// class vselect<> ///////////////////////////////////

// define a function by selecting one function value from a sequence of
// functions

template <class X, class Y>
class vselect: public FncObj<X,Y>{

   const V< F<X,Y> > f;
   const F<X,Z>  sel;

public:

   vselect(const V< F<X,Y> >& f_, const F<X,Z> & sel_):
      f(f_),sel(sel_){}

   Y operator()(X const& )const;

};

template <class X, class Y>
Y vselect<X,Y>::operator()(X const& x)const
{
   Z i=sel(x);
   return (f[i])(x);
}

} // aux

/////////////////// function select<> //////////////////////////////////

template <class X, class Y>
F<X,Y> select(F<X,Y>  const& f, F<X,Y>  const& g, const F<X,Z>& h)
// The returned function res has the property res(x)==f(x) if h(x)==1,
// res(x)== g(x) if h(x)==2, cpmerror() else
{
   return F<X,Y>(new aux::sel<X,Y>(f,g,h));
}

template <class X, class Y>
F<X,Y> select(const V<F<X,Y> >& f, const F<X,Z>& h)
// The returned function res has the property res(x)=(f[g(x)])(x);
{
   return F<X,Y>(new aux::vselect<X,Y>(f,h));
}

} // namespace

#endif
