//? cpmsystemdependencieswrc.h
//? C+- by Ulrich Mutze. Status of work 2011-03-30.
//? Copyright (c) 2011 Ulrich Mutze, www.ulrichmutze.de
//? All rights reserved

#ifndef CPM_SYSTEMDEPENDENCIES_H_
#define CPM_SYSTEMDEPENDENCIES_H_
/*************************************************************************
   cpmsystemdependencies.h
   Description: By this file one can place defines in implementation
    files in cases that the proper implementation is dependent on the
    computer system, e.g. on the access to a display or a file system.
    These defines may also modify the way how we use such facilities;
    e.g. using or not using the graphical display. Or, creating or
    not creating log files. Especially such changes of usage should
    not entail too much recompilation. So the present file should
    never be included in header files. Presently it is only included
    in six files: cpmnumbers.cpp, cpmsystem.cpp, cpmapplication.cpp
    cpmgraph.cpp, cpmimg24.cpp, cpminifilebasapp.cpp.

    Notice that changes in file cpmdefinitions.h trigger recompilation
    of virtually all C+- translation units.

    See also cpmconfig.ini, where most data can be overwritten.

************************************************************************/
// See file cpmdefinitionswrc.h for the meaning of the 'wrc'-appendix.

#define CPM_REL_POS1 "."
#define CPM_REL_POS2 "./control"
#define CPM_REL_POS3 ".."
#define CPM_REL_POS4 "../control"
   // Four positions relative to the executables or the IDE's
   // idea of working directory. This is needed only for
   // finding cpmconfig.ini, on which the locations for further
   // input and for font files can be given.
   // One may use absolute pathes here if relative paths don't work as
   // expected

#define CPM_INP_DIR "./control"
   // Default location of the directory for input.
   // Typically not needed since the input directory can
   // comfortably set in cpmconfig.ini. This allows to prepare several
   // control data sets e.g. ./controlTest1, ./controlTest2,
   // ./controlFinal and switch between them by a suitable entry in
   // cpmconfig.ini

#define CPM_OUT_DIR "."
   // Directory in which the log-files cpmcerr.txt and cpmdata.txt
   // will be generated. By default it is also the directory to which
   // all other output data of the application get written.
   // Typically by adjusting the output directory in cpmconfig.ini one
   // will flexibly select places such as ./runTest1, ./runTest2,
   // ./runFinal for the example of the previous paragraph.

#define CPM_FONT_DIR "../fonts"
   // Location of the fonts file.

#define CPM_HEIGHT 832
#define CPM_WIDTH  1432
   // Screen data in pixels.

#define CPM_PANE1 350
#define CPM_PANE2 250
#define CPM_PANE3 250
#define CPM_PANE4 100
   // Relative size (arbitrary units) of the status bar subfields
   // ('panes').

#define CPM_STEREOLEFT_R 255
#define CPM_STEREOLEFT_G 0
#define CPM_STEREOLEFT_B 0

#define CPM_STEREORIGHT_R 0
#define CPM_STEREORIGHT_G 255
#define CPM_STEREORIGHT_B 255
   // Optional input of color values for generation of anaglyph images.
   // If no input from here, code in cpmimagingtools.cpp comes in
   // action. If there is input from cpmconfig.ini this input overrides
   // all previously fixed values. See cpmconfigwrc.ini

#define CPM_RUN_ID_LENGTH 4
   // Number of digits of the run ID.
   // Notice that the run ID is generated at the point in time of
   // program start and is unrelated to the runName_ data member of class
   // CpmApplication::IniFileBasedApplication, which can be read at
   // run time from its ini-file. Only if runName_ is asked to be
   // 'auto', the run ID will be used to replace runName_.
   // CpmTime::Greg::getCode(Z) is the function which determines the
   // run ID.

#define CPM_WRITE_PRECISION 8
   // Numerical precision in writing floating point data to file.

#define CPM_WRITE_TITLE
   // Writing class instances to file will contain the
   // class name as 'title'.

//#define CPM_CERR_EXTEND
   // If this is defined, the run ID will be appended
   // to the name of the log files cpmcerr and cpmdata.
   // This is recommended for important program runs since
   // cpmcerr conserves all data which characterize a program
   // run. However, in a mere experimental phase of program
   // development it could be useful not to let each program
   // run create log files with new names with the consequence
   // of overloading the working directory. In such cases one
   // not defines this macro. Even for serious runs this is
   // a viable option since one may then, after the run,
   // rename cpmcerr.txt by appending then a more indicative
   // run name (preferable the one which was appended to the
   // other documentation files). Since 2011-03-30 there is
   // the possibility to set CpmApplication::doc_=true which then
   // makes a copy of cpmcerr with the run ID appended to the name.
   // The new function CpmSystem::Message::augRunId(Word const&)
   // allows to augment the numerical run ID by 'speaking' text.
   // 2014-02-13: It is now recommended to let #define CPM_CERR_EXTEND
   // commentarized and use cpmconfig.ini for requesting the name
   // mangeling of output files. Precisely set in section 'append run id'
   //    B val=true.
   // Then by a new logic all outputfiles created by a program run will
   // aquire the same name appendix which contains a auto-generated
   // digit sequence (the length of which is CPM_RUN_ID_LENGTH) and
   // IniFileBasedApplication::runName_ (if cpminifilebasapp.cpp is among
   // the source files of our project).

//#define CPM_NOLOG
   // If defined, no cpmcerr and cpmdata will be created.
   // This may be an option if either these log-files would
   // be very large, or the program has been very well tested
   // and understood. Normally having cpmcerr.txt generated is
   // very useful.

//#define CPM_NOGRAPHICS
   // If defined, no graphical content will be shown on screen
   // and no graphics libraries (openGL and GLUT) need to be linked.
   // Nevertheless ppm-images (of arbitrary size) may be created.
   // To enable meaningful program runs without relying on graphics
   // libraries turned out to be very useful to make sure that a final
   // 'segmentation fault' that after some Ubuntu-update occurred regularly
   // came from the graphics libraries, presumably from freeglut.

 #define CPM_USE_EIGEN
   // using the Eigen library for fast matrix and vector algorithms
   // for real and for complex quantities. The algorithms for
   // eigenvalues/eigenvectors and SVD do not cooperate with multiple
   // precision unless compilation is done under the option
   // -fpermissive. Using Eigen requires no linking with a lib. Only
   // include files are required.

#if defined(CPM_USE_EIGEN)
   #include <Eigen/Core>
   #include <Eigen/Dense>
#endif

#if !defined(CPM_NOGRAPHICS)
	// otherwise we don't need graphics libraries
  //  'Cygwin' or 'Linux'
	#include <GL/glut.h>
#endif

#endif

/**************************************************************************
2020-04-18 Compilation tests for program quantumscatteringL:

C+- macros:
CPM_USE_BOOST     x     x     x     x     x
CPM_MP            0     x     0     x     x
CPM_USE_EIGEN     0     0     x     x     x
-------------------------------------------

compiler options:
-fpermissive      0     0     0     0     x
-pedantic         x     x     x     x     0
-------------------------------------------

output:
errors            0     0     0     50    0
warnings          0     0     0     0     156

Quintessence: If one needs fast linear algebra from Eigen together with
consistent multiprecision arithmetic from boost one can get it but only
when compiling under -fpermissive. I hope that for future versions of
boost, eigen, and gcc everything will work together under -pedantic.
**************************************************************************/


