//? cpmv.cpp
//? C+- by Ulrich Mutze. Status of work 2022-05-06.
//? Copyright (c) 2022 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//?
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//?
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for
//? more details.


#include <cpmv.h>

using CpmRoot::Z;
using CpmRoot::Word;
using CpmArrays::V;
using CpmArrays::IvZ;

Z CpmArrays::dimMax=100000000;
Z CpmArrays::firInd=1; // first index of arrays. Means to let C+-
   // cooperate with std containers.
bool CpmArrays::ranChc=true;
bool CpmArrays::ranChcAlw=false;
bool CpmArrays::signal=false;

   // if this is true some screen messages are enabled
   // e.g.
   // if (signal) cout<<"lazy ~V called, p_ = "<<p_<<endl;

void CpmArrays::setDimMax(Z n)
{
   if (n<0)
      cpmerror(cpmwrite(n)&" not OK as value for CpmArrays::dimMax");
   dimMax=n;
}

Z CpmArrays::makeIndValNotMember(Z i, IvZ iv, Outside mode)
{
   if (iv.hasElm(i)) return i;
   if (mode==CYCLIC) return iv.cyc(i);
   if (mode==CONSTANT) return iv.con(i);
   if (mode==DEFAULT){ cpmwarning("value DEFAULT of Outside not allowed here");return -137;}
   if (mode==ZERO){ cpmwarning("value ZERO of Outside not allowed here"); return -137;}
   else { cpmwarning("undefined value of Outside"); return -137;}
}
      //: make index valid

Z CpmArrays::safeDim(Z n)
{
   if (n<0 || n>dimMax)
      cpmerror(cpmwrite(n)&" not OK as dimension of CpmArrays::V<>");
   return n;
}

V<Z> CpmArrays::IvZtoVofZ(IvZ const& iv)
{
   Z n=iv.car();
   V<Z> res(n);
   if (n==0) return res;
   Z i,val=iv.inf();
   for (i=res.b();i<=res.e();++i) res[i]=val++;
   return res;
}

V<Z> CpmArrays::VofIvZtoVofZ(V<IvZ> const& viv)
{
   V<Z> res(0);
   for (Z i=viv.b();i<=viv.e();++i) res&=IvZtoVofZ(viv[i]);
   return res;
}

V<Word> CpmArrays::comLine(int argc, char* argv[])
// see BS4, p.254 function
//    vector<string> arguments(int argc, char* argv[])
// for the corresponding construct within namespace std.
{
   V<Word> res(argc);
   for (Z i=res.b();i<=res.e();i++) res[i]=argv[i-1];
   return res;
}
