//? cpmapplication.cpp
//? C+- by Ulrich Mutze. Status of work 2022-05-06.
//? Copyright (c) 2022 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//?
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//?
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for
//? more details.


#include <cpmapplication.h>
#include <cpmrecordhandler.h>
#include <cpmgreg.h>
#include <cpmsystemdependencies.h>
#include <cpmimagingtools.h>
#include <cstdlib>
#include <filesystem> // needs C++20

using CpmRoot::Word;
using CpmRoot::Z;
using CpmRoot::R;
using CpmRootX::RecordHandler;
using CpmRootX::extComLine;
using CpmRootX::sup;
using CpmArrays::V;
using CpmSystem::Message;
using CpmSystem::updFromConfigFile;

using namespace std;

void CpmSystem::updFromConfigFile(void)
// When this gets called, the file cpmcerr is yet working
{
   Z mL=1;
   Word loc("CpmSystem::updFromConfigFile()");
   CPM_MA
   Word fn0("cpmconfig.ini");
   Word fn1=CpmSystem::glue(Message::relPos1,fn0);
   Word fn2=CpmSystem::glue(Message::relPos2,fn0);
   Word fn3=CpmSystem::glue(Message::relPos3,fn0);
   Word fn4=CpmSystem::glue(Message::relPos4,fn0);
   cpmmessage(mL,"ahead of V<Word constructor");
   V<Word> vc{fn1,fn2,fn3,fn4};
   cpmcerr<<"vc="<<vc<<endl;
   RecordHandler rch(vc,"","call in updFromConfigFile()");
   if (!rch.isVal()){
      cpmmessage("file cpmconfig.ini not found");
      CPM_MZ
      return;
   }
   else cpmmessage("file cpmconfig.ini found");
   Z ped=1; // degree of pedantary
      // all data are initialized anyway, so 1 is OK
   bool suc=false; // success

   Word sec="input directory";
   suc=rch.read(sec,"dir",Message::inpDir,ped);
   if (suc) cpmmessage("input directory got from cpmconfig.ini as "
              &Message::inpDir);

   sec="output directory";
   suc=rch.read(sec,"dir",Message::outDir2,ped);
   if (suc) cpmmessage("output directory got from cpmconfig.ini as "
              &Message::outDir2);

   std::filesystem::path outPath(Message::outDir2.str());
   bool b=std::filesystem::exists(outPath);
   if (b){
      cpmmessage("output directory "&Message::outDir2&" already exists");
   }
   else{
      cpmmessage("output directory "&Message::outDir2&" does not yet\
       exist");
       bool b1=std::filesystem::create_directory(outPath);
       if (b1) cpmmessage("output directory "&Message::outDir2&" created");
       cpmassert(b1,loc);
       cpmassert(std::filesystem::exists(outPath),loc);
   }

   sec="font directory";
   rch.read(sec,"dir",Message::fontDir,ped);
   cpmmessage("font directory got from cpmconfig.ini as "
              &Message::fontDir);

   sec="size of viewport";
   rch.read(sec,"width",Message::vpw_,ped);
   rch.read(sec,"height",Message::vph_,ped);

   sec="size of frame store";
   rch.read(sec,"width",Message::fsw_,ped);
   rch.read(sec,"height",Message::fsh_,ped);

   sec="double buffering";
   rch.read(sec,"db",CpmGraphics::Rec::db,ped);

   sec="pane sizes";
   rch.read(sec,"pane1",Message::sttBarWid1,ped);
   rch.read(sec,"pane2",Message::sttBarWid2,ped);
   rch.read(sec,"pane3",Message::sttBarWid3,ped);
   rch.read(sec,"pane4",Message::sttBarWid4,ped);

   sec="write precision";
   rch.read(sec,"val",CpmRoot::wrtPrc,ped);

   #if defined(CPM_MP)
         cpmmessage(loc&": numerical precision is "&cpm(Z(CPM_MP)));
    #else // no multiple precision
      #if defined(CPM_FLOAT)
         cpmmessage("numerical precision is that of type float");
      #elif defined(CPM_LONG)
         cpmmessage("numerical precision is that of type long double");
      #elif defined(CPM_QUAD)
         cpmmessage("numerical precision is that of type quad");
      #else
         cpmmessage("numerical precision is that of type double");
      #endif
   #endif

   sec="write title";
   rch.read(sec,"val",CpmRoot::wrtTit,ped);

   sec="append run id";
   rch.read(sec,"val",Message::appRunId,ped);
//   CpmApplication::doc_= Message::appRunId;
      // If doc_ is true, a copy of cpmcerr.txt and cpmdata.txt will be
      // made at the end of program execution with runId appended.

   sec="run id length ";
   rch.read(sec,"val",Message::runIdLength,ped);
   CpmTime::Greg date;
   date.now();
   Message::runId=date.getCode().tail(Message::runIdLength);
      // re-setting runId with proper length. Since the old Id needs to be
      // changed anyway it is not useful to take the old one and cut or
      // extend it

#if !defined(CPM_NOLOG)
   sec="suppress messages";
   rch.read(sec,"val",Message::silent,ped);
#endif

   sec="stereo colors";
   V<Z> left, right;
   rch.read(sec,"left",left,ped);
   rch.read(sec,"right",right,ped);
   if (left.dim()==3){
      CpmImaging::stereoLeftR=left[1];
      CpmImaging::stereoLeftG=left[2];
      CpmImaging::stereoLeftB=left[3];
   }
   if (right.dim()==3){
      CpmImaging::stereoRightR=right[1];
      CpmImaging::stereoRightG=right[2];
      CpmImaging::stereoRightB=right[3];
   }
   CPM_MZ
}

Word CpmApplication::title_("defaulttitle");
V<Word> CpmApplication::args_=V<Word>(0);

#include <cpmframe.h>

using CpmGraphics::Frame;

using CpmApplication::main_;
using CpmApplication::title_;
using CpmApplication::args_;

namespace{
// variables common to functions main and idleFunc
   Word loc_;
   Word argv_;
}

namespace CpmApplication{
   void inspectCpmdefinitions()
   {
      Z mL=1;
      Word loc("CpmApplication::inspectCpmdefinitions()");
      CPM_MA
   //   cpmmessage(mL,"constructor V<Word> ahead");
      V<Word> definitions;
//      cpmmessage(mL,"constructor V<Word> done");
//      cpmcerr<<"we got definitions = "<<endl;
//      definitions.prnOn(cpmcerr);
#if defined(CPM_NAMEOF)
      definitions&="CPM_NAMEOF";
#endif
#if defined(CPM_FLOAT)
      definitions&="CPM_FLOAT";
#endif
#if defined(CPM_DOUBLE)
      definitions&="CPM_DOUBLE";
#endif
#if defined(CPM_LONG)
      definitions&="CPM_LONG";
#endif
#if defined(CPM_QUAD)
      definitions&="CPM_QUAD";
#endif
#if defined(CPM_USECOUNT)
      definitions&="CPM_USECOUNT";
#endif
#if defined(CPM_RANGE_CHECK)
      definitions&="CPM_RANGE_CHECK";
#endif
#if defined(CPM_MP)
      Z val=CPM_MP;
      Word wmp="CPM_MP = "&cpm(val);
      definitions&=wmp;
#endif
#if defined(CPM_Fn)
      definitions&="CPM_Fn";
#endif
   cpmcerr<<" list of macros from cpmdefinitions.h "<<endl;
   cpmcerr<<"size of definitions = "<<definitions.size()<<endl;
   cpmcerr<<definitions<<endl;
   CPM_MZ
   }

   void inspectCpmsystemdependencies()
   {
      V<Word> dependencies;
      Word val;
      Z i;
#if defined(CPM_REL_POS1)
      val=CPM_REL_POS1;
      dependencies&=("CPM_REL_POS1 = "&val);
#endif
#if defined(CPM_REL_POS2)
      val=CPM_REL_POS2;
      dependencies&=("CPM_REL_POS2 = "&val);
#endif
#if defined(CPM_REL_POS3)
      val=CPM_REL_POS3;
      dependencies&=("CPM_REL_POS3 = "&val);
#endif
#if defined(CPM_REL_POS4)
      val=CPM_REL_POS4;
      dependencies&=("CPM_REL_POS4 = "&val);
#endif
#if defined(CPM_INP_DIR)
      val=CPM_INP_DIR;
      dependencies&=("CPM_INP_DIR = "&val);
#endif
#if defined(CPM_OUT_DIR)
      val=CPM_OUT_DIR;
      dependencies&=("CPM_OUT_DIR = "&val);
#endif
#if defined(CPM_FONT_DIR)
      val=CPM_FONT_DIR;
      dependencies&=("CPM_FONT_DIR = "&val);
#endif
#if defined(CPM_HEIGHT)
      i=CPM_HEIGHT;
      dependencies&=("CPM_HEIGHT = "&cpm(i));
#endif
#if defined(CPM_WIDTH)
      i=CPM_WIDTH;
      dependencies&=("CPM_WIDTH = "&cpm(i));
#endif
#if defined(CPM_PANE1)
      i=CPM_PANE1;
      dependencies&=("CPM_PANE1 = "&cpm(i));
#endif
#if defined(CPM_PANE2)
      i=CPM_PANE2;
      dependencies&=("CPM_PANE2 = "&cpm(i));
#endif
#if defined(CPM_PANE3)
      i=CPM_PANE3;
      dependencies&=("CPM_PANE3 = "&cpm(i));
#endif
#if defined(CPM_PANE4)
      i=CPM_PANE4;
      dependencies&=("CPM_PANE4 = "&cpm(i));
#endif

#if defined(CPM_STEREOLEFT_R)
      i=CPM_STEREOLEFT_R;
      dependencies&=("CPM_STEREOLEFT_R = "&cpm(i));
#endif
#if defined(CPM_STEREOLEFT_G)
      i=CPM_STEREOLEFT_G;
      dependencies&=("CPM_STEREOLEFT_G = "&cpm(i));
#endif
#if defined(CPM_STEREOLEFT_B)
      i=CPM_STEREOLEFT_B;
      dependencies&=("CPM_STEREOLEFT_B = "&cpm(i));
#endif
#if defined(CPM_STEREORIGHT_R)
      i=CPM_STEREORIGHT_R;
      dependencies&=("CPM_STEREORIGHT_R = "&cpm(i));
#endif
#if defined(CPM_STEREORIGHT_G)
      i=CPM_STEREORIGHT_G;
      dependencies&=("CPM_STEREORIGHT_G = "&cpm(i));
#endif
#if defined(CPM_STEREORIGHT_B)
      i=CPM_STEREORIGHT_B;
      dependencies&=("CPM_STEREORIGHT_B = "&cpm(i));
#endif

#if defined(CPM_RUN_ID_LENGTH)
      i=CPM_RUN_ID_LENGTH;
      dependencies&=("CPM_RUN_ID_LENGTH = "&cpm(i));
#endif
#if defined(CPM_WRITE_PRECISION)
      i=CPM_WRITE_PRECISION;
      dependencies&=("CPM_WRITE_PRECISION = "&cpm(i));
#endif
#if defined(CPM_WRITE_TITLE)
      dependencies&="CPM_WRITE_TITLE";
#endif
#if defined(CPM_NOLOG)
      dependencies&="CPM_NOLOG";
#endif
#if defined(CPM_NOGRAPHICS)
      dependencies&="CPM_NOGRAPHICS";
#endif
#if defined(CPM_USE_EIGEN)
      dependencies&="CPM_USE_EIGEN";
#endif
#if defined(CPM_DIMDEF2)
      dependencies&="CPM_DIMDEF2";
#endif
#if defined(CPM_DIMDEF3)
      dependencies&="CPM_DIMDEF3";
#endif
#if defined(CPM_POINT_PARTICLES)
      dependencies&="CPM_POINT_PARTICLES";
#endif
#if defined(CPM_M)
      dependencies&="CPM_M";
#endif
   cpmcerr<<" list of macros from cpmsystemdependencies.h "<<endl;
   cpmcerr<<dependencies<<endl;
   }
} // end of namespace CpmApplication

///////////////// int main(int argc, char *argv[]) ///////////////////////

#if !defined(CPM_NOGRAPHICS)
// main for application  which uses OpenGL and glu, glut (or OpenGlut)
// for output to screen

void displayFunc(void)
// everything is done by direct graphical commands
{ glutSwapBuffers(); } // needed for cooperations with panes
/* (from OpenGL Programming Guide, 2nd edition, page 601)
 If exact two-dimensional raterization is desired, you must
 carefully specify both the orthographic projection and the vertices
 of primitives that are to be rasterized.  The orthographic projection
 should be specified with integer coordinates, as shown in the
 following example:
      gluOrtho2D(0, width, 0, height);
 where width and height are the dimensions of the viewport.  Given
 this projection matrix, polygon vertices and pixel image positions
 should be placed at integer coordinates to rasterize predictably.
 For example, glRecti(0,0,1,1) reliably fills the lower left pixel of
 the viewport, and glRasterPos2i(0,0) reliably positions an unzoomed
 image at the lower left of the viewport.  Point vertices, line
 vertices, and bitmap positions should be placed at half-integer
 locations, however.  For example, a line drawn from (x1,0.5) to
 (x2,0.5) will be reliably rendered along the bottom row of pixels
 into the viewport, and a point drawn at (0.5,0.5) will reliably fill
 the same pixel as glRecti(0,0,1,1).

 An optimum compromise that allows all primitives to be specified at
 integer positions, while still ensuring predictable rasterization, is
 to translate x and y by 0.375, as shown in the following code
 fragment [reshapeCB()].  Such a translation keeps polygon and pixel
 image edges safely away from the centers of pixels, while moving line
 vertices close enough to the pixel centers.
*/

void glToglut(int w, int h)
// sets the glViewport and coordinate transformations of gl in a way that
// the concept of position in the image plane coincides with glut's
// definitions. The code and comments are from an code example 'canvas.c'
// that Thomas Dera found somewhere.
// Seems to be needed only in function reshape which will be only invoked
// at program begin.
{
  glViewport(0,0,w,h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(0,w,0,h);
  glScalef(1.0,-1.0,1.0);   // this flips the coordinate system so
         // that y increases going down the
         // screen (see below)
  glTranslatef(0.0,-h,0.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
   // glTranslatef(0.375,0.375,0.0); so we can draw using integer
   // coordinates (see above)  has no influence on my graphics functions

   glPixelStorei(GL_UNPACK_ALIGNMENT,1);
   // in order to intoduce no alignement errors depending on pixel
   // matrices having sizes that are a multiple of 4 or 8.;
   // value 4 (instead of our 1) is default

}

/*
 Note the flip of the y coordinate.  GLUT thinks the upper left hand
 corner of the screen or window is (0,0).  OpenGL, by default, thinks
 the lower left corner is (0,0).  This presents a problem when, for
 example, you compare the coordinates of a mouse event (from GLUT) with
 the coordinates of an OpenGL object.  The y coordinate flip makes OpenGL
 coordinates correspond to GLUT coordinates.
*/

void reshapeFunc(int w, int h)   // generic 2D reshape callback
{
   Z wi=CpmSystem::Message::w();
   Z hi=CpmSystem::Message::h();
   glToglut((int)wi, (int)hi);
  // glutSwapBuffers(); panes remain partially inactive after resizing the window
  // with and without this statement
}

void idleFunc( void ) // calls main_ in a suitable context
// The function Z CpmApplication::main_(void) has to be provided by the
// user. In its definition, the following data are available:
// CpmGraphics::Viewport and thus CpmGraphics::Frame and
// CpmGraphics::Graph are initialized to the size of the graphics window
// the size of which is set in cpmconfig.ini.
// Together with the graphics classes the accompanying fonts are
// available. A name of the program is available as
// CpmApplication::args_[1]. More components of args_ are available only
// if provided by a calling command line or by reading a ini-file.
// When calling the present function the task of completing the
// command-line input by data from an ini-file is already done.
{
   cpmmessage("idleFunc() started");
   using CpmApplication::args_;
   Z nx=Message::w(); //instead of glutGet(GLUT_WINDOW_WIDTH);
   Z ny=Message::h(); //instead ofglutGet(GLUT_WINDOW_HEIGHT);
   // GLUT_WINDOW_WIDTH, and GLUT_WINDOW_HEIGHT don't always have the
   // previously set values, strange
 //  Z rank=1;
 //  Z size=1;
   Word title=args_[1];
   title&=", CpmGraphics::Viewport initialized";
   CpmGraphics::Viewport vp(nx,ny,title);
      // initializes the viewport nx and ny as read from
      // cpmconfig.ini. No restriction to actual screensize
   cpmwait(1);
   Z vi=CpmGraphics::Viewport::isInitialized();
   if (vi==0) cpmerror(loc_&"not able to initialize Viewport");
   nx=CpmGraphics::Viewport::pelX();
   ny=CpmGraphics::Viewport::pelY();
   Word mes=loc_&" initialized Viewport with nx="&cpm(nx)&
      ", ny="&cpm(ny);
      // nx and ny may be much larger than the computers screen and
      // graphical hardware supports
   Frame fr;
   fr.clr_();
   cpmmessage(mes,-1);
   R tStart=cpmtime();
   Z res=main_(); // this is the main thing
   cout<<"main_() done, res = "<<res<<endl;
   R tTot=cpmtime();
   tTot-=tStart; //now the name applies
   cpmmessage(loc_&"main_() done; res was "&cpm(res),-1);
   cpmurgent("Main computation done in "&cpm(tTot)&" seconds");
   std::ostringstream ost;
   ost<<"idleFunc() nearly done, return value of main_() is "
         <<res<<endl;
   ost<<"Function main() done for program "<<argv_;
   ost<<"Computation time was "<<tTot<<" seconds"<<endl;
   cpmmessage(ost,-1);
// negative status bar segment writes to
//       console if there is one, even if graphics is active
//       then an individualized copy of cpmcerr will be made.
//       individualization is by appending cpmrunid. In phases of
//       experimentation it might be inconvenient to create a copy of
//       cpmcerr. This is achieved by (append run id)::val=0 in
//       cpmconfig.ini
   R someTimeWait=0.5;
   // notice that a wait time for inspecting the final graphics can be
   // set with the *.ini-file of the main application so that there is no
   // need to have a long time here
   cpmwait(someTimeWait);
   Word mesf="idlefunc(): exit() ahead, last writing to cpmcerr.txt, \n";
   mesf&="still writing on cout";
   cpmmessage(mesf,-1);
   CpmSystem::Message::fini();
// Now it should be safe to copy cpmcerr.txt and cpmdata.txt.
// Earlier, I opened file cpmcerr.txt although it was in use.
// This resulted in unreadable files and in core dumps. In my mind I
// blaimed the CPM_USE_BOOST functionality for that.
// To see nevertheless what goes on in the following I inserted
// cout statements.
   CpmSystem::docRun(true);
  // CpmSystem::docRun(false);
   cout<<"idlefunc(): exit() ahead, res = "<<res<<endl;
   cpmwait(2.,2);
   // was 12. instead of 2., such a long pause was very enoying in project
   // smallExperiments.
  // #if __cplusplus > 199711L // hint from the net
  //    quick_exit(res);
  // #else
      exit(res);
  // #endif
      // c++11 has the function quick_exit. It avoids the
      // 'Segmentation fault (core dumped)' event in the rare cases where
      // exit(res) causes such an event to happen. When running my
      // program smallExperiments within Ruby
      //  (e.g. ruby uni_proj_to_website2016a.rb with suitable entries
      // in the file uni_proj_to_website2016a.rb) the ruby function exec
      // may show a file status false after quick-exit() --- a behavior
      // that never was observed under exit().
}

int main(int argc, char *argv[])
// Notice the C-native argument types int and char* which are chosen in
// order not to irritate some compilers.
// This is the main function for a console application. Under Windows it
// should also be startable by (double)clicking on the exe-file.
// Then nothing but the application's name gets transmitted to the present
// function, meaning that only args_[1] is then defined.
// When starting the program from the commandline, e.g. from the cygwin
// bash, e.g.
//    > ./myexcel2.exe 2
// one can provide it with further parameters for which the present
// function, however, does nothing more than storing them into the
// variabe CpmApplications::args_ of type V<Word>. They are then in place
// for the application-specific function main_ to make use of them.
// Notice that V<Word> is a univeral data type and that more convenient
// data formats comprising a type and an identifier are too complex to be
// convenient as command line input. They also have to follow specific
// rules to be readable and it is too easy to violate such rules while
// typing to the command line.

// In the present function there is a mechanism built in, which asks for
// information in a ini-file to be used to extend the command line
// content. This ini-file is expected to have the name of the executable
// with the file extension replaced by ".ini". (For this to work the
// executable needs not really have a file extension.) Also, if no
// ini-file can be found at any of the expected positions, the command
// line simply stays unchanged.
// If for some component of args_ there is input both from an ini-file
// and the command line, the latter alway is used.
// The 'expected positions' for the ini-file are explained in the
// comment to function CpmSystem::Message::ext(Word,Word) in cpmsystem.h.
// Notice that the expected positions for cpmconfig.ini are set by
// static data in C+- code and that the expected positions for ini-files
// depend on the base directory CpmSystem::Message::inpDir read from file
// cpmconfig.ini.

// Addionally, there is a mechanism built in to obtain data for the
// size of a graphical application window and an internal frame store
// (which by default has the same size as the application window).
// These data are also expected from a file cpmconfig.ini.

// It is the responsibility of function main_() to make contact to
// additonal files and ask for more data. See also idleFunc().
// Notice that messaging goes entirely through cpmmessage so that
// cout gets involved as defined in CpmSystem::Message.
{
   using CpmApplication::args_;
   using CpmGraphics::Viewport;
   using CpmGraphics::Frame;
   using CpmSystem::Message;
   Word mes("C+- says Hello World in function ");
   mes&="int main(int argc, char *argv[])";
   cpmmessage(mes);
      // initializes the message streams cpmcerr and cpmdata
   std::ostringstream ost;
   ost<<endl<<"Function main() started for program "<<argv[0]<<endl;
   cpmmessage(ost);
   CpmApplication::inspectCpmdefinitions();
   CpmApplication::inspectCpmsystemdependencies();
   CpmSystem::updFromConfigFile(); // now cpmconfig.ini has been read
      // Message::(vpw_,vph_,fsw_,fsh_) vp: viewport, fs: frame store
      // are set. Values from cpmsystemdependencies.h overwritten
   args_=extComLine(argc,argv); // extended command line
   // see explanation of this function in cpmrecordhandler.h
   std::ostringstream ost2;
   ost2<<"Extended command line args_ transformed to: "<<args_;
   cpmmessage(ost2);
   argv_=args_[1];
   loc_=Word( "main(" );
   loc_=loc_&cpm(Z(argc))&", "&args_[1]&"): ";
   Frame::iniFonts(); // at this position font reading is 80 times faster
   // than when done as a part of Frame::setStaticSize(...);
   glutInit( &argc, argv );
   cout<<"glutInit("<<loc_<<") done"<<endl;
   if (CpmGraphics::Rec::db)
      glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE| GLUT_DEPTH );
   else
      glutInitDisplayMode( GLUT_RGB | GLUT_SINGLE| GLUT_DEPTH );
      // Since 2020-05-02 working double buffered is enabled.
      // An optional 'glutSwapBuffers()' as the last code line in
      // CpmGraphics::Rec.vis() (see cpmviewport.h) does the trick.
   glutInitWindowSize(200,1); // new 2013-10-17 make window at this stage
      // hardly visible since it is chaotic and dissected. Also this window
      // should not hide the console window.
   glutInitWindowPosition(0,0); // new 2013-04-02
      // hope that this reduces even more the probability with which the
      // system fails to create the window as requested by
      // glutReshapeWindow(width,height);
   Word gcw="Non-interactive OPENGLUT-window for "&argv_;
   glutCreateWindow(-gcw);
#if !defined(WIN32)
   glutPositionWindow(0,0);
    // essential for this to work under Ubuntu
#endif
   int gdb=glutGet(GLUT_WINDOW_DOUBLEBUFFER);
   cout<<endl<<"GLUT_WINDOW_DOUBLEBUFFER="<<gdb<<endl;
   Z dmp=glutGet(GLUT_DISPLAY_MODE_POSSIBLE);
   cout<<"GLUT_DISPLAY_MODE_POSSIBLE="<<dmp<<endl;
   Z wbs=glutGet(GLUT_WINDOW_BUFFER_SIZE);
   cout<<"GLUT_WINDOW_BUFFER_SIZE="<<wbs<<endl;
   cpmwait(0.5);
   Z width=Message::w(); // // = Message::vpw_
   Z height=Message::h(); // = Message::vph_
   glutReshapeWindow(width,height);
   Z hSB=Viewport::getHeightStatusBar();
   Z widthFrmStr=Message::ws(); // = Message::fsw_
   Z heightFrmStr=Message::hs(); // = Message::fsh_
   Z heightFree=height-hSB;
   if (heightFrmStr<=height && heightFrmStr>heightFree){
      // then showing the bitmaps should fill the
      // available screen window as completely as possible
      heightFrmStr=heightFree;
   }
   cout<<"widthFrmStr="<<widthFrmStr<<endl;
   cout<<"heightFrmStr="<<heightFrmStr<<endl;
   Frame::setStaticSize(widthFrmStr,heightFrmStr);
   // The size of the system's display screen has no influence on this
   // action. Class' Frame internal frame store is set to this size
   // and if the display screen is smaller, it displays only a
   // part of the frame store. The whole frame store can always be
   // represented as a PPM-image file on disk.
   // This call also triggers loading the Cpm fonts, which will be used
   // by placing text to frame store content. This takes some time (18.4 s
   // presently). Since this waiting time was considered disturbing I
   // separated font reading and placed it prior to calling glutInit.
   // Now it takes about 0.25 s.
   glutReshapeFunc(reshapeFunc); // Here viewport size from Message
      // is transfered to glut
   glutDisplayFunc( displayFunc );
   glutIdleFunc( idleFunc );
      // This function will turn the chaotic dissected initial window
      // into a regular one.
      // This is the place where main_() gets called.
   glutMainLoop();
   return 0; // never done since finally idleFunc calls exit
}

#else // i.e. defined(CPM_NOGRAPHICS)
// no glut and openGL functionality needed. Graphics
// may nevertheless be generated as *.ppm bitmaps of arbitrary
// pixel count (to be set in file cpmconfig.ini)

Z mainFunc( )
// calls main_ in a suitable context
{
   cpmmessage("mainFunc() started");
   Z res=main_();
   cpmmessage(loc_&"main_() done; res was "&cpm(res));
      // notice that loc_ got a reasonable value in main()
   cpmurgent("Computation done");
   R someTimeWait=0.5;
   // notice that a wait time for inspecting the final graphics can be
   // set with the *.ini-file of the main application so that there is no
   // need to have a long time here
   cpmwait(someTimeWait);
   cpmmessage("mainFunc() done, return value of main_()) is "
      &cpm(res));
   return res;
}

//#if defined(WIN32)&&defined(_CONSOLE)
//#include <tchar.h>
//int _tmain(int argc, _TCHAR* argv[])
//#else
int main(int argc, char *argv[])
//#endif
// notice C-native argument types in order to not irritate the compiler.
// This is the main function for a application that under Windows should
// be startable by (double)clicking on the exe-file. Then nothing but
// the application's name gets transmitted to the present function
// (only args_[1] is the defined). For providing additional
// data such as the desired size of the application window,
// we read data from file cpmconfig.ini which is expected to
// be located in the same directory as the executable
{
   using CpmApplication::args_;
   using CpmGraphics::Viewport;
   using CpmGraphics::Frame;
   cpmmessage("C+- says Hello World");
 //  if (rank==1){
      std::ostringstream ost;
      ost<<endl<<"Function main() started for program "<<argv[0]<<endl;
      ost<<endl<<" notice that CPM_NOGRAPHICS is defined"<<endl;
      cpmmessage(ost);
      cpmwait(1.);
 //  }
   updFromConfigFile(); // now cpmconfig.ini has been read
   args_=extComLine(argc,argv);
   std::ostringstream ost2;
   ost2<<"Extended command line transformed to "<<args_;
   cpmmessage(ost2);
   argv_=args_[1];
   loc_=Word( "main(" );
   loc_=loc_&cpm(Z(argc))&", "&args_[1]&"): ";
   Z height=Message::w();
   Z hSB=Viewport::getHeightStatusBar();
   Z widthFrmStr=Message::ws();
   Z heightFrmStr=Message::hs();
   Z heightFree=height-hSB;
   if (heightFrmStr<=height && heightFrmStr>heightFree){
      // then showing the bitmaps should fill the
      // available screen window as completely as possible
      heightFrmStr=heightFree;
   }
   Frame::setStaticSize(widthFrmStr,heightFrmStr);
      // the actual screen size has no influence on this action
   Z res=mainFunc();
      // this is the place where main_() gets called.
   cpmmessage("function main() done for program "&argv_);
   return res;
}

#endif // defined(CPM_NOGRAPHICS)
