//? cpmfftmore.cpp
//? C+- by Ulrich Mutze. Status of work 2022-05-06.
//? Copyright (c) 2022 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//? 
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//? 
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for 
//? more details.


#include <cpmfftmore.h>

using CpmRoot::Z;
using CpmRoot::R;
using CpmRoot::C;
using CpmRoot::Word;

using CpmArrays::V;
using CpmArrays::VV;
using CpmArrays::VVV;

using CpmFourier::SD1;
using CpmFourier::FD1;
using CpmFourier::SD2;
using CpmFourier::FD2;
using CpmFourier::SD3;
using CpmFourier::FD3;

namespace{

Z powOf2(Z n) // see N po2(N) in cpmfft.cpp
{
   Z res=1;
   while (res<n) res*=2;
   return res;
}

}// namespace

////////// classes SD1, SD2, SD3, FD1, FD2, FD3 ////////////////////////
// constructors

SD1::SD1(Z n1):rep(powOf2(n1)){}
SD2::SD2(Z n1,Z n2):rep(powOf2(n1),powOf2(n2)){}
SD3::SD3(Z n1,Z n2,Z n3):rep(powOf2(n1),powOf2(n2),powOf2(n3)){}

FD1::FD1(Z n):rep(powOf2(n)){}
FD2::FD2(Z n1,Z n2):rep(powOf2(n1),powOf2(n2)){}
FD3::FD3(Z n1,Z n2,Z n3):rep(powOf2(n1),powOf2(n2),powOf2(n3)){}

// The parallelism of implementation that is apparent from the previous
// lines will be rationalized for the more complex function bodies

// construction from arrays
#define CPM_SC1 \
{\
   if (!careful) rep=v;\
   else{\
      Z n=v.dim();\
      Z n2=powOf2(n);\
      if (n==n2) rep=v;\
      else{\
         rep=V<C>(n2);\
         for (Z i=1;i<=n;++i) rep[i]=v[i];\
      }\
   }\
}

SD1::SD1(V<C> const& v, bool careful)
CPM_SC1

FD1::FD1(V<C> const& v, bool careful)
CPM_SC1

#undef CPM_SC1

#define CPM_SC2 \
{\
   if (!careful) rep=v;\
   else{\
      Z n1=v.dim1();\
      Z n2=v.dim2();\
      Z d1=powOf2(n1);\
      Z d2=powOf2(n2);\
      Z i,j;\
      if (d1==n1 && d2==n2) rep=v;\
      else{\
         rep=VV<C>(d1,d2);\
         for (i=1;i<=n1;++i) for (j=1;j<=n2;++j){\
            rep[i][j]=v[i][j];\
         }\
      }\
   }\
}

SD2::SD2(VV<C> const& v, bool careful)
CPM_SC2

FD2::FD2(VV<C> const& v, bool careful)
CPM_SC2

#undef CPM_SC2

#define CPM_SC3 \
{\
   if (!careful) rep=v;\
   else{\
      Z n1=v.dim1();\
      Z n2=v.dim2();\
      Z n3=v.dim3();\
      Z d1=powOf2(n1);\
      Z d2=powOf2(n2);\
      Z d3=powOf2(n3);\
      Z i,j,k;\
      if (d1==n1 && d2==n2 && d3==n3) rep=v;\
      else{\
         rep=VVV<C>(d1,d2,d3);\
         for (i=1;i<=n1;++i) for (j=1;j<=n2;++j) for (k=1;k<=n3;++k){\
            rep[i][j][k]=v[i][j][k];\
         }\
      }\
   }\
}

SD3::SD3(VVV<C> const& v, bool careful)
CPM_SC3

FD3::FD3(VVV<C> const& v, bool careful)
CPM_SC3

#undef CPM_SC3

// random constructor

#define CPM_SC1 \
{\
   Z i,n2=rep.dim();\
   for (i=1;i<=n2;++i){\
      rep[i]=z.ran(seed++);\
   }\
}

SD1::SD1(Z n, C z, Z& seed):rep(powOf2(n))
CPM_SC1

FD1::FD1(Z n, C z, Z& seed):rep(powOf2(n))
CPM_SC1

#undef CPM_SC1

#define CPM_SC2 \
{\
   Z i,j,d1=rep.dim1(),d2=rep.dim2();\
   for (i=1;i<=d1;++i) for (j=1;j<=d2;++j){\
      rep[i][j]=z.ran(seed++);\
   }\
}

SD2::SD2(Z n1, Z n2, C z, Z& seed):rep(powOf2(n1),powOf2(n2))
CPM_SC2

FD2::FD2(Z n1, Z n2, C z, Z& seed):rep(powOf2(n1),powOf2(n2))
CPM_SC2

#undef CPM_SC2

#define CPM_SC3 \
{\
   Z i,j,k,d1=rep.dim1(),d2=rep.dim2(),d3=rep.dim3();\
   for (i=1;i<=d1;++i) for (j=1;j<=d2;++j) for (k=1;k<=d3;++k){\
      rep[i][j][k]=z.ran(seed++);\
   }\
}

SD3::SD3(Z n1,Z n2,Z n3,C z,Z& seed):
rep(powOf2(n1),powOf2(n2),powOf2(n3))
CPM_SC3

FD3::FD3(Z n1,Z n2,Z n3,C z,Z& seed):
rep(powOf2(n1),powOf2(n2),powOf2(n3))
CPM_SC3

#undef CPM_SC3

// arithmetics
// sum
SD1 SD1::operator+(SD1 const& fd)const{ return SD1(rep+fd.rep);}
SD2 SD2::operator+(SD2 const& fd)const{ return SD2(rep+fd.rep);}
SD3 SD3::operator+(SD3 const& fd)const{ return SD3(rep+fd.rep);}

FD1 FD1::operator+(FD1 const& fd)const{ return FD1(rep+fd.rep);}
FD2 FD2::operator+(FD2 const& fd)const{ return FD2(rep+fd.rep);}
FD3 FD3::operator+(FD3 const& fd)const{ return FD3(rep+fd.rep);}

// difference
SD1 SD1::operator-(SD1 const& fd)const{ return SD1(rep-fd.rep);}
SD2 SD2::operator-(SD2 const& fd)const{ return SD2(rep-fd.rep);}
SD3 SD3::operator-(SD3 const& fd)const{ return SD3(rep-fd.rep);}

FD1 FD1::operator-(FD1 const& fd)const{ return FD1(rep-fd.rep);}
FD2 FD2::operator-(FD2 const& fd)const{ return FD2(rep-fd.rep);}
FD3 FD3::operator-(FD3 const& fd)const{ return FD3(rep-fd.rep);}

// minus
SD1 SD1::operator-()const{ return SD1(-rep);}
SD2 SD2::operator-()const{ return SD2(-rep);}
SD3 SD3::operator-()const{ return SD3(-rep);}

FD1 FD1::operator-()const{ return FD1(-rep);}
FD2 FD2::operator-()const{ return FD2(-rep);}
FD3 FD3::operator-()const{ return FD3(-rep);}

// product with C
SD1 SD1::operator*(C const& z)const{ return SD1(rep*z);}
SD2 SD2::operator*(C const& z)const{ return SD2(rep*z);}
SD3 SD3::operator*(C const& z)const{ return SD3(rep*z);}

FD1 FD1::operator*(C const& z)const{ return FD1(rep*z);}
FD2 FD2::operator*(C const& z)const{ return FD2(rep*z);}
FD3 FD3::operator*(C const& z)const{ return FD3(rep*z);}

// scalar product
C SD1::operator |(SD1 const& fd)const{ return (rep|fd.rep);}
C SD2::operator |(SD2 const& fd)const{ return (rep|fd.rep);}
C SD3::operator |(SD3 const& fd)const{ return (rep|fd.rep);}

C FD1::operator |(FD1 const& fd)const{ return (rep|fd.rep);}
C FD2::operator |(FD2 const& fd)const{ return (rep|fd.rep);}
C FD3::operator |(FD3 const& fd)const{ return (rep|fd.rep);}

// discrete fourier transformation
FD1 SD1::fft()const{ return FD1(CpmFourier::fft(rep,1));}
FD2 SD2::fft()const{ return FD2(CpmFourier::fft(rep,1));}
FD3 SD3::fft()const{ return FD3(CpmFourier::fft(rep,1));}

SD1 FD1::fft()const{ return SD1(CpmFourier::fft(rep,-1));}
SD2 FD2::fft()const{ return SD2(CpmFourier::fft(rep,-1));}
SD3 FD3::fft()const{ return SD3(CpmFourier::fft(rep,-1));}

// conjugation
SD1 SD1::con()const{ return SD1(rep.con());}
SD2 SD2::con()const{ return SD2(rep.con());}
SD3 SD3::con()const{ return SD3(rep.con());}

// conjugation in frequency domain is different

FD1 FD1::con()const
{
   Z i,ic,n=rep.dim();
   Z np=n+2;
   V<C> res(n);
   for (i=1;i<=n;++i){
      ic=(i==1 ? 1 : np-i);
         // ic is what in the general explanation is written i~
      res[i]=rep[ic].con();
   }
   return FD1(res);
}

FD2 FD2::con()const
{
   Z i,j,ic,jc,n1=rep.dim1(),n2=rep.dim2();
   Z n1p=n1+2, n2p=n2+2;
   VV<C> res(n1,n2);
   for (i=1;i<=n1;++i){
      ic=(i==1 ? 1 : n1p-i);
      for (j=1;j<=n2;++j){
         jc=(j==1 ? 1 : n2p-j);
         res[i][j]=rep[ic][jc].con();
      }
   }
   return FD2(res);
}

FD3 FD3::con()const
{
   Z i,j,k,ic,jc,kc,n1=rep.dim1(),n2=rep.dim2(),n3=rep.dim3();
   Z n1p=n1+2, n2p=n2+2, n3p=n3+2;
   VVV<C> res(n1,n2,n3);
   for (i=1;i<=n1;++i){
      ic=(i==1 ? 1 : n1p-i);
      for (j=1;j<=n2;++j){
         jc=(j==1 ? 1 : n2p-j);
         for (k=1;k<=n3;++k){
            kc=(k==1 ? 1 : n3p-k);
            res[i][j][k]=rep[ic][jc][kc].con();
         }
      }
   }
   return FD3(res);
}

R CpmFourier::fftTest1()
// demonstrates unitarity of the fourier transform
{
   Z n=32;
   C z(1,1);
   Z i,j,k=1;
   VVa<C> x1(n,n);
   VVa<C> x2(n,n);
   for (i=1;i<=n;++i) for (j=1;j<=n;++j){
      x1[i][j]=z.ran(k++);
      x2[i][j]=z.ran(k++);
   }
   C sp1=(x1|x2);
   Z ex=-1;
   VVa<C> y1=fft(x1,ex);
   VVa<C> y2=fft(x2,ex);
   C sp2=(y1|y2);
   R diff=(sp1-sp2).abs();
   return diff;
}

R CpmFourier::fftTest2()
// demonstrates linearity of the fourier transform
// and correctness of the inversion
{
   Z n=256;
   C z1(1,1);
   Z seed=1;
   FD1 x1(n,z1,seed);
   FD1 x2(n,z1,seed);
   FD1 x3=(x1+x2)*z1;
   SD1 y1=x1.fft();
   SD1 y2=x2.fft();
   SD1 y3=(y1+y2)*z1;
   SD1 y3_=x3.fft();
   R diff=(y3-y3_).abs();
   FD1 x3_=y3.fft();
   R diff2=(x3-x3_).abs();
   return diff+diff2;
}

R CpmFourier::fftTest3()
// demonstrates correct behavior of re
{
   Z n=256;
   C z(1,1);
   Z seed=1;
   SD1 x(n,z,seed);
   SD1 xc=~x;
   FD1 y=x.fft();
   FD1 yc=~y;
   FD1 yc_=xc.fft();
   R diff=yc.dis(yc_);
   return diff;
}

