//? cpmframe.cpp
//? C+- by Ulrich Mutze. Status of work 2022-05-06.
//? Copyright (c) 2022 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//?
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//?
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for
//? more details.


#include <cpmframe.h>
#include <cpms.h>
#include <cpmfontgeneration.h>
#include <cpmimage.h>

namespace CpmGraphics{

   using CpmImaging::Rect;
   using CpmImaging::Mask;
   using CpmImaging::Image;
   using CpmImaging::Histogram;
   using CpmImaging::Color;
   using CpmImaging::INVISIBLE;
   using CpmImaging::BLACK;
   using CpmImaging::STEREOBLACK;
   using CpmImaging::WHITE;
   using CpmImaging::WRITECOLOR;
   using CpmImaging::LIGHTGREEN;
   using CpmRootX::order;

   using namespace CpmRoot;
   using namespace CpmSystem;
   using namespace CpmArrays;
   using namespace CpmFunctions;

////////////////////// class FrmText ///////////////////////////////////

FrmText::FrmText(V<TextCom> const& tcs):tcs_(0)
{
   Z mL=3;
   Word loc("FrmText(V<TextCom>)");
   CPM_MA
   Z n=tcs.dim();
   S<R2> ps;
   for (Z i=1;i<=n;++i){
      TextCom ti=tcs[i];
      R2 pi=ti.c1();
      bool bi=ps.addAct_(pi);
      if (bi) tcs_&=ti;
   }
   CPM_MZ
}

FrmText::FrmText(V<Word> const& vw, Frame const& fr, V<Color> const& c,
   R down, R toRight):tcs_(vw.dim())
{
   Z mL=3;
   Word loc("FrmText(V<Word>,Frame,V<Color>,R))");
   CPM_MA
   using CpmFonts::Font;
   Z i,j,nL=vw.dim();
   V<Font> ft=Frame::ft_;
   IvZ fdo=ft.dom();
   Vo<Z> ht(fdo);
      // total minimum height (in pixels) needed for the text
      // in the various available fonts
   Vo<Z> wt(fdo);
      // total minimum width (in pixels) needed for the text
      // in the various available fonts
   Z hFr=fr.h();
   Z wFr=fr.w();
   for (i=ft.b();i<=ft.e();++i){
      Font fi=ft[i];
      ht[i]=fi.h()*nL;
      Vo<Z> wi(nL);
      for (j=1;j<=nL;++j){ wi[j]=fi.len(vw[j]);}
      wt[i]=wi.sup();
   }
// Font selection
   V<B> fit(fdo);
   for (i=ft.b();i<=ft.e();++i){
      bool bi=ht[i]<hFr && wt[i]<wFr;
      if (bi) fit[i]=true;
      else break;
   }
   Z i0=fit.b();
   if (!fit.fir()) ;
   else if (fit.last()) i0=fit.e();
   else{
      while (i0<ft.e() && fit[i0+1]) i0++;
   }
   Font ft0=ft[i0];
   R hFont=ft0.h();
   Z h0=ht[i0];
      // maximum width of text for ft0
   Z w0=wt[i0];
      // pure height of text for ft0
   Z dh=hFr-h0;
   Z dw=wFr-w0;
   R x0=0, y0=1; // start position
      // if no space is available
   R hFontRel=hFont/hFr;
   R lineShiftRel=hFontRel*1.25;
   if (dw>0) x0=(dw*0.25)/wFr+toRight;
  // if (dw>0) x0=(dw*0.05)/wFr;
      // leave a left margin if there is space
   if (dh>0) y0=R(1)-down-(dh*0.25)/hFr;
      // leave an upper margin if there is space
// place the lines from top to bottom
   for (i=1;i<=nL;++i){
      tcs_[i]=X4<R2,Word,Color,R>(R2(x0,y0),vw[i],c[i],hFont);
      y0-=lineShiftRel;
   }
   CPM_MZ
}

FrmText& FrmText::operator+=(TextCom const& tp)
{
   R2 p=tp.c1();
   Z i=1,n=tcs_.dim();
   bool found=false;
   while (!found && i<=n){
      TextCom ti=tcs_[i];
      if (p==ti.c1()){
         tcs_[i]=tp;
         found=true;
      }
      i++;
   }
   if (!found) tcs_&=tp;
   return *this;
}

FrmText FrmText::operator+(TextCom const& tp)const
{
   FrmText res=*this;
   return res+=tp;
}

bool  FrmText::prnOn( ostream& str) const
{
   cpmwat;
   cpmp(tcs_);
   return true;
}

bool  FrmText::scanFrom (istream& str)
{
   cpms(tcs_);
   return true;
}

Z FrmText::com(FrmText const& s)const
{
   if (tcs_<s.tcs_) return 1;
   if (tcs_>s.tcs_) return -1;
   return 0;
}

////////////// utility functions for class Frame ///////////////////////

namespace{ // anonymus

   inline Z discr(R x){ return cpmtoz(x);}

   void reduce(R& x)
// Auxiliar function which brings the argument back into the the interval
//      [0,1]
{
   static const R tiny=1e-6; // 1./ maximum pixel number to be considered
   static const R upper=R(1)-tiny;
   if (x<=tiny) x=tiny;
   if (x>=upper) x=upper;
}

   inline void confine(R& x, R low, R upp)
       // inline version of previous function
   {
      if (x<low) x=low;
      if (x>upp) x=upp;
   }

   R powOf10(R x)
{
   R ten(10.);
   static const R ln10=cpmlog(ten);
   return cpmexp(x*ln10);
}

   R cut(R xCut)
// Returns the nearest number to x, which up to sign and power of 10
// is one of the L-1 first numbers in the array c[nL] in the function
// body.
// say c[nL]={1,2,5,10}
{
   const Z mL=3;
   cpmmessage(mL,"cut(xCut) called for xCut="&cpmwrite(xCut));
   cpmcheck(xCut); // the function gets values which often are obtained
      // by complex computations. I sometimes had #INF or inf here.
   Z nL=5;
   V<R> c(nL,R(10.),0); // initialized as 10
   c[0]=1.;    // starting with 0 !
   c[1]=2.;
   c[2]=5.;
   c[3]=7.5;
      // The list must be in ascending  order and must contain
      // (nL - 1) numbers r 0<r<10
      // These
   R y1,yu,yl,yf;
   if (xCut==0.) return 0.;
   Z  s=(xCut<0. ? -1: 1);
   y1=cpmabs(xCut);
   R ylog=cpmlog10(y1);
   R ylogInt=cpmfloor(ylog);
   R yMantisse=ylog-ylogInt;
   R y=powOf10(yMantisse);
   if (!(y>=1.)){
      cpmwarning("function R cut(R) (Frame): !(y>=1), xCut="&cpm(xCut));
      y=1.;
   }
   if (!(y<10.)){
      cpmwarning("function R cut(R) (Frame): !(y<=10), xCut="&cpm(xCut));
   }
   Z i=0;
   while (y>c[i]) i++;
   if (i<0 ||i>=nL) cpmerror("unexpected failure in function R cut(R)");
   yu=c[i];// yu is the smallest c[k] which is larger or equal to y
   if (i==0){
      yf=yu;
   }
   else{
      if ((i-1)<0 ||(i-1)>=nL)
      cpmerror("unexpected failure in function R cut(R)");
      yl=c[i-1];
      yf=(yu-y<y-yl ? yu : yl);
   }
   // now yf is the c[k] which is nearest to y
   R res=yf*powOf10(ylogInt)*s; // sign and exponent are restored
   cpmmessage(mL,"cut(xCut) done with result="&cpmwrite(res));
   return res;
}

}//namespace

Z lad_4(R lower, R upper, Z  n, R2& x)
{
   static Word loc("CpmGraphics::lad_4(R,R,Z,R2)");
   Z mL=3;
   CPM_MA
  // cpmcheck(lower); // the arguments of cpmcheck should have informative
      // names since the generated message contains the argument name
  // cpmcheck(upper);
   const R epsilon=1e-6;
      // introduced 2004-11-24, see place of usage
   if (lower>upper) CpmRootX::swap<R>(lower,upper);
   if (n<0) n=-n;
   if (n==0) n=1;
   R nInv=R(1)/n;

   if (lower==upper){   // 92-9-7 to avoid error message in cases where
   // function or list is constant
      if (lower==0){
         upper=1.; lower=-1.;
      }
      else{
         R step=cpmabs(lower)*0.1;
         upper+=step; lower-=step;
      }
   }

   R d=(upper-lower)*nInv;
   R d_=cut(d); // here is the Pudels Kern
   if (d_==0) cpmerror("Zero division in function ladder");
   R d_inv=R(1)/d_;
   R k_b=cpmceil(upper*d_inv-epsilon);
      // without the epsilon correction it depends on roundoff
      // errors whether b_ becomes too large (2004-11-24)
   R k_a=cpmfloor(lower*d_inv+epsilon);
      // without the epsilon correction it depends on roundoff
      // errors whether a_ becomes too small
   R b_=k_b*d_;
   R a_=k_a*d_;
   if (b_<upper){
      cpmwarning(
         loc&": mild accuracy problem: b_ < upper should not happen");
      cpmdebug(b_);
      cpmdebug(upper);
   }
   if (a_>lower){
      cpmwarning(
         loc&": mild accuracy problem: a_ > lower should not happen");
      cpmdebug(a_);
      cpmdebug(lower);
   }
   Z  n_= cpmrnd((b_-a_)*d_inv); // this shows that correcting a_ and b_
   // in case of the previously stated mild accuracy problem makes no
   // sense.
   x.x1=a_;
   x.x2=b_;
   CPM_MZ
   return (n_==0 ? 1 : n_);
}

///////////////////// class  Frame //////////////////////////////////////
// notice: colorActive is not static so it does not appear here !!!

bool Frame::xyPixelInitialized=false;
bool Frame::fontInitialized=false;

Z Frame::xPixel_=0;
Z Frame::yPixel_=0;
R Frame::xPixelInv=0;
R Frame::yPixelInv=0;
R Frame::hFrac=0.0225;
   // see function textHeight()
R Frame::yText=0.15;
Color Frame::defaultColorActive=WHITE;

V<CpmFonts::Font> Frame::ft_;

xyxy Frame::scrRec;
   // screen rectangle, static member. Will be meaningfully
   // initialized in Frame::iniStatics(void).

Z Frame::imageCounter=0;

static const Z N_EXCESS=40;
static const R IMMENSE=1e99;

Z Frame::fontSelector( R h)
{
   R_Vector vh(ft_.dom());
   for (Z i=vh.b();i<=vh.e();++i) vh[i]=ft_[i].h();
   return vh.indOfNerCom(h);
}

Z2  Frame::get_pU(void)const{ return pU;}

Z2  Frame::get_pL(void)const{ return pL;}

void Frame::updateRowCol()
{
   nRow=pL.x2-pU.x2+1; // The lower corner has the larger y-component
      // since y is counted from above !
   nCol=pU.x1-pL.x1+1;
   cpmassert(nCol>0 && nRow>0,"void Frame::updateRowCol(void)");
   nRowInv=1./nRow;
   nColInv=1./nCol;
   ColSty sty=img.getSty();
   img=Img24(nCol,nRow,colorActive,sty);
}

void Frame::stereo()
{
   img.stereo();
   colorActive=Color(STEREOBLACK);
}

void Frame::updateFromDiscrete()
{
   pU.x1=pL.x1+nCol-1;
   if (pU.x1>=xPixel_) pU.x1=xPixel_-1;
   if (pU.x1<0) pU.x1=0;

   pU.x2=pL.x2-nRow+1;
   if (pU.x2>=yPixel_) pU.x2=yPixel_-1;
   if (pU.x2<0) pU.x2=0;

   nRow=pL.x2-pU.x2+1; // The lower corner has the larger y-component
      // since y is counted from above !
   nCol=pU.x1-pL.x1+1;
   cpmassert(nCol>0 && nRow>0,"void Frame::updateFromDiscrete(void)");
   nRowInv=R(1)/nRow;
   nColInv=R(1)/nCol;

   cL.x1=R(pL.x1)*xPixelInv;
   cU.x1=R(pU.x1+1)*xPixelInv;
   cL.x2=R(1)-R(pL.x2+1)*yPixelInv;
   cU.x2=R(1)-R(pU.x2)*yPixelInv;
   img.resize(nCol,nRow);  // paints the image with default color
}

Z2  Frame::SxyToSab(R2 c)
{
   static const R low=1e-6;
   static const R upp=R(1)-low;
   R x=c.x1, y=R(1)-c.x2;
   confine(x,low,upp); // inline, thus fast
   confine(y,low,upp);
   return Z2(cpmtoz(x*xPixel_),
      cpmtoz(y*yPixel_)); // efficient
}

Z2  Frame::discrete(R2& pLower, const R2& p, R2& pUpper)
   {
      Word loc("Frame::discrete(R2,R2,R2)");
      R2 q=p;
      reduce(q.x1); reduce(q.x2);
      if (xPixel_<=0) xPixel_=1;
      if (yPixel_<=0) yPixel_=1;
      cpmassert(xPixel_!=0,loc);
      cpmassert(yPixel_!=0,loc);
      R Dx=1./xPixel_;
      R Dy=1./yPixel_;
      R nx=q.x1*xPixel_;
      R ny=q.x2*yPixel_;
      Z2 z;
      z.x1=discr(nx); // result never <= -1 negative or >=xPixel_
         // due to reduce
      z.x2=discr(ny);
      pLower.x1=R(z.x1)*Dx;
      pLower.x2=R(z.x2)*Dy;
      pUpper.x1=pLower.x1+Dx;
      pUpper.x2=pLower.x2+Dy;
      z.x2=yPixel_-z.x2-1; // since z.x2 counts from above
      return z;
   }

Z2  Frame::getiFjF(const R2& xFyF)const
   // given coordinates of a point in the frame system, the
   // matrix indexes of the pixel to which the point belongs
   // is given
{
   R x=xFyF.x1, y=R(1)-xFyF.x2;
   reduce(x); reduce(y);
    Z i,j;
   i=cpmtoz(y*nRow); j=cpmtoz(x*nCol);
   return Z2(i+1,j+1);
}

void  Frame::setxy(R x, R y, R dx, R dy)
// Helps to consistently set cL.x, cU.x, cL.x2, cU.x2
// The pL, pU, nRow, nCol, nRowInv, nColInv are set accordingly
{
   R x_=x+dx, y_=y+dy;
   order(x,x_);
   order(y,y_);
   R2 cUpper,cLower;
   R2 cLow(x,y);
   R2 cUpp(x_,y_);
   pL=discrete(cL,cLow,cUpper);  // setting pL and cL (cUpper not used)
   pU=discrete(cLower,cUpp,cU);  // setting pU and cU (cLower not used)
   updateRowCol();
   if (nCol==1 || nRow==1){
      Word mess=" Frame is degenerated to a line or a point";
      cpmwarning("Frame",mess);
   }
}

void  Frame::setpLpU(Z2 pLIn, Z2 pUIn)
{
   Z iL=pLIn.x1;
   Z jL=pLIn.x2;
   Z iU=pUIn.x1;
   Z jU=pUIn.x2;
   order(iL,iU);  // i horizontal distance from left screen column
   order(jU,jL); // notice j : vertical distance from upper screen line.
      // thus jL must be larger than jU
   if (iL<0) iL=0;
   if (iU>=xPixel_) iU=xPixel_-1;
   if (jU<0) jU=0;
   if (jL>=yPixel_) jL=yPixel_-1;
   pL.x1=iL; // (*)
   pL.x2=jL;
   pU.x1=iU;
   pU.x2=jU;
   cL=SabToSxy(pL); // was placed directly after (*) till 2002-02-08
      // and thus did cause mysterious malfunction of the Frames
      // constructors.
   cU=SabToSxy(pU);
   updateRowCol(); // makes image black
   xy pNow=img.cor();
   xy pNew=cor();
   xy shift=pNew-pNow;
   img+=shift;
}

void  Frame::iniStatics(void)
{
   Z mL=2;
   Word loc("Frame::iniStatics(void)");
   CPM_MA
   if (!xyPixelInitialized && xPixel_>0 && yPixel_>0){
      // since this is a static member of the class it can be
      // manipulated by member functions
      scrRec=xyxy(xy(0,0),xy(xPixel_,yPixel_));
      xPixelInv=1./xPixel_;
      yPixelInv=1./yPixel_;
      xyPixelInitialized=true;
   }
   CPM_MZ
}

void  Frame::iniFonts(void)
{
   Z mL=2;
   Word loc("Frame::iniFonts()");
   CPM_MA
   if (!fontInitialized){
      Word fontfile=Message::getFontDir()&"/cpmfont1.txt";
      IFileStream in(fontfile);
      cpmmessage("loading C+- fonts started",-1);
      // This writes to the console
      fontInitialized=ft_.scanFrom(in());
      ft_.b_(1); // for ordering according to font height this
         // is needed.
      Z n=ft_.dim();
      cpmmessage("loading of "&cpm(n)&" C+- fonts done",-1);
      Vo<CpmFonts::Font> fto(ft_); // Font implements CPM_ORDER
         // by height
      fto.sort_();
      ft_=fto;
      for (Z i=ft_.b();i<=ft_.e();++i){
         cpmmessage("height of font "&cpm(i)&" is "&cpm(ft_[i].h())
         &" pixel",-1);
      }
   }
   CPM_MZ
}

void Frame::setStaticSize(Z width, Z height)
{
   xPixel_=width;
   yPixel_=height;
   xyPixelInitialized=false;
   iniFonts();
   iniStatics();
}

void  Frame::iniFrame(R x, R y, R dx, R dy, const Color& color)
   // tool for defining constructors without code repetition
{
   Z mL=3;
   Word loc("Frame::iniFrame(R, R, R, R, Color)");
   CPM_MA
   iniStatics();
   setxy(x, y, dx, dy);
   colorActive=color;
   if (color.stereo()) img.stereo();
   CPM_MZ
}

Frame::Frame(R x, R y, R dx, R dy, const Color& color)
// generic constructor
{
   Z mL=3;
   Word loc("Frame(R,R,R,R,Color)");
   CPM_MA
   iniFrame(x,y,dx,dy,color);
   img=Img24(nCol,nRow,color);
   if (color.stereo()) img.stereo();
   img+=cor();
      // there should be nothing wrong with showing a new Frame
   img.vis();
   CPM_MZ
}

Frame::Frame(const Img24& im, R x, R y, bool showIt)
{
   Z mL=3;
   Word loc("Frame(Img24,R,R,bool)");
   CPM_MA
   iniStatics();
   Z nx=im.getCol();
   Z ny=im.getLin();
   R dx=((R)nx)/xPixel_;
   R dy=((R)ny)/yPixel_;
   cpmassert(x>=0&&x<=1.,loc);
   cpmassert(y>=0&&y<=1.,loc);
   setxy(x,y,dx,dy); // changes img
   Z2 pL0=pL;
   Z2 pU0=pL0+Z2(nx-1,-ny+1);
   setpLpU(pL0,pU0);
   img=im; // has to come after setxy(...) and setpLpU(...)
   img+=cor();
   if (showIt) vis();
   CPM_MZ
}

Frame::Frame(void)
// default constructor, no painting
{
   Z mL=3;
   Word loc("Frame(void)");
   CPM_MA
   Color c=INVISIBLE; // this avoids the painting action
   iniFrame(0,0,1,1,c); // so far, colorActive=INVISIBLE
   colorActive=defaultColorActive;
   img=Img24(nCol,nRow,colorActive);
   img+=cor();
   CPM_MZ
}

Frame::Frame(const Color& c)
{
   Z mL=3;
   Word loc("Frame(Color)");
   CPM_MA
   iniFrame(0,0,1,1,c);
   img=Img24(nCol,nRow,c);
   img+=cor();
   if (c.stereo()) img.stereo();
   CPM_MZ
}

Frame::Frame(Z code)
   // Constructor for convenient placing of a frame
   // The screen is divided in quadrants denoted in matrix-style
   // 11 12
   // 21 22
   // Posible position codes are 11, 12, 21, 22, 1112 (for upper
   // half part) 2122 (lower half part), 1121 (left half part)
   // 1222 (right part)
{
   Z mL=2;
   Word loc("Frame(Z)");
   CPM_MA
   if (cpmverbose>=4) cpmmessage("Frame(Z code) called");
   iniStatics();
   switch (code){
      case 11:   setxy(0.0, 0.5, 0.5, 0.5); break;
      case 12:   setxy(0.5, 0.5, 0.5, 0.5); break;
      case 21:   setxy(0.0, 0.0, 0.5, 0.5); break;
      case 22:   setxy(0.5, 0.0, 0.5, 0.5); break;
      case 1112: setxy(0.0, 0.5, 1.0, 0.5); break;
      case 2122: setxy(0.0, 0.0, 1.0, 0.5); break;
      case 1121: setxy(0.0, 0.0, 0.5, 1.0); break;
      case 1222: setxy(0.5, 0.0, 0.5, 1.0); break;
      default:   setxy(0.0, 0.0, 1.0, 1.0); break;
   }
   colorActive=defaultColorActive;
   if (colorActive.visible()) paint(colorActive);
   img=Img24(nCol,nRow,colorActive);
   img+=cor();
   CPM_MZ
}

Rect  Frame::getRectangle(void)const
   // returns the rectangle from which the Frame 'could have been
   // created'.
{
   return Rect(cL.x1,cU.x1,cL.x2,cU.x2);
}

Frame::Frame(const Rect& rec)
   // constructor from a rectangle in the whole screen
{
   Z mL=2;
   Word loc("Frame(Rect)");
   CPM_MA
   R x=rec.getxL();
   R y=rec.getyL();
   R dx=rec.getxU()-x;
   R dy=rec.getyU()-y;
   iniFrame(x,y,dx,dy,defaultColorActive);
   img=Img24(nCol,nRow,colorActive);
   img+=cor();
   CPM_MZ
}

Frame::Frame(const Frame& fr, const Rect& rec)
   // constructor from a rectangle of the rectangle given by fr
{
   Z mL=2;
   Word loc("Frame(Frame,Rect)");
   CPM_MA
   Rect recfr=fr.getRectangle();
   Rect recfinal=recfr.sub(rec);
   R x=recfinal.getxL();
   R y=recfinal.getyL();
   R dx=recfinal.getxU()-x;
   R dy=recfinal.getyU()-y;
   iniFrame(x,y,dx,dy,defaultColorActive);
   img=Img24(nCol,nRow,colorActive);
   img+=cor();
   CPM_MZ
}

Z  Frame::ix(R xF)const
// 0<= xF <= 1 within the frame. Return value is the pixel number on
// screen which corresponds to to the coordinate xF within the frame.
{
   if (xF<0) return pL.x1;
   else if (xF>1) return pU.x1;
   else{
      R a=cL.x1+(cU.x1-cL.x1)*xF;
      return discr(R(xPixel_)*a);
   }
}

Z  Frame::jy(R yF)const
// 0<= yF <= 1 within the frame. Return value is the pixel number on
// screen which corresponds to to the coordinate yF within the frame.
// Notice that the meaning of pixel numbers is determined by the syntax
// of the putPel function. The corners of the screen are
// putPel(0,0) (upper-left)        putPel(xPixel_,0) (upper-right)
// putPel(0,yPixel_) (lower-left)   putPel(xPixel_,yPixel_) (lower-right)
{
   if (yF<=0) return pL.x2;
   else if (yF>=1) return pU.x2;
   else{
      R b=cL.x2+(cU.x2-cL.x2)*yF;
      return discr(R(yPixel_)*(R(1)-b));
   }
}

void  Frame::paint(const Color& c)
{
   if (c.visible()) img.paint(c);
}

void Frame::fitImg(Img24 const& im, bool conAspRat)const
{
   if (!conAspRat){
      img.insert(xy(),im.interpolate(img.w(),img.h()));
      return;
   }
   xyxy r1=img.to_xyxy();
//   cout<<"r1="<<r1<<endl;
   xyxy r2=im.to_xyxy();
//   cout<<"r2="<<r2<<endl;
   xyxy r=r1.scaleToFit(r2);
//   cout<<"r="<<r<<endl;
   img.insert(xy(),im.interpolate(r.w(),r.h()));
}

void Frame::setAspRatHoldWidth(R asp)
{
   Word loc("setAspRatHoldWidth");
   cpmassert(asp>0,loc);
   nRow=cpmrnd(R(nCol)/asp);
   updateFromDiscrete();
}

void Frame::setAspRatHoldHeight(R asp)
{
   Word loc("setAspRatHoldHeight");
   cpmassert(asp>0,loc);
   nCol=cpmrnd(R(nRow)*asp);
   updateFromDiscrete();
}

void Frame::cut(R asp, Z dyPixel)
{
   Z mL=4;
   Word loc("Frame::cut(R,Z)");
   cpmmessage(mL,loc&" started");
   cpmassert(asp>0,loc);
   cpmassert(dyPixel>0,loc);
   R4 xydxdy=getxy();
   R x=xydxdy[1];
   R y=xydxdy[2];
   R dx=xydxdy[3];
   R dy=xydxdy[4];
   R dy0=(1.*dyPixel)/yPixel_;
      // made such that dy0*yPixel_=dyPixel
   R a=asp*screenAspRatInv();
   R dx0=a*dy0;
      // made such that dx0*xPixel_/(dy0*yPixel_)=asp
   R f1=dx/dx0;
   R f2=dy/dy0;
   R f3=1.;
   R f=cpminf(f1,f2,f3);
   R dx1=dx0*f;
   R dy1=dy0*f;
   R x1=x+(dx-dx1)*0.5;
   R y1=y+(dy-dy1)*0.5;
   setxy(x1,y1,dx1,dy1);
   img=Img24(nCol,nRow,colorActive);
   cpmmessage(mL,loc&" done");
}

void Frame::line(R a1, R b1, R a2, R b2, rgb c)const
{
   R x1=a1-a0(),y1=b1-b0(),x2=a2-a0(),y2=b2-b0();
   img.line(x1,y1,x2,y2,c);
}

namespace{
   void lineArtPrc(Img24& img)
   { // using Image<R> as workhorse, quite efficient
      const R limL=193;
      const R limU=255;
      // these constants are adjusted to the properties
      // of the specific binomial mask to be used.
      Image<R> im(img);
      Mask m=Mask::bin3();
      im=im.conv(m);
      R2 a(limL,limU);
      R2 b(0,255);
      bool linear=true;
      im=im.transformToneScale(R_Func::transit(a,b,linear));
      img=im.toImg24();
   }
}//namespace

Word Frame::vis(bool file,bool lineArtProcessing)const
{
   Z mL=3;
   Word loc("Frame::vis(bool,bool)");
   CPM_MA
   if (!noText()){
      for (Z i=text_.b();i<=text_.e();++i){
         TextCom ti=text_[i];
         R2 xi=ti.c1();
         xy pi=FxyToFab(xi);
         Word wi=ti.c2();
         ColRef ci=ti.c3();
         R hi=ti.c4();
         Z fi=fontSelector(hi);
         ft_[fi].wrt(wi,pi,img,ci);
      }
   }
   if (lineArtProcessing) lineArtPrc(img);
   img+=cor();
   if (!markMode) img.vis();
   Word res;
   if (file){
      imageCounter++;
      Word fn=Message::getOutDir2()&"/frame_"&cpm(imageCounter,3);
      if (Message::getAppRunId()){
         Word app("_");
         app&=Message::getRunId();
         fn&=app;
      }
      res=img.write(fn);
      cpmmessage(loc&": image "&res&" written");
   }
   CPM_MZ
   return res;
}

Word Frame::write(Word const& file)const
{
   Word res;
   Z mL=3;
   Word loc("Frame::write(const Word& file)");
   cpmmessage(mL,loc&" "&file&" started");
   Word fext=Message::extOut(file);
  // bool binary=false;
   res=img.write(fext);
   //res=img.write(file);
   cpmmessage(mL,loc&" "&file&" done");
   return res;
}

Word Frame::write(void)const
{
   Word res;
   Z mL=3;
   Word loc("Frame::write(void)");
   CPM_MA
   res=img.write();
   CPM_MZ
   return res;
}

void  Frame::clear()
{
   Z mL=3;
   Word loc("Frame::clear()");
   CPM_MA
   clearText();
   paint(colorActive);
   CPM_MZ
}

void Frame::fullClear()
{
   Z mL=3;
   Word loc("Frame::fullClear()");
   CPM_MA
   img.setColor(Color(BLACK));
   img.clear();
   CPM_MZ
}

bool  Frame::prnOn( ostream& str) const
{
   cpmwat;
   cpmp(colorActive);
   cpmp(cL);
   cpmp(cU);
   cpmp(img); // new: 2009-10-02
   cpmp(text_);
   return true;
}

bool  Frame::scanFrom (istream& str)
{
   cpms(colorActive);
   cpms(cL);
   cpms(cU);
   cpms(img); // new: 2009-10-02
   cpms(text_);
   R x=getx();
   R y=gety();
   R dx=getdx();
   R dy=getdy();
   setxy(x,y,dx,dy);
   return true;
}

Z Frame::com(Frame const& s)const
{
   if (colorActive<s.colorActive) return 1;
   if (colorActive>s.colorActive) return -1;
   if (cL<s.cL) return 1;
   if (cL>s.cL) return -1;
   if (cU<s.cU) return 1;
   if (cU>s.cU) return -1;
   if (img<s.img) return 1;
   if (img>s.img) return -1;
   if (text_<s.text_) return 1;
   if (text_>s.text_) return -1;
   return 0;
}

R Frame::textHeight()const
{
   Z n=CpmRootX::inf<Z>(nRow,nCol);
   return cpmrnd(hFrac*n);
}

void Frame::setText(Word const& text, R x, R y, Color c, R h)
{
   Z mL=3;
   Word loc("Frame::setText(Word,R,R,Color,R)");
   CPM_MA
   if (y<=0) y=yText;
   if (h<=0) h=textHeight();
   TextCom tc(R2(x,y),text,c,h);
   setText(V<TextCom>{tc});
   CPM_MZ
}

R2 Frame::plcText(Word const& text, R2 const& leftUppCor, Color c, Z ft)
{
   using CpmFonts::Font;
   R x=leftUppCor[1];
   R y=leftUppCor[2];
   if (x<0) x=0;
   if (x>=1) return leftUppCor;
   if (y>1) y=1;
   if (y<=0) return leftUppCor;
   if (ft<ft_.b()) ft=ft_.b();
   if (ft>ft_.e()) ft=ft_.e();
   Font font=ft_[ft];
   R wi=font.len(text);
   R hi=font.h();
   R wr=wi/w();
   R hr=hi/h();
   R xf=x+wr;
   R yf=y-hr;
   TextCom tc(R2(x,y),text,c,hi);
   setText(V<TextCom>{tc});
   return R2(xf,yf);
}

void Frame::setText(V<Word> const& lines, V<Color> const& c, R down, R toRight)
{
   Z mL=3;
   Word loc("Frame::setText(V<Word>,V<Color>,R)");
   CPM_MA
   FrmText ft(lines,*this,c,down,toRight);
   setText(ft);
   CPM_MZ
}

void Frame::addText(FrmText const& text)
{
   Z n=text.dim();
   for (Z i=1;i<=n;++i) text_+=text[i];
}

void Frame::addText(const Word& text, R x, R y, Color c, R h)
{
   if (y<=0) y=yText;
   if (h<=0) h=textHeight();
   TextCom tc(R2(x,y),text,c,h);
   addText(V<TextCom>{tc});
}

R Frame::markVVC(V<V<C>> const& matA, bool matrixstyle, Iv range,
   Z method, bool faithful, bool periodic, R secFac, bool bw)const
{
   Z mL=3;
   Word loc("Frame::markVVC(V<V<C>>,bool,Iv,Z,bool,bool,R,bool)");
   CPM_MA
   static bool firstrun=true;
   static Iv rangeAct;
   Z i,j;
   V< V<C> > mat=matA;
   mat.b_(0);
   for (i=mat.b();i<=mat.e();++i) mat[i].b_(0);
      // Now, independently of index ranges of matA, mat has
      // index ranges which start with 0. We use here code which made
      // this assumption.

   // From now on we are allowed to use plain indexing .pi(Z)

   Z mx=mat.dim();
 //  cpmdebug(mx);
   cpmassert(mx>0,loc);
   Z my=mat.pi(0).dim();
 //  cpmdebug(my);
   cpmassert(my>0,loc);
      // it is assumed that the first index of mat corresponds to a
      // x-coordinate and the second index to either the x-coordinate of
      // a second particle and or the y-coordinate of the first particle
      // (then there is no second one)
// Renewing range if needed
   R res=range.isVoid() ? 0. : range.sup();
   if ((range.isVoid()&& secFac<=0)||(firstrun && secFac>0)){
      R mm=-1;
      for (i=0;i<mx;++i){ //
         for (j=0;j<my;++j){
            R mij=mat.pi(i).pi(j).abs();
            if (mij>mm) mm=mij;
          }
      }
      if (secFac<=0){
         rangeAct=Iv(0,mm);
      }
      else{
         cpmassert(firstrun==true,loc);
         firstrun=false;
         rangeAct=Iv(0,secFac*mm);
      }
 //     cpmdebug(rangeAct);
      res=rangeAct.sup();
   }
// checking method
   cpmassert(method==0||method==1,loc);
// Main loops
   R rx=1., ry=1.;
   if (faithful){ // we assume square pixels on screen
      R alpha_= (matrixstyle ? (1.*my)/mx : (1.*mx)/my);
         // aspect ratio of image
      R alpha=getAspRat(); // aspect ratio of frame
      if (alpha_ > alpha){ // image has to be placed full width
         rx=1.;
         ry=alpha/alpha_; // <= 1
      }
      else{ // image has to be placed full hight
         rx=alpha_/alpha; // <= 1
         ry=1.;
      }
   }
   Z nx,ny;
   nx=cpmrnd(R(getCol())*rx);
   ny=cpmrnd(R(getLin())*ry);
      // Number of Graph-pixels to be used for rhe
      // image; in x-direction and y-direction respectively.
      // These numbers determine the pixel loops
   cpmassert(nx>0,loc);
   cpmassert(ny>0,loc);
   R dpx=mx*1./nx, dpy=my*1./ny;
   R dpxh=dpx*0.5, dpyh=dpy*0.5;
   Z i1,i2{0},j1,j2;
   R px{0.},py{0.},qx{0.},qy{0.},dx{0.},dy{0.},w1x{0.},
      w2x{0.},w1y{0.},w2y{0.};
   if (matrixstyle){
      for (i=0,px=dpxh;i<nx;i++,px+=dpx){
         i1=cpmtoz(px);
         if (i1==mx) i1--;
         if (method!=0){
            qx=i1+0.5;
            dx=px-qx;
            w2x=cpmabs(dx); // <= 1
            w1x=R(1)-w2x;
            i2=dx>=0 ? i1+1 : i1-1;
         }
         for (j=0,py=dpyh; j<=ny; j++,py+=dpy){
            j1=cpmtoz(py);
            if (j1==my) j1--;
            if (method!=0){
               qy=j1+0.5;
               dy=py-qy;
               w2y=cpmabs(dy); // <=1
               w1y=R(1)-w2y;
               j2=dy>=0 ? j1+1 : j1-1;
               C v11=mat.pi(i1).pi(j1); // i1,j1 always in range
               C v12,v21,v22;
               if (periodic){
                  v12=mat.pi(i1).cyc(j2);
                  v21=mat.cyc(i2).pi(j1);
                  v22=mat.cyc(i2).cyc(j2);
               }
               else{
                  v12=mat.pi(i1).con(j2);
                  v21=mat.con(i2).pi(j1);
                  v22=mat.con(i2).con(j2);
               }
               R w11=w1x*w1y, w12=w1x*w2y, w21=w2x*w1y, w22=w2x*w2y;
               C z=v11*w11+v12*w12+v21*w21+v22*w22;
               img.putPel(j,i,Color(z,rangeAct).toRGB(bw));
            }
            else{
               C z=mat.pi(i1).pi(j1);
               img.putPel(j,i,Color(z,rangeAct).toRGB(bw));
            }
         }
      }
   } // matrixstyle
   else{ // !matrixstyle
      for (i=0,px=dpxh;i<nx;i++,px+=dpx){
         i1=cpmtoz(px);
         if (i1==mx) i1--;
         if (method!=0){
            qx=i1+0.5;
            dx=px-qx;
            w2x=cpmabs(dx); // <= 1
            w1x=R(1)-w2x;
            i2=dx>=0 ? i1+1 : i1-1;
         }
         for (j=ny-1,py=dpyh; j>=0; j--,py+=dpy){
            j1=cpmtoz(py);
            if (j1==my) j1--;
            if (method!=0){
               qy=j1+0.5;
               dy=py-qy;
               w2y=cpmabs(dy); // <=1
               w1y=R(1)-w2y;
               j2=dy>=0 ? j1+1 : j1-1;
               C v11=mat.pi(i1).pi(j1); // i1,j1 always in range
               C v12,v21,v22;
               if (periodic){
                  v12=mat.pi(i1).cyc(j2);
                  v21=mat.cyc(i2).pi(j1);
                  v22=mat.cyc(i2).cyc(j2);
               }
               else{
                  v12=mat.pi(i1).con(j2);
                  v21=mat.con(i2).pi(j1);
                  v22=mat.con(i2).con(j2);
               }
               R w11=w1x*w1y, w12=w1x*w2y, w21=w2x*w1y, w22=w2x*w2y;
               C z=v11*w11+v12*w12+v21*w21+v22*w22;
               img.putPel(i,j,Color(z,rangeAct).toRGB(bw));
            }
            else{
               C z=mat.pi(i1).pi(j1);
               img.putPel(i,j,Color(z,rangeAct).toRGB(bw));
            }
         }
      }
   } // !matrixstyle
   CPM_MZ
   return res;
}

} // namespace
