//? cpmrecordhandler.cpp
//? C+- by Ulrich Mutze. Status of work 2022-05-06.
//? Copyright (c) 2022 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//? 
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//? 
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for 
//? more details.


#include <cpmrecordhandler.h>
#include <cpmsystem.h>
#include <cpmfile.h>
#include <cpmapplication.h>

using CpmRoot::Z;
using CpmRoot::N;
using CpmRoot::R;
using CpmRoot::Word;
using CpmRootX::B;
using CpmRootX::RecordHandler;
using CpmRootX::Record;
using CpmRootX::sup;
using CpmSystem::IFileStream;
using CpmSystem::glue;
using CpmSystem::Message;
using CpmArrays::V;
using namespace std;

Word CpmApplication::iniFile_("defaultinifile");

V<Word> CpmRootX::combine(V<Word> const& argsC, V<Word> const& argsF)
{
   if (!argsC.valInd(1)) return V<Word>();
   if (!argsF.valInd(1)) return argsC;
   V<Word> argsIni=argsF.prepend(argsC[1]);
   Z nMax=sup<Z>(argsIni.dim(),argsC.dim());
   V<Word> res(nMax);
   for (Z i=res.b();i<=res.e();++i){
      res[i]= argsC.valInd(i) ? argsC[i] : argsIni[i];
   }
   return res;
}

V<Word> CpmRootX::extComLine(int argc, char *argv[])
{
   Z mL=1;
   Word loc("CpmRootX::extComLine(int,*char[])");
   CPM_MA
   V<Word> args=CpmArrays::comLine(argc, argv);
   cpmassert(args.valInd(1),loc);
   Word prog=args[1].baseName();
      // Using '.baseName' is important, since args_[1] carries the
      // full path. Consider as an example the C+- program vqm2. Here prog
      // may read vqm2MS, or vqm2CPM_R, or vqm2CPM_RLONG, or vqm2CPM_RMP.
      // So one should also remove the suffixes which the user may have
      // added to the executables name in order to allow comparison of
      // various compilers or compiler options. For this to work it is
      // important that the program name proper does not contain uppercase
      // letters at its end. I once had a program name graphicsTest which
      // made me wonder where the request for a file graphics.ini came
      // from. Since the migration to Code::Blocks and Linux I allow
      // program names such as smallExperiments which is OK, since here
      // the isolated upper case letter is followed by a lower case one.
   Word iniFile=prog.remApp()&".ini";
 //  RecordHandler rch(iniFile,"","CpmRootX::extComLine(int, *char[])");
   RecordHandler rch(iniFile,Message::getInpDir(),
      "CpmRootX::extComLine(int, *char[])", true); // silent = true
   if (!rch.isVal()){
      cpmwarning(loc&
         ": RecordHandler not valid, commandline not extended");
      CPM_MZ
      return args;
      // Thus no action if file fn does not exist
   }
   CpmApplication::iniFile_=iniFile;
// Now we have a valid RecordHandler to get data from
   V<Word> argsIni=rch.comLine();
   V<Word> res=combine(args,argsIni);
   CPM_MZ
   return res;
}

namespace{
   Word makeIni(Word const& w)
   // appends the extension ".ini" if w is a filename without an extension
   {
      Word res(w);
      Word p(".");
      if (w.find(p)==0) res&=".ini";
      return res;
   }
}

void RecordHandler::ini_(V<Word> const& files, Word const& path,
                         Word const& callMes, bool silent)
{
   Z mL=1;
   cout<<"RecordHandler::ini_(...) started"<<endl;
   std::filesystem::path ad_(std::filesystem::current_path());
   cout<<"Current path is "<<ad_<<endl;
   Word path1=Message::getInpDir();
   Word path2=path;
   Word pathAct =  path2;
   cout<<"preliminary message:"<<endl;
   cout<<"files="<<files<<" path="<<pathAct<<" callMessage="
   <<callMes<<endl;
   ostringstream ost;
   ost<<"files="<<files<<" path="<<pathAct<<" callMessage="
   <<callMes<<endl;
   Word mes(ost);
   cpmmessage("path1 = "&path1);
   cpmmessage("path2 = "&path2);
   Word loc="RecordHandler::ini_("&mes&")";
      // probably long, so not use too often
   CPM_MA
      // it is important to see the arguments

   Z i,fd=files.dim();
   V<Word> ffs(fd);
   for (i=1;i<=fd;++i) ffs[i]=glue(pathAct,makeIni(files[i]));
   bool binary=true;
   i=1;
   while(!valid && i<=fd){ // B valid is data member,
         // thus initialized as false
      valid=rec.readFromFile(ffs[i],binary);
      filename=ffs[i];
      cpmdebug(filename);
      i++;
   }
   if (!valid){
      cpmwarning(" could not read any of the "&cpm(fd)&" files");
      if (size>1) cpmwarning("nothing sent to other processes");
   }
   else{
      if (size>1 || mL<=1){
         if (size>1) cpmmessage(mL,"Rank 1 to all other processes:");
         else{
            if (!silent){
               cpmmessage(mL,"this is what we got:");
               ostringstream ost;
               rec.writeFormatted(ost,false);
               cpmmessage(mL,Word(ost));
            }
         }
      }
   }
   cpmmessage(": valid="&cpm(valid));
   CPM_MZ
   cout<<"RecordHandler::ini_(...) done"<<endl;
}

///////////////////////// read ///////////////////////////////////////////

bool RecordHandler::read(const Word& section, const Word& key,
   Z& value, Z pedantic)const
{
   bool b=rec.get(section,key,value,pedantic);
      // creates a warning if does not find
      // Z changed only if get successful
   if (b==false && pedantic>0){
      ostringstream ost;
      ost<<string("RecordHandler::read(): not able to read a Z")<<
      string(" in section=")<<section.toStr()<<string(" at key=")<<
      key.toStr();
      // very similar to the warning from get. But good to have
      // the text here too since it comes to the status line
      if (pedantic==2) cpmerror(Word(ost.str()));
   }
   return b;
}

bool RecordHandler::read(const Word& section, const Word& key,
   N& value, Z pedantic)const
{
   bool b=rec.get(section,key,value,pedantic);
      // creates a warning if does not find
      // Z changed only if get successful
   if (b==false && pedantic>0){
      ostringstream ost;
      ost<<string("RecordHandler::read(): not able to read a N")<<
      string(" in section=")<<section.toStr()<<string(" at key=")<<
      key.toStr();
      // very similar to the warning from get. But good to have
      // the text here too since it comes to the status line
      if (pedantic==2) cpmerror(Word(ost.str()));
   }
   return b;
}


bool RecordHandler::read(const Word& section, const Word& key,
   R& value, Z pedantic)const
{
   bool b=rec.get(section,key,value,pedantic);
   if (b==false && pedantic>0){
      ostringstream ost;
      ost<<string("RecordHandler::read(): not able to read a R")<<
      string(" in section=")<<section.toStr()<<string(" at key=")<<
      key.toStr();
      if (pedantic==2) cpmerror(Word(ost.str()));
   }
   return b;
}

bool RecordHandler::read(const Word& section, const Word& key,
   Word& value, Z pedantic)const
{
   bool b=rec.get(section,key,value,pedantic);
   if (b==false && pedantic>0){
      ostringstream ost;
      ost<<string("RecordHandler::read(): not able to read a Word")<<
      string(" in section=")<<section.toStr()<<string(" at key=")<<
      key.toStr();
      if (pedantic==2) cpmerror(Word(ost.str()));
   }
   return b;
}

bool RecordHandler::read(const Word& section, const Word& key,
   B& value, Z pedantic)const
{
   bool b=rec.get(section,key,value,pedantic);
   if (b==false && pedantic>0){
      ostringstream ost;
      ost<<string("RecordHandler::read(): not able to read a B")<<
      string(" in section=")<<section.toStr()<<string(" at key=")<<
      key.toStr();
      if (pedantic==2) cpmerror(Word(ost.str()));
   }
   return b;
}

bool RecordHandler::read(const Word& section, const Word& key,
   V<B>& value, Z pedantic)const
{
   bool b=rec.get(section,key,value,pedantic);
   if (b==false && pedantic>0){
      ostringstream ost;
      ost<<string("RecordHandler::read(): not able to read Bs")<<
      string(" in section=")<<section.toStr()<<string(" at key=")<<
      key.toStr();
      if (pedantic==2) cpmerror(Word(ost.str()));
   }
   return b;
}

bool RecordHandler::read(const Word& section, const Word& key,
   V<Z>& value, Z pedantic)const
{
   bool b=rec.get(section,key,value,pedantic);
   if (b==false && pedantic>0){
      ostringstream ost;
      ost<<string("RecordHandler::read(): not able to read Zs")<<
      string(" in section=")<<section.toStr()<<string(" at key=")<<
      key.toStr();
      if (pedantic==2) cpmerror(Word(ost.str()));
   }
   return b;
}

bool RecordHandler::read(const Word& section, const Word& key,
   V<R>& value, Z pedantic)const
{
   bool b=rec.get(section,key,value,pedantic);
   if (b==false && pedantic>0){
      ostringstream ost;
      ost<<string("RecordHandler::read(): not able to read Rs")<<
      string(" in section=")<<section.toStr()<<string(" at key=")<<
      key.toStr();
      if (pedantic==2) cpmerror(Word(ost.str()));
   }
   return b;
}

bool RecordHandler::read(const Word& section, const Word& key,
   V<Word>& value, Z pedantic)const
{
   bool b=rec.get(section,key,value,pedantic);
   if (b==false && pedantic>0){
      ostringstream ost;
      ost<<string("RecordHandler::read(): not able to read Ws")<<
      string(" in section=")<<section.toStr()<<string(" at key=")<<
      key.toStr();
      if (pedantic==2) cpmerror(Word(ost.str()));
   }
   return b;
}

///////////////////////// write ////////////////////////////////////////

bool RecordHandler::write(const Word& section, const Word& key,
   const Z& value)
{
   bool b=rec.set(section,key,value);
   if (b==false)
      cpmerror("RecordHandler::write(Z): failed at "&section&" "&key);
   return b;
}


bool RecordHandler::write(const Word& section, const Word& key,
   const R& value)
{
   bool b=rec.set(section,key,value);
   if (b==false)
      cpmerror("RecordHandler::write(R): failed at "&section&" "&key);
   return b;
}

bool RecordHandler::write(const Word& section, const Word& key,
   const Word& value)
{
   bool b=rec.set(section,key,value);
   if (b==false)
      cpmerror("RecordHandler::write(Word): failed at "&section&" "&key);
   return b;
}

bool RecordHandler::write(const Word& section, const Word& key,
   const B& value)
{
   //bool bVal=value;
   //bool b=rec.set(section,key,bVal);
   bool b=rec.set(section,key,value);
   if (b==false)
      cpmerror("RecordHandler::write(B): failed at "&section&" "&key);
   return b;
}

bool RecordHandler::write(const Word& section, const Word& key,
   const V<B>& value)
{
   bool b=rec.set(section,key,value);
   if (b==false)
      cpmerror("RecordHandler::write(V<B>): failed at "&section&
         " "&key);
   return b;
}

bool RecordHandler::write(const Word& section, const Word& key,
   const V<Z>& value)
{
   bool b=rec.set(section,key,value);
   if (b==false)
      cpmerror("RecordHandler::write(V<Z>): failed at "&section&" "&key);
   return b;
}

bool RecordHandler::write(const Word& section, const Word& key,
   const V<R>& value)
{
   bool b=rec.set(section,key,value);
   if (b==false)
      cpmerror("RecordHandler::write(V<R>): failed at "&section&" "&key);
   return b;
}

bool RecordHandler::write(const Word& section, const Word& key,
   const V<Word>& value)
{
   bool b=rec.set(section,key,value);
   if (b==false)
      cpmerror("RecordHandler::write(V<Word>): failed at "
         &section&" "&key);
   return b;
}

bool RecordHandler::writeFormatted(ostream& str,bool writeComments)const
{
   return rec.writeFormatted(str,writeComments);
}

V<Word> RecordHandler::comLine()const
{
   Z mL=1;
   Word loc("RecordHandler::comLine()");
   CPM_MA
   Word sec1("command line"); // preferred style
   Word sec2("command_line"); // for compatibility
   Word key("args");
   V<Word> res;
   bool valid=false;
   Z ped=1; // only warn
   valid=read(sec1,key,res,ped);
   cout<<"res1="<<res<<endl;
   if (valid){
      CPM_MZ
      return res;
   }
   valid=read(sec2,key,res,ped);
   cout<<"res2="<<res<<endl;
   if (valid){
      CPM_MZ
      return res;
   }
   CPM_MZ
   return V<Word>();
}
