//? cpmcameraxcpp.h
//? C+- by Ulrich Mutze. Status of work 2022-05-06.
//? Copyright (c) 2022 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//? 
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//? 
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for 
//? more details.


/*
Description: see cpmcamerax.h
*/

V<C> Camera::captMono(const V<Spc>& vs)const
{
   Z vsd=vs.dim();
   V<C> res(vsd);
   for (Z i=1;i<=vsd;i++){
      res[i]=captureMono(vs[i]);
   }
   return res;
}

T2< V<C> > Camera::capt(const V<Spc>& vs)
{
   Z vsd=vs.dim();
   V<C> res1(vsd);
   V<C> res2(vsd);
   for (Z i=1;i<=vsd;i++){
      T2<C> pcc=capture(vs[i]);
      res1[i]=pcc.c1();
      res2[i]=pcc.c2();
   }
   return T2< V<C> >(res1,res2);
}

VecGraph Camera::captMono(const Skeleton& sk)const
{
   Z skd=sk.dim();

   V<C> vr(skd);
   for (Z i=1;i<=skd;i++){
      vr[i]=captureMono(sk[i]);
   }
   VecGraph res(vr);
   res.set_neg(sk.get_neg());
   return res;
}

T2<VecGraph> Camera::capt(const Skeleton& sk)const
{
   const Z mL=5;
   cpmmessage(mL,"Camera::capt(Skeleton) started");
   Z skd=sk.dim();

   V<C> v1(skd);
   V<C> v2(skd);
   for (Z i=1;i<=skd;i++){
      T2<C> prr=capture(sk[i]);
      v1[i]=prr.c1();
      v2[i]=prr.c2();
   }
   VecGraph res1(v1);
   VecGraph res2(v2);
   res1.set_neg(sk.get_neg());
   res2.set_neg(sk.get_neg());
   cpmmessage(mL,"Camera::capt(Skeleton) done");
   return T2<VecGraph>(res1,res2);
}

VecGraphs Camera::captMono(const WireFrame& wf)const
{
   Z dwf=wf.dim();
   V<VecGraph> vv(dwf);
   for (Z i=1;i<=dwf;i++){
      vv[i]=captMono(wf[i]);
   }
   return VecGraphs(vv);
}

T2<VecGraphs> Camera::capt(const WireFrame& wf)const
{
   Z dwf=wf.dim();
   V<VecGraph> vv1(dwf);
   V<VecGraph> vv2(dwf);
   for (Z i=1;i<=dwf;i++){
      T2<VecGraph> pvv=capt(wf[i]);
      vv1[i]=pvv.c1();
      vv2[i]=pvv.c2();
   }
   return T2<VecGraphs>(VecGraphs(vv1),VecGraphs(vv2));
}

void Camera::mark(const Skeleton& sk, const Color& c)const
{
   const Z mL=3;
   bool stereo=!c.visible()&&is3D()&&stereo_;
   cpmmessage(mL,"Camera::mark(Skeleton,Color) started");
   if (stereo){
      T2< VecGraph> pvv=capt(sk);
      pvv.c1().setColor(getColor1());
      pvv.c2().setColor(getColor2());
      pvv.c1().mark(gr);
      pvv.c2().mark(gr);
   }
   else{
      VecGraph vgr=captMono(sk);
      vgr.setColor(c);
      vgr.mark(gr);
   }
   cpmmessage(mL,"Camera::mark(Skeleton,Color) done");
}

void Camera::mark(Triangle const& tri, Color const& c)const
{
   Z mL=3;
   Word loc("Camera::mark(Triangle,Color)");
   CPM_MA
   bool stereo=!c.visible()&&is3D()&&stereo_;
   Skeleton sk(tri.p1(),tri.p2(),tri.p3());
   if (stereo){
      T2< VecGraph > pvv=capt(sk);
      pvv.c1().setColor(getColor1());
      pvv.c2().setColor(getColor2());
      pvv.c1().markTriAng(gr);
      pvv.c2().markTriAng(gr);
   }
   else{
      VecGraph vgr=captMono(sk);
      vgr.setColor(c);
      vgr.markTriAng(gr);
   }
   CPM_MZ
}


void Camera::mark(const WireFrame& wf, const Color& c)const
{
   Z dwf=wf.dim();
   for (Z i=1;i<=dwf;i++) mark(wf[i],c);
}


void Camera::mark(const RefSys& rs, const Color& c, R lengthOfAxes)const
{
   WireFrame wf=WireFrame::tripod(lengthOfAxes);
   Group g=rs.frmMasSys();
   wf*=g;
   mark(wf,c);
}

void Camera::mark(const SubsetOfSpace& sos, SitRec k,
   const Color& c)const
{
   Z mL=2;
   Word loc(nameOf()&"::mark(SubsetOfSpace,SitRec,Color)");
   CPM_MA
   const R fineFac=0.25;
   const R coarseFac=5;
   const R steepnessLimit=20;
      // factors for setting the method parameters of
      // SubsetOfSpace::hit()
      // in terms of the basic visual resolution as derived from
      // pixel size and/or sos.get_ds().
      // Same factors appear in Camera::lookTo
   R ds=sos.get_ds();
   R length=(k==LOW || k==UPP ? getdX() : getdY() );
      // length of the k-site of the window in object space
   Z nPix=(k==LOW || k==UPP ? dim1() : dim2() );
   Z np=1+cpmrnd(length/ds);
   np=CpmRootX::inf(np,nPix);
   if (np<=1)
      cpmerror("Camera::mark(SubsetOfSpace& ...): np<=1");
   R ds0=length/(np-1);
      // adjusted interval ds ~ ds0, ds0 fraction of length
   C vr2Start=( k==LOW||k==RIG ?
      C(getxRight(),getyLower()) : C(getxLeft(),getyUpper()) );
   V<C> vr2(np);
   vr2[1]=vr2Start;
   C step;
   if (k==LOW) step[1]=-ds0; else if (k==RIG) step[2]=ds0;
      else if (k==UPP) step[1]=ds0; else step[2]=-ds0;
   Z i;
   for (i=2;i<=np;i++) vr2[i]=vr2[i-1]+step;
   V<Spc> spc(np);
   R mag=1;
   for (i=1;i<=np;i++) spc[i]=inverse(vr2[i],mag);
   Vec dir;
   if (k==LOW) dir=v; else if (k==RIG) dir=-h;
     else if (k==UPP) dir=-v; else dir=h;
   R sFine=fineFac*ds0;
   R sCoarse=coarseFac*ds0;
   R dLim=steepnessLimit*ds0;
   std::vector<Spc> valid;
      // list of points to display. Class vector provides fast
      // appending at the end ('push_back')
   for (i=1;i<=np;i++){
      X2<Spc,bool> psz=sos.hit(spc[i],dir,sFine,sCoarse);
      if (psz.c2()){ // then the point is OK
         valid.push_back(psz.c1());
      }
   }
   Z nValid=(Z)valid.size();
   for (i=1;i<nValid;i++){
      Spc sa=valid[i-1];
      Spc sb=valid[i];
      R len=(sb-sa).abs();
      bool ok=(len<dLim);
         // notice that the coordinate difference in scan direction
         // of adjacent points is just ds0. So we give no OK for
         // steep steps in projection direction.
         // If such steps happen to be real, then they become
         // regularly imaged for a new value of k
       if (ok)mark(sa,sb,c);
   }
   CPM_MZ
}

void Camera::markCen(const SubsetOfSpace& sos, Z nP, const Color& c)const
{
   Z mL=2;
   Word loc(nameOf()&"::markCen(SubsetOfSpace,Z,Color)");
   CPM_MA
   R ds=sos.get_ds();
   R dr=sos.get_dr();
   cpmassert(ds>0,loc);
   if (dr==0){
      dr=ds*10;
      cpmwarning(loc&": dr was 0, set to "&cpm(dr));
   }
   Spc sc=get_x();
   R dPhi=cpm2pi/nP;
   Vec dir=h;
   X2<Spc,bool> spb=sos.hit(sc,dir,ds,dr);
   Spc spFirst=spb.c1();
   bool succ=spb.c2();
   cpmassert(succ==true,loc);
   R phi=0;
   for (Z i=1;i<=nP;++i){
      phi+=dPhi;
      dir=h*cpmcos(phi)+v*cpmsin(phi);
      spb=sos.hit(sc,dir,ds,dr);
      Spc spAct=spb.c1();
      succ=spb.c2();
      cpmassert(succ==true,loc);
      mark(spFirst,spAct,c);
      spFirst=spAct;
   }
   CPM_MZ
}

void Camera::markFan(const SubsetOfSpace& sos, Spc const& p,
   Vec const& e, AxVec const& ax, R ang, Z nP, Color const& c )const
{
   Z mL=2;
   Word loc(nameOf()&"::markFan(...)");
   CPM_MA
   R r=sos.get_r();

   if (r==0){
      cpmwarning(loc&": sos.r==0 not useful, nothing done");
      CPM_MZ
      return;
   }
   if (nP<1){
      cpmwarning(loc&": np="&cpm(nP)&" not useful, nothing done");
      CPM_MZ
      return;
   }
   Vec dir=e;
   R dd=dir.nor_();
   if (dd==0.){
      cpmwarning(loc&": no direction defined, nothing done");
      CPM_MZ
      return;
   }
   R ds=sos.get_ds();
   R dr=sos.get_dr();
   cpmassert(ds>0,loc);
   if (dr==0){
      dr=ds*10;
      cpmwarning(loc&": dr was 0, set to "&cpm(dr));
   }
   X2<Spc,bool> spb;
   bool succ;
   if (ang==0.){ // only one direction, thus mark a point
         // (and not a line)
      cpmwarning(loc&": angle is zero, have to generate only one point");\

      spb=sos.hit(p,dir,ds,dr);
      succ=spb.c2();
      cpmassert(succ==true,loc&" at ang==0.");
      if (succ) mark(spb.c1(),c);
      CPM_MZ
      return;
   }
   AxVec axis=ax;
   R aa=axis.nor_();
   if (aa==0.){ // only one direction, thus mark a point
         // (and not a line)
      cpmwarning(loc&
         ": no valid rotation axis, have to generate only one point");
      spb=sos.hit(p,dir,ds,dr);
      succ=spb.c2();
      cpmassert(succ==true,loc&" at aa==0.");
      if (succ) mark(spb.c1(),c);
      CPM_MZ
      return;
   }
   Angle dPhi(ang/nP,RAD);
   AxVec rot(dPhi,axis);
   spb=sos.hit(p,dir,ds,dr);
   Spc spFirst=spb.c1();
   succ=spb.c2();
   cpmassert(succ==true,loc&" before i-loop");
   for (Z i=1;i<=nP;++i){
      dir*=rot;
      spb=sos.hit(p,dir,ds,dr);
      Spc spAct=spb.c1();
      succ=spb.c2();
      cpmassert(succ==true,loc&" i="&cpm(i));
      mark(spFirst,spAct,c);
      spFirst=spAct;
   }
   CPM_MZ
}

void Camera::rot(const AxVec& axv)
{
   Group g(poi,axv);
   operator *=(g);
}

RefSys Camera::getRefSys()const
{
   Z d=Spc::dimension();
   RefSys master;
   V<Vec> b1=master.getVectors();
   V<Vec> b2=getVectors();
   Vec a=poi-Spc();
   AxVec axv(b1,b2,false); // bases are OK, have not to be rebuilt
   Group g(axv,a);
   return master*=g;
}

bool Camera::isVisible( const Spc& x)const
{
   C px=captureMono(x);
   return gr.indFunc(R2(px));
}

Camera::Camera(RecordHandler const& rch, Word const& sec,
   Frame& fr)
{
   const Z mL=2;
   Word loc(nameOf()&"::Camera(RecordHandler,Word,Frame)");
   cpmmessage(mL,loc&" started");
   R fieldOfViewHeight=1, fieldOfViewCenterX=0;
   R fieldOfViewCenterY=0, fieldOfViewCenterZ=0;
   R tiltDeg=0, panDeg=0, rollDeg=0, shiftX=0, shiftY=0;
   R viewingAngleDegree=20, convergenceAngleDegree=3.5, frac=1,
      aspectRatio=1;
   R responseGamma=-1,responseLowerThreshold=0,responseUpperThreshold=1;
   Z yPixel=400,responseMode=6;
   B writeOnShow;
   B isMono;
   cpmrhf(isMono); // influence only in 3D
   // notice:
   //     #define cpmrhf(X) rch.read(sec,#X,X,1)
   // which thus depends on rch and sec
   Z ped=1;
// so one needs not to read any of the following quantities from file
   cpmrhf(fieldOfViewHeight);
   cpmrhf(fieldOfViewCenterX);
   cpmrhf(fieldOfViewCenterY);
   cpmrhf(fieldOfViewCenterZ);
   cpmrhf(shiftX);
   cpmrhf(shiftY);
   cpmrhf(tiltDeg);
   cpmrhf(panDeg);
   cpmrhf(rollDeg);
   cpmrhf(viewingAngleDegree);
   cpmrhf(convergenceAngleDegree);
   cpmrhf(frac);
   cpmrhf(aspectRatio);
   cpmrhf(writeOnShow);
      // entry not available in ini-files older than 03-01-23
   cpmrhf(yPixel);
   if (yPixel<1){ // 'deactivates' the camera if it is
         // introduced to a program by calling this constructor
      *this=Camera();
      return;
   }
   cpmrhf(responseMode);
   cpmrhf(responseGamma);
   cpmrhf(responseLowerThreshold);
   cpmrhf(responseUpperThreshold);

   fr.cut(aspectRatio,yPixel); // this places fr

   Angle viewingAngle(viewingAngleDegree);
   Angle conv(convergenceAngleDegree);
   *this=Camera(fieldOfViewHeight,aspectRatio,viewingAngle,frac,conv,fr);
   rsp.setGamma(responseGamma);
   rsp.setMode(responseMode);
   rsp.setRangeX(responseLowerThreshold,responseUpperThreshold);
   rsp.setRangeY(0,255);
   shift(fieldOfViewCenterX, fieldOfViewCenterY, fieldOfViewCenterZ);
   shiftGate(shiftX,shiftY);
   pan(Angle(panDeg));
   tilt(Angle(tiltDeg));
   roll(Angle(rollDeg));
   if (isMono || is2D()) mono_();
   else stereo_=B(true);
   V<Z> rgbBackground;
   bool succ=cpmrhf(rgbBackground);
   if (succ) color3=Color(rgbBackground);
   if (!stereo_) gr.setColors();
      // colors are bodycolors, 'film is assumed to be virginal'
      // i.e. no background has to be taken into account.
      // colors get simply set
   else gr.stereo(); // in 3D always fast stereo
   gr.setWriteOnShow(writeOnShow);
   gr.setColorActive(color3);
   gr.paint(color3);
   if (cpmverbose>=mL){
      ostringstream ost;
      ost<<loc<<" done: "<<"dim1="<<gr.dim1()<<" dim2="<<gr.dim2();
      cpmmessage(ost);
   }
}

void Camera::setScreenArea( const Graph& g)const
{
   Graph gTemp=g;
   gTemp.getObjectSpaceFrom(gr);
   gr=gTemp;
}

V<Spc> Camera::grid(Z ny)const
{
   Z mL=3;
   Word loc("Camera::grid(Z)");
   cpmmessage(mL,loc&" started");
   Z nx=cpmtoz(R(ny)*gr.getAspRat());
   Iv ivx=gr.getIvX();
   Iv ivy=gr.getIvY();
   V<R> vx=ivx.centers(nx);
   V<R> vy=ivy.centers(ny);
   V<Spc> res(nx*ny);
   Z i,j;
   C xij;
   Z count=0;
   for (i=1;i<=nx;i++){
      xij[1]=vx[i];
      for (j=1;j<=ny;j++){
         xij[2]=vy[j];
         R magFac=1.;
         res[++count]=inverse(xij,magFac);
      }
   }
   cpmmessage(mL,loc&" done");
   return res;
}

void Camera::lookTo(const SubsetOfSpace& sos, Angle tilt, Angle pan,
   const Texture& tx, const RefSys& rs, bool bodyFix, R fitFactor)
{
   dirTo_(sos,fitFactor,tilt,pan,rs);
   show(sos,tx,bodyFix);
}
