//? cpmlineartcpp.h
//? C+- by Ulrich Mutze. Status of work 2022-05-06.
//? Copyright (c) 2022 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//? 
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//? 
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for 
//? more details.


using namespace CpmRoot;
using namespace CpmImaging;
using namespace CpmArrays;

using CPM_DIM::Vec;
using CPM_DIM::AxVec;
using CPM_DIM::Spc;
using CPM_DIM::Group;
using CPM_DIM::Rdim;
using CPM_DIM::Rdim_;
using CPM_DIM::Skeleton;
using CPM_DIM::Triangle;
using CPM_DIM::Path;
using CPM_DIM::IndCon;

using CPM_CAMERA::Camera;

using CpmGeo::Angle;

/////////////////////////////// class Wrt ////////////////////////////////

V<Vec> Wrt::bas()const
{
   Vec ex=e1();
   Vec ey=e2();
   Z n=Spc::dimension();
   V<Vec> res(n);
   res[1]=ex;
   res[2]=ey;
   if (n==3) res[3]=Vec(ex.cross(ey));
      // cross directly gives an AxVec
   return res;
}

Wrt& Wrt::operator *=(Group const& g)
{
   for (Z i=v_.b(); i<=v_.e(); ++i) v_[i]*=g;
   f_*=g;
   l_*=g;
   u_*=g;
   return *this;
}

Wrt& Wrt::operator +=(Vec const& vec)
{
   for (Z i=v_.b(); i<=v_.e(); ++i) v_[i]+=vec;
   f_+=vec;
   l_+=vec;
   u_+=vec;
   return *this;
}

Wrt& Wrt::operator *=(AxVec const& av)
{
   for (Z i=v_.b(); i<=v_.e(); ++i) v_[i].rot_(f_,av);
   l_=l_.rot_(f_,av);
   u_=u_.rot_(f_,av);
   return *this;
}

Wrt& Wrt::operator *=(R scl)
{
   for (Z i=v_.b(); i<=v_.e(); ++i) v_[i].scl_(f_,scl);
   h_*=scl;
   w_*=scl;
   d_*=scl;
   l_.scl_(f_,scl);
   u_.scl_(f_,scl);
   return *this;
}

Wrt& Wrt::scl_(Spc const& fix, R fac)
{
   for (Z i=v_.b(); i<=v_.e(); ++i) v_[i].scl_(fix,fac);
   h_*=fac;
   w_*=fac;
   d_*=fac;
   f_.scl_(fix,fac); // was missing
   l_.scl_(fix,fac);
   u_.scl_(fix,fac);
   return *this;
}

Wrt& Wrt::thc_(R r, Z n)
{
   if (r<=0 || n<=1) return *this;
   Spc o;
   Path disk=Path::circle(o,r,n);
   V<Spc> cir=disk();
   Z nc=cir.dim(); // paranoia, should be n
   V<Vec> vcir(nc);
   Z i,j;
   for (i=1;i<=nc;++i) vcir[i]=cir[i]-o;
   V<Path> vThick=v_;
   Z np=v_.dim();
   for (j=1;j<=nc;++j){
      V<Path> vj=v_;
      Vec sj=vcir[j];
      for (i=1; i<=np; ++i) vj[i]+=sj;
      vThick&=vj;
   }
   v_=vThick;
   Vec sh(r*2,0);
   l_+=sh;
   u_+=sh;
   h_+=r; // height over base line grows only by r not 2r
   w_+=r*2;
   d_+=r;
   return *this;
}

Wrt& Wrt::to_(Spc const& pp)
{
   Vec shift=pp-f_;
   f_+=shift;
   l_+=shift;
   u_+=shift;
   for (Z i=v_.b(); i<=v_.e(); ++i) v_[i]+=shift;
   return *this;
}

bool Wrt::prnOn(ostream& str)const
{
   cpmwat;
   cpmp(v_);
   cpmp(f_);
   cpmp(l_);
   cpmp(u_);
   cpmp(h_);
   cpmp(w_);
   cpmp(d_);
   return true;
}

bool Wrt::scanFrom(istream& str)
{
   cpms(v_);
   cpms(f_);
   cpms(l_);
   cpms(u_);
   cpms(h_);
   cpms(w_);
   cpms(d_);
   return true;
}

void Wrt::mark(Camera const& ca, Color const& c,  bool points)const
{
   for (Z i=v_.b();i<=v_.e();++i) ca.mark(v_[i],c);
   Color cb(GREEN);
   if (points) ca.mark(box(),cb);
}

/********* This was a hard lesson in C++. Problem finally solved by
// using the debugger. With the MS Visual C++ 2008 Express Edition there
// is a debugger available that works for my graphical C+- programs.
// Nice!
// void Wrt::mark(Camera const& ca, Color const& c,  bool points)const
// this gives widely black field but does not so if Color(BLUE)
// and Color(CYAN) are set as arguments. An automatic conversion from
// an enumeration to a class (here Palette to Color) seems not to
// follow he rules which are valid for conversions from class to class.
// Aha! There is a function Camera::mark(Spc,R) and this is chosen
// since C++ prefers the conversion int->double over user-defined
// conversions, such as my enum->Color!
// What looks natural for mere int is not at all natural for enumerations
// which are carefully protected against getting converted to other
// enumerations.
// This is not a wise design decision in C++ in my opinion, but this
// is the fact! Also gcc did it the same way.
{
   Z mL=2;
   Word loc("Wrt::(Camera,Color,bool)");
   CPM_MA
   if (points){
      cpmtrigger=1;
      cpmmessage("trigger set");
      Spc p1=fir(),p2=upp();
      ca.mark(p1,BLUE);
         // ca.mark(p1,Color(BLUE)); does the expected thing
      cpmtrigger=0;
      ca.mark(p2,CYAN);
   }
   for (Z i=v_.b();i<=v_.e();++i) ca.mark(v_[i],c);
   CPM_MZ
}
*************/

Wrt& Wrt::vec_()
{
   Vec ex=e1();
   Vec ey=e2();
   R hv=h_*1.2; // 1.2 adjustable parameter
   Spc p1=f_+ey*hv;
   Spc p2=p1+ex*w_;
   V<Path> vs(2);
   Vec dir=ex*(-2)+ey; // -2 adjustable parameter
   dir.nor_();
   dir*=(w_*0.3); // 0.3 adjustable parameter
   vs[1]=Path(p1,p2);
   vs[2]=Path(p2,p2+dir);
   v_&=vs;
   R dh=(dir|ey);
   h_=hv+dh;
   u_=f_+ex*w_+ey*h_;
   return *this;
}

Wrt& Wrt::dot_()
{
   R dh=h_*0.275;
   Spc cen=uppMid();
   cen+=e2()*dh;
   R r1=dh*0.4;
   v_&=Path::circle(cen,r1);
   return *this;
}

Wrt& Wrt::ddot_()
{
   R dh=h_*0.275;
   Vec ex=e1(),ey=e2();
   Spc cen=uppMid();
   cen+=ey*dh;
   R r1=dh*0.4;
   Vec sh=ex*(r1*1.5);
   Spc cen1=cen-sh, cen2=cen+sh;
   v_&=Path::circle(cen1,r1);
   v_&=Path::circle(cen2,r1);
   return *this;
}

Wrt& Wrt::sub_(V<Wrt> const& index, IndCon const& ic)
{
   V<Vec> b=bas();
   Vec sl=spc();
   LineArt la=LineArt::text(index);
   Spc fpi=la.cw().fir().fir();
   la.scl_(fpi,ic.fac);
   R dh=h_*ic.dh, dw=w_*ic.dw;
   Spc con=last()+b[1]*dw+b[2]*dh;
   Vec shift=con-fpi;
   la+=shift;
   V<Wrt> vw=la.cw();
   for (Z i=vw.b();i<=vw.e();++i) v_&=vw[i].v();
   Spc lpi=la.cw().last().rigLowCor();
   w_=((lpi-f_)|b[1]);
   l_=f_+w_*b[1]+sl;
   // u_ unchanged, to allow for upper indexes to be appended
   // after lower ones
   return *this;
}

Wrt& Wrt::sup_(V<Wrt> const& index, IndCon const& ic)
{
   V<Vec> b=bas();
   Vec sl=spc();
   LineArt la=LineArt::text(index);
   Spc fpi=la.cw().fir().fir();
   la.scl_(fpi,ic.fac);
   R dh=h_*ic.dh, dw=w_*ic.dw;
   Spc con=rigUppCor()+b[1]*dw+b[2]*dh+sl;
   Vec shift=con-fpi;
   la+=shift;
   V<Wrt> vw=la.cw();
   for (Z i=vw.b();i<=vw.e();++i) v_&=vw[i].v();
   Spc upi=la.cw().last().rigUppCor();
   u_=upi;
   Vec uv=u_-f_;
   R w_=(uv|b[1]);
   R h_=(uv|b[2]);
   l_=f_+b[1]*w_+sl;
   return *this;
}

/// what follows is a gigantic function which defines some vector font ///

Wrt::Wrt(Word const& name)
{
   Word loc("Wrt(Word)");
   Spc o(0,0);
   Vec e1(1,0);
   Vec e2(0,1);
   R uR=1./7;
   AxVec rot180(Angle(180,DEG),3);
   AxVec rot90(Angle(90,DEG),3);
   AxVec rot45(Angle(45,DEG),3);
   AxVec rot90m(Angle(-90,DEG),3);
   R w,h;
   R wi=1,wj=2,wr=3,wa=4,wm=7;
   R wI=1,wJ=3,wL=4,wB=5,wA=6,wW=7;
   R dA=2;
   R hA=7, ha=5, dg=2;
   R wPeriod=1; // width of period and 'i-tüpfelchen'
   if (name=="A"){
      w=wA;
      h=hA;
      R hr=0.3;
      v_=V<Path>(3);
      f_=o;
      l_=o+Vec(w+dA,0);
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      Spc top(w*0.5,h);
      Spc foot(w,0);
      Spc m1=o.versus(top,hr);
      Spc m2=foot.versus(top,hr);
      v_[1]=Path(o,top);
      v_[2]=Path(m1,m2);
      v_[3]=Path(top,foot);
   }
   if (name=="Au"){
      Wrt au("A");
      w=wA;
      R w1=w*0.3;
      R w2=w*0.7;
      h=hA;
      R r=wPeriod*0.5;
      Spc pc1(w1,h+wPeriod);
      Spc pc2(w2,h+wPeriod);
      V<Path> add(2);
      add[1]=Path::circle(pc1,r);
      add[2]=Path::circle(pc2,r);
      au&=add;
      *this=au;
      h_+=wPeriod*1.5;
      u_=Spc(w_,h_);
   }
   else if (name=="B"){
      w=wB;
      h=hA;
      R hr=0.5;
      v_=V<Path>(6);
      f_=o;
      l_=o+Vec(w+dA,0);
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      R horB=w-h*0.25;
      Spc t1(0,h);
      Spc m1=o.versus(t1,hr);
      Vec hor(horB,0);
      Spc t2=t1+hor;
      Spc m2=m1+hor;
      Spc f2=o+hor;
      v_[1]=Path(o,t1);
      v_[2]=Path(o,f2);
      v_[3]=Path(m1,m2);
      v_[4]=Path(t1,t2);
     // v_[5]=Path::arc(f2,f2.cen(m2),-rot180);
     // v_[6]=Path::arc(m2,m2.cen(t2),-rot180);
      v_[5]=Path::arc(f2,f2.cen(m2),rot180);
      v_[6]=Path::arc(m2,m2.cen(t2),rot180);
         // corrected 2012-10-18
   }
   else if (name=="C"){
      w=wB;
      h=hA;
      R l1=0.3;
      R l2=0.7;
      v_=V<Path>(5);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      R h1=h*l1, h2=h*l2, w1=w*0.5, w2=w;
      Spc p1(0,h1);
      Spc p2(0,h2);
      v_[1]=Path(p1,p2);
      Spc p3(w1,h);
      v_[2]=Path::quarterEllipse(p2,e2,p3);
      Spc p4(w2,h2);
      v_[3]=Path::quarterEllipse(p3,e1,p4);
      Spc q1(w1,0);
      v_[4]=Path::quarterEllipse(p1,-e2,q1);
      Spc q2(w2,h1);
      v_[5]=Path::quarterEllipse(q1,e1,q2);
   }
   else if (name=="D"){
      w=wB;
      h=hA;
      v_=V<Path>(6);
      f_=o;
      l_=Spc(w+dA,0);
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      Spc t0(0,h);
      v_[1]=Path(o,t0);
      R dr=0.2;
      R d=dr*h;
      R r=(h-d)*0.5;
      R dz=w-r;
      cpmassert(dz>0,"Wr(D)");
      Spc a1(dz,h);
      Spc c1(dz,h-r);
      Spc m1(w,h-r);
      v_[2]=Path::arc(a1,c1,rot90m);
      Spc a2(dz,0);
      Spc c2(dz,r);
      Spc m2(w,r);
      v_[3]=Path::arc(a2,c2,rot90);
      v_[4]=Path(m1,m2);
      v_[5]=Path(o,a2);
      v_[6]=Path(t0,a1);
   }
   else if (name=="E"){
      w=wB;
      h=hA;
      R rm=0.75;
      v_=V<Path>(4);
      f_=o;
      l_=Spc(w+dA,0);
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      Spc t(0,h);
      v_[1]=Path(o,t);
      Vec h(w,0);
      Vec hm(w*rm,0);
      Spc t1=t+h;
      Spc f1=o+h;
      Spc m=o.versus(t,0.5);
      v_[2]=Path(o,f1);
      v_[3]=Path(t,t1);
      v_[4]=Path(m,m+hm);
   }
   else if (name=="F"){
      w=wB;
      h=hA;
      R rm=0.75;
      v_=V<Path>(3);
      f_=o;
      l_=Spc(w+dA,0);
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      Spc t(0,h);
      v_[1]=Path(o,t);
      Vec h(w,0);
      Vec hm(w*rm,0);
      Spc t1=t+h;
      Spc m=o.versus(t,0.5);
      v_[2]=Path(t,t1);
      v_[3]=Path(m,m+hm);
   }
   else if (name=="G"){
      w=wB;
      h=hA;
      v_=V<Path>(8);
      f_=o;
      R tW=w+dA;
      l_=o+e1*tW;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      R l2=0.7;
      R l1=0.3;
      Spc t(0,h);
      Spc m1=o.versus(t,l1);
      Spc m2=o.versus(t,l2);
      Vec w1=e1*w;
      v_[1]=Path(m1,m2);
      Spc m3=m1+w1;
      Spc m4=m2+w1;
      Vec w2=w1*0.5;
      Spc f2=o+w2;
      Spc t2=t+w2;
      v_[3]=Path::quarterEllipse(m2,e2,t2);
      v_[4]=Path::quarterEllipse(t2,e1,m4);
      v_[5]=Path::quarterEllipse(m1,-e2,f2);
      v_[6]=Path::quarterEllipse(f2,e1,m3);
      Spc m5(w,h_*0.5);
      v_[7]=Path(m3,m5);
      Spc m6=m5-e1*(w*0.4);
      v_[8]=Path(m5,m6);
   }
   else if (name=="H"){
      w=wB;
      h=hA;
      v_=V<Path>(3);
      f_=o;
      R tH=w+dA;
      l_=o+e1*tH;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      Spc t(0,h);
      Vec v=e1*w;
      Spc m1=o.versus(t,0.5);
      v_[1]=Path(o,t);
      v_[2]=Path(o+v,t+v);
      v_[3]=Path(m1,m1+v);
   }
   else if (name=="I"){
      w=wI;
      h=hA;
      v_=V<Path>(1);
      f_=o;
      R tI=w+dA;
      l_=o+e1*tI;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      Vec v=e1*(wI*0.5);
      Spc t(0,h);
      v_[1]=Path(o+v,t+v);
   }
   else if (name=="J"){
      w=wJ;
      h=hA;
      R h1r=0.2;
      R wJh=w*0.5;
      v_=V<Path>(3);
      f_=o;
      R tJ=w+dA;
      l_=o+e1*tJ;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      R h1=h*h1r;
      Spc p1(0,h1);
      Spc p2(wJh,0);
      Spc p3(w,h1);
      Spc p4(w,h);
      v_[1]=Path::quarterEllipse(p1,-e2,p2);
      v_[2]=Path::quarterEllipse(p2,e1,p3);
      v_[3]=Path(p3,p4);
   }
   else if (name=="K"){
      w=wB;
      h=hA;
      R h1=0.45;
      R h2=0.25;
      v_=V<Path>(3);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      Spc p1(o);
      Spc p2(0,h);
      Spc p3=p1.versus(p2,h1);
      Spc p4(w,h);
      Spc p5=p3.versus(p4,h2);
      Spc p6(w,0);
      v_[1]=Path(p1,p2);
      v_[2]=Path(p3,p4);
      v_[3]=Path(p5,p6);
   }
   else if (name=="L"){
      w=wL;
      h=hA;
      v_=V<Path>(2);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      Spc p1(o);
      Spc p2(0,h);
      Spc p3(wL,0);
      v_[1]=Path(p1,p2);
      v_[2]=Path(p1,p3);
   }
   else if (name=="M"){
      w=wA;
      h=hA;
      R hr=0.4;
      h_=h; w_=w; u_=Spc(w_,h_);
      R h1=h_*hr;
      v_=V<Path>(4);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      d_=0;
      Spc p1(o);
      Spc p2(0,h);
      Spc p3(w*0.5,h1);
      Spc p4(w,h);
      Spc p5(w,0);
      v_[1]=Path(p1,p2);
      v_[2]=Path(p2,p3);
      v_[3]=Path(p3,p4);
      v_[4]=Path(p4,p5);
   }
   else if (name=="N"){
      w=wB;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(3);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;

      Spc p1(o);
      Spc p2(0,h);
      Spc p3(w,0);
      Spc p4(w,h);

      v_[1]=Path(p1,p2);
      v_[2]=Path(p2,p3);
      v_[3]=Path(p3,p4);
   }
   else if (name=="O"){
      w=wB;
      h=hA;
      v_=V<Path>(6);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      R l2=0.75;
      R l1=0.25;
      Spc t(0,h);
      Spc m1=o.versus(t,l1);
      Spc m2=o.versus(t,l2);
      Vec w1=e1*w;
      v_[1]=Path(m1,m2);
      Spc m3=m1+w1;
      Spc m4=m2+w1;
      v_[2]=Path(m3,m4);
      Vec w2=w1*0.5;
      Spc f2=o+w2;
      Spc t2=t+w2;
      v_[3]=Path::quarterEllipse(m2,e2,t2);
      v_[4]=Path::quarterEllipse(t2,e1,m4);
      v_[5]=Path::quarterEllipse(m1,-e2,f2);
      v_[6]=Path::quarterEllipse(f2,e1,m3);
         // quarterEllipse works as expected
   }
   if (name=="Ou"){
      Wrt au("O");
      w=wB;
      R w1=w*0.3;
      R w2=w*0.7;
      h=hA;
      R r=wPeriod*0.5;
      Spc pc1(w1,h+wPeriod);
      Spc pc2(w2,h+wPeriod);
      V<Path> add(2);
      add[1]=Path::circle(pc1,r);
      add[2]=Path::circle(pc2,r);
      au&=add;
      *this=au;
      h_+=wPeriod*1.5;
      u_=Spc(w_,h_);
   }
   else if (name=="P"){
      w=wB;
      h=hA;
      R hr=0.5;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      R sP=w-h*(R(1)-hr)*0.5;
      cpmassert(sP>0,loc);
      v_=V<Path>(4);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      Vec v=e1*sP;

      Spc p1(o);
      Spc p2(0,h);
      Spc p3=p1.versus(p2,hr);
      Spc p4=p2+v;
      Spc p5=p3+v;

      v_[1]=Path(p1,p2);
      v_[2]=Path(p2,p4);
      v_[3]=Path(p3,p5);
      //v_[4]=Path::arc(p5,p4.cen(p5),-rot180);
      v_[4]=Path::arc(p5,p4.cen(p5),rot180);
         // corrected 2012-10-18
   }
   else if (name=="Q"){
      w=wB;
      h=hA;
      v_=V<Path>(7);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      R l2=0.75;
      R l1=0.25;
      Spc t(0,h);
      Spc m1=o.versus(t,l1);
      Spc m2=o.versus(t,l2);
      Vec w1=e1*w;
      v_[1]=Path(m1,m2);
      Spc m3=m1+w1;
      Spc m4=m2+w1;
      v_[2]=Path(m3,m4);
      Vec w2=w1*0.5;
      Spc f2=o+w2;
      Spc t2=t+w2;
      v_[3]=Path::quarterEllipse(m2,e2,t2);
      v_[4]=Path::quarterEllipse(t2,e1,m4);
      v_[5]=Path::quarterEllipse(m1,-e2,f2);
      v_[6]=Path::quarterEllipse(f2,e1,m3);
      R hx=l1*h;
      v_[7]=Path(Spc(w,0),Spc(w-hx,hx) );
   }
   else if (name=="R"){
      w=wB;
      h=hA;
      R hr=0.5;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      R sR=w-h*(R(1)-hr)*0.5;
      cpmassert(sR>0,loc);
      v_=V<Path>(5);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      Vec v=e1*sR;

      Spc p1(o);
      Spc p2(0,h);
      Spc p3=p1.versus(p2,hr);
      Spc p4=p2+v;
      Spc p5=p3+v;

      v_[1]=Path(p1,p2);
      v_[2]=Path(p2,p4);
      v_[3]=Path(p3,p5);
      //v_[4]=Path::arc(p5,p4.cen(p5),-rot180);
      v_[4]=Path::arc(p5,p4.cen(p5),rot180);
      v_[5]=Path(p5,Spc(w,0));
   }
   else if (name=="S"){
      w=wB;
      h=hA;
      R h1r=0.25;
      R h2r=0.5;
      R h3r=0.75;
      h_=h; w_=w; u_=Spc(w_,h_);
      R h1=h1r*h;
      R h2=h2r*h;
      R h3=h3r*h;
      R w1=w*0.5;
      d_=0;
      v_=V<Path>(6);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      Vec v=e1*w;
      Vec vh=v*0.5;
      Spc p1(0,h3);
      Spc p2(w1,h);
      v_[1]=Path::quarterEllipse(p1,e2,p2);
      Spc p3(w,h3);
      v_[2]=Path::quarterEllipse(p2,e1,p3);
      Spc q1(w1,h2);
      v_[3]=Path::quarterEllipse(p1,-e2,q1);
      Spc q2(w,h1);
      v_[4]=Path::quarterEllipse(q1,e1,q2);
      Spc q3(w1,0);
      v_[5]=Path::quarterEllipse(q2,-e2,q3);
      Spc q4(0,h1);
      v_[6]=Path::quarterEllipse(q3,-e1,q4);
   }
   else if (name=="T"){
      w=wB;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(2);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;

      R w1=w*0.5;

      Spc p1(w1,0);
      Spc p2(w1,h);
      Spc p3(0,h);
      Spc p4(w,h);

      v_[1]=Path(p1,p2);
      v_[2]=Path(p3,p4);
   }
   else if (name=="U"){
      w=wB;
      h=hA;
      R hr=0.25;
      h_=h; w_=w; u_=Spc(w_,h_);
      R h1=h*hr;
      d_=0;
      v_=V<Path>(4);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;

      R w1=w*0.5;

      Spc p1(0,h);
      Spc p2(0,h1);
      v_[1]=Path(p1,p2);
      Spc p3(w1,0);
      v_[2]=Path::quarterEllipse(p2,-e2,p3);
      Spc p4(w,h1);
      v_[3]=Path::quarterEllipse(p3,e1,p4);
      Spc p5(w,h);
      v_[4]=Path(p4,p5);
   }
   if (name=="Uu"){
      Wrt au("U");
      w=wB;
      R w1=w*0.3;
      R w2=w*0.7;
      h=hA;
      R r=wPeriod*0.5;
      Spc pc1(w1,h+wPeriod);
      Spc pc2(w2,h+wPeriod);
      V<Path> add(2);
      add[1]=Path::circle(pc1,r);
      add[2]=Path::circle(pc2,r);
      au&=add;
      *this=au;
      h_+=wPeriod*1.5;
      u_=Spc(w_,h_);
   }
   else if (name=="V"){
      w=wB;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(2);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;

      R w1=w*0.5;

      Spc p1(0,h);
      Spc p2(w1,0);
      v_[1]=Path(p1,p2);
      Spc p3(w,h);
      v_[2]=Path(p2,p3);
   }
   else if (name=="W"){
      w=wW;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(4);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      R w1=w*0.25;
      R w2=w*0.5;
      R w3=w*0.75;
      Spc p1(0,h);
      Spc p2(w1,0);
      v_[1]=Path(p1,p2);
      Spc p3(w2,h);
      v_[2]=Path(p2,p3);
      Spc p4(w3,0);
      v_[3]=Path(p3,p4);
      Spc p5(w,h);
      v_[4]=Path(p4,p5);
   }
   else if (name=="X"){
      w=wB;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(2);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;

      Spc p1(0,0);
      Spc p2(0,h);
      Spc p3(w,0);
      Spc p4(w,h);
      v_[1]=Path(p1,p4);
      v_[2]=Path(p2,p3);
   }
   else if (name=="Y"){
      w=wB;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(3);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;

      R w1=w*0.5;
      R h1=h*0.5;

      Spc p1(0,h);
      Spc p2(w1,h1);
      v_[1]=Path(p1,p2);
      Spc p3(w,h);
      v_[2]=Path(p2,p3);
      Spc p4(w1,0);
      v_[3]=Path(p2,p4);
   }
   else if (name=="Z"){
      w=wB;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(3);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;

      Spc p1(0,0);
      Spc p2(0,h);
      Spc p3(w,0);
      Spc p4(w,h);
      v_[1]=Path(p2,p4);
      v_[2]=Path(p4,p1);
      v_[3]=Path(p1,p3);
   }
   else if (name=="a"){
      w=wa;
      h=ha;
      R h1r=0.3, h2r=0.6;
      R wre=0.2;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(7);
      f_=o;
      R h1=h1r*h;
      R h2=h2r*h;
      R w1=w*0.5;
      R we=w*(wre+1);
      R wl=we+dA;
      l_=o+e1*wl;
      Spc p1(0,h1);
      Spc p2(0,h2);
      v_[1]=Path(p1,p2);
      Spc p3(w1,h);
      v_[2]=Path::quarterEllipse(p2,e2,p3);
      Spc p4(w,h2);
      v_[3]=Path::quarterEllipse(p3,e1,p4);
      Spc q1(w1,0);
      v_[4]=Path::quarterEllipse(p1,-e2,q1);
      Spc q2(w,h1);
      v_[5]=Path::quarterEllipse(q1,e1,q2);
      Spc q3(w,h);
      v_[6]=Path(q3,q2);
      Spc q4(we,0);
      v_[7]=Path::quarterEllipse(q2,-e2,q4);
   }
   if (name=="au"){
      Wrt au("a");
      w=wa;
      R w1=w*0.3;
      R w2=w*0.7;
      h=ha;
      R r=wPeriod*0.5;
      Spc pc1(w1,hA-r);
      Spc pc2(w2,hA-r);
      V<Path> add(2);
      add[1]=Path::circle(pc1,r);
      add[2]=Path::circle(pc2,r);
      au&=add;
      *this=au;
      h_=hA;
      u_=Spc(w_,h_);
   }
   else if (name=="b"){
      w=wa;
      h=ha;
      R h1r=0.3, h2r=0.6;
      h_=hA;
      d_=0;
      v_=V<Path>(6);
      f_=o;
      w_=w; u_=Spc(w_,h_);
      R wl=w+dA;
      l_=o+e1*wl;

      R h1=h1r*h;
      R h2=h2r*h;
      R w1=w*0.5;

      Spc p1(0,h_);
      Spc p2(0,0);
      v_[1]=Path(p1,p2);
      Spc p3(0,h2);
      Spc p4(w1,h);
      v_[2]=Path::quarterEllipse(p3,e2,p4);
      Spc p5(w,h2);
      v_[3]=Path::quarterEllipse(p4,e1,p5);
      Spc p6(w,h1);
      v_[4]=Path(p5,p6);
      Spc p7(w1,0);
      v_[5]=Path::quarterEllipse(p6,-e2,p7);
      Spc p8(0,h1);
      v_[6]=Path::quarterEllipse(p7,-e1,p8);
   }
   else if (name=="c"){
      w=wa;
      h=ha;
      R h1r=0.3, h2r=0.6;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(5);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;

      R h1=h1r*h;
      R h2=h2r*h;
      R w1=w*0.5;

      Spc p1(0,h1);
      Spc p2(0,h2);
      v_[1]=Path(p1,p2);
      Spc p3(w1,h);
      v_[2]=Path::quarterEllipse(p2,e2,p3);
      Spc p4(w,h2);
      v_[3]=Path::quarterEllipse(p3,e1,p4);
      Spc q1(w1,0);
      v_[4]=Path::quarterEllipse(p1,-e2,q1);
      Spc q2(w,h1);
      v_[5]=Path::quarterEllipse(q1,e1,q2);
   }
   else if (name=="d"){
      w=wa;
      h=ha;
      R h1r=0.3, h2r=0.6;
      R wre=0.2;
      h_=hA;
      d_=0;
      w_=w; u_=Spc(w_,h_);
      v_=V<Path>(7);
      f_=o;
      R h1=h1r*h;
      R h2=h2r*h;
      R w1=w*0.5;
      R we=w*(wre+1);
      R wl=we+dA;
      l_=o+e1*wl;
      Spc p1(0,h1);
      Spc p2(0,h2);
      v_[1]=Path(p1,p2);
      Spc p3(w1,h);
      v_[2]=Path::quarterEllipse(p2,e2,p3);
      Spc p4(w,h2);
      v_[3]=Path::quarterEllipse(p3,e1,p4);
      Spc q1(w1,0);
      v_[4]=Path::quarterEllipse(p1,-e2,q1);
      Spc q2(w,h1);
      v_[5]=Path::quarterEllipse(q1,e1,q2);
      Spc q3(w,h_);
      v_[6]=Path(q3,q2);
      Spc q4(we,0);
      v_[7]=Path::quarterEllipse(q2,-e2,q4);
   }
   else if (name=="e"){
      w=wa;
      h=ha;
      R h1r=0.3, h2r=0.5;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(6);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;

      R h1=h1r*h;
      R h2=h2r*h;
      R w1=w*0.5;

      Spc p1(0,h1);
      Spc p2(0,h2);
      v_[1]=Path(p1,p2);
      Spc p3(w1,h);
      v_[2]=Path::quarterEllipse(p2,e2,p3);
      Spc p4(w,h2);
      v_[3]=Path::quarterEllipse(p3,e1,p4);
      Spc q1(w1,0);
      v_[4]=Path::quarterEllipse(p1,-e2,q1);
      Spc q2(w,h1);
      v_[5]=Path::quarterEllipse(q1,e1,q2);
      Spc p5(0,h2);
      v_[6]=Path(p4,p5);
   }
   else if (name=="f"){
      w=wr;
      h=hA;
      R h1=ha, h2r=0.85;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(3);
      f_=o;
      R h2=h2r*h;
      R w1=w*0.5;
      R wl=w+dA;
      l_=o+e1*wl;
      Spc p1(0,h1);
      Spc p2(w,h1);
      v_[1]=Path(p1,p2);
      Spc p3(w1,0);
      Spc p4(w1,h2);
      v_[2]=Path(p3,p4);
      Spc p5(w,h);
      v_[3]=Path::quarterEllipse(p4,e2,p5);
   }
   else if (name=="g"){
      w=wa;
      h=ha;
      R h1r=0.3, h2r=0.6;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=dg;
      v_=V<Path>(8);
      f_=o;
      R h1=h1r*h;
      R h2=h2r*h;
      R w1=w*0.5;
      R wl=w+dA;
      l_=o+e1*wl;
      Spc p1(0,h1);
      Spc p2(0,h2);
      v_[1]=Path(p1,p2);
      Spc p3(w1,h);
      v_[2]=Path::quarterEllipse(p2,e2,p3);
      Spc p4(w,h2);
      v_[3]=Path::quarterEllipse(p3,e1,p4);
      Spc q1(w1,0);
      v_[4]=Path::quarterEllipse(p1,-e2,q1);
      Spc q2(w,h1);
      v_[5]=Path::quarterEllipse(q1,e1,q2);
      Spc q3(w,h);
      R ys=-d_*0.4;
      Spc q4(w,ys);
      v_[6]=Path(q3,q4);
      Spc q5(w1,-d_);
      v_[7]=Path::quarterEllipse(q4,-e2,q5);
      Spc q6(0,ys);
      v_[8]=Path::quarterEllipse(q5,-e1,q6);

   }
   else if (name=="h"){
      w=wa;
      h=ha;
      R h2r=0.6;
      h_=hA;
      d_=0;
      w_=w; u_=Spc(w_,h_);
      v_=V<Path>(4);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      R h2=h2r*h;
      R w1=w*0.5;

      Spc p1(0,h_);
      Spc p2(0,0);
      v_[1]=Path(p1,p2);
      Spc p3(0,h2);
      Spc p4(w1,h);
      v_[2]=Path::quarterEllipse(p3,e2,p4);
      Spc p5(w,h2);
      v_[3]=Path::quarterEllipse(p4,e1,p5);
      Spc p6(w,0);
      v_[4]=Path(p5,p6);
   }
   else if (name=="i"){
      w=wi;
      h=ha;
      h_=hA;
      d_=0;
      w_=w; u_=Spc(w_,h_);
      v_=V<Path>(2);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;

      R w1=w*0.5;
      R r=wPeriod*0.5;

      Spc p1(w1,h);
      Spc p2(w1,0);
      v_[1]=Path(p1,p2);
      Spc cen(w1,h_-r);
      v_[2]=Path::circle(cen,r);
   }
   else if (name=="j"){
      w=wj;
      h_=hA;
      h=ha;
      d_=dg;
      v_=V<Path>(3);
      f_=o;
      w_=w; u_=Spc(w_,h_);
      R wl=w+dA;
      l_=o+e1*wl;

      R w1=w*0.5;
      R r=wPeriod*0.5;

      Spc p1(w-r,h);
      Spc p2(w-r,0);
      v_[1]=Path(p1,p2);
      Spc cen(w-r,h_-r);
      v_[2]=Path::circle(cen,r);

      Spc p3(0,-d_);
      v_[3]=Path::quarterEllipse(p2,-e2,p3);
   }
   else if (name=="k"){
      w=wa;
      R h1=0.14;
      R h2=0.5;
      v_=V<Path>(3);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      h_=hA;
      h=ha;
      d_=0;
      w_=w; u_=Spc(w_,h_);
      Spc p1(o);
      Spc p2(0,h_);
      Spc p3=p1.versus(p2,h1);
      Spc p4(w,h);
      Spc p5=p3.versus(p4,h2);
      Spc p6(w,0);
      v_[1]=Path(p1,p2);
      v_[2]=Path(p3,p4);
      v_[3]=Path(p5,p6);
   }
   else if (name=="l"){
      w=wj;
      h_=hA;
      h=hA;
      d_=0;
      v_=V<Path>(2);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      w_=w; u_=Spc(w_,h_);

      R w1=w*0.5;
      R h1=h*0.14;

      Spc p1(0,h);
      Spc p2(0,h1);
      v_[1]=Path(p1,p2);
      Spc p3(w,0);
      v_[2]=Path::quarterEllipse(p2,-e2,p3);
   }
   else if (name=="m"){
      w=wm;
      R hr=0.75;
      h=ha;
      d_=0;
      v_=V<Path>(7);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      h_=h; w_=w; u_=Spc(w_,h_);

      R w1=w*0.25;
      R w2=w*0.5;
      R w3=w*0.75;
      R h1=h*hr;

      Spc p1(0,0);
      Spc p2(0,h);
      v_[1]=Path(p1,p2);
      Spc q1(0,h1);
      Spc q2(w1,h);
      v_[2]=Path::quarterEllipse(q1,e2,q2);
      Spc q3(w2,h1);
      v_[3]=Path::quarterEllipse(q2,e1,q3);
      Spc q4(w3,h);
      v_[4]=Path::quarterEllipse(q3,e2,q4);
      Spc q5(w,h1);
      v_[5]=Path::quarterEllipse(q4,e1,q5);
      Spc f1(w2,0);
      v_[6]=Path(q3,f1);
      Spc f2(w,0);
      v_[7]=Path(q5,f2);
   }
   else if (name=="n"){
      w=wa;
      R hr=0.75;
      h=ha;
      d_=0;
      v_=V<Path>(4);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      h_=h; w_=w; u_=Spc(w_,h_);

      R w1=w*0.5;
      R h1=h*hr;

      Spc p1(0,0);
      Spc p2(0,h);
      v_[1]=Path(p1,p2);
      Spc q1(0,h1);
      Spc q2(w1,h);
      v_[2]=Path::quarterEllipse(q1,e2,q2);
      Spc q3(w,h1);
      v_[3]=Path::quarterEllipse(q2,e1,q3);
      Spc f1(w,0);
      v_[4]=Path(q3,f1);
   }
   else if (name=="o"){
      w=wa;
      h=ha;
      d_=0;
      v_=V<Path>(1);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      h_=h; w_=w; u_=Spc(w_,h_);
      R h1=h*0.5;
      R w1=w*0.5;
      Spc pc(w1,h1);
      Path el=Path::ellipse(pc,w1,h1,32);
      v_[1]=el;
   }
   if (name=="ou"){
      Wrt au("o");
      w=wa;
      R w1=w*0.3;
      R w2=w*0.7;
      h=ha;
      R r=wPeriod*0.5;
      Spc pc1(w1,hA-r);
      Spc pc2(w2,hA-r);
      V<Path> add(2);
      add[1]=Path::circle(pc1,r);
      add[2]=Path::circle(pc2,r);
      au&=add;
      *this=au;
      h_=hA;
      u_=Spc(w_,h_);
   }
   else if (name=="p"){
      w=wa;
      R h1r=0.3, h2r=0.6;
      h_=ha;
      h=h_;
      d_=dg;
      v_=V<Path>(6);
      f_=o;
      R h1=h1r*h;
      R h2=h2r*h;
      R w1=w*0.5;
      R wl=w+dA;
      l_=o+e1*wl;
      w_=w; u_=Spc(w_,h_);

      Spc q1(0,h);
      Spc q2(0,-d_);
      v_[1]=Path(q1,q2);

      Spc p1(0,h2);
      Spc p2(w1,h);
      v_[2]=Path::quarterEllipse(p1,e2,p2);
      Spc p3(w,h2);
      v_[3]=Path::quarterEllipse(p2,e1,p3);
      Spc p4(w,h1);
      v_[4]=Path(p3,p4);
      Spc p5(w1,0);
      v_[5]=Path::quarterEllipse(p4,-e2,p5);
      Spc p6(0,h1);
      v_[6]=Path::quarterEllipse(p5,-e1,p6);
   }
   else if (name=="q"){
      w=wa;
      w_=w;
      R h1r=0.3, h2r=0.6;
      h_=ha;
      u_=Spc(w_,h_);
      h=h_;
      d_=dg;
      v_=V<Path>(6);
      f_=o;
      R h1=h1r*h;
      R h2=h2r*h;
      R w1=w*0.5;
      R wl=w+dA;
      l_=o+e1*wl;
      Spc q1(w,h);
      Spc q2(w,-d_);
      v_[1]=Path(q1,q2);
      Spc p1(0,h2);
      Spc p2(w1,h);
      v_[2]=Path::quarterEllipse(p1,e2,p2);
      Spc p3(w,h2);
      v_[3]=Path::quarterEllipse(p2,e1,p3);
      Spc p4(w,h1);
      Spc p5(w1,0);
      v_[4]=Path::quarterEllipse(p4,-e2,p5);
      Spc p6(0,h1);
      v_[5]=Path::quarterEllipse(p5,-e1,p6);
      v_[6]=Path(p1,p6);
   }
   else if (name=="r"){
      w=wr;
      R h1r=0.6;
      h_=ha;
      h=h_;
      d_=0;
      v_=V<Path>(2);
      f_=o;
      R h1=h1r*h;
      R wl=w+dA;
      l_=o+e1*wl;
      w_=w; u_=Spc(w_,h_);
      Spc q1(0,0);
      Spc q2(0,h);
      v_[1]=Path(q1,q2);
      Spc p1(0,h1);
      Spc p2(w,h);
      v_[2]=Path::quarterEllipse(p1,e2,p2);
   }
   else if (name=="s"){
      w=wa;
      R h1r=0.25;
      R h2r=0.5;
      R h3r=0.75;
      h_=ha;
      R h1=h1r*h_;
      R h2=h2r*h_;
      R h3=h3r*h_;
      R w1=w*0.5;
      d_=0;
      v_=V<Path>(6);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      w_=w; u_=Spc(w_,h_);
      Vec v=e1*w;
      Vec vh=v*0.5;
      Spc p1(0,h3);
      Spc p2(w1,h_);
      v_[1]=Path::quarterEllipse(p1,e2,p2);
      Spc p3(w,h3);
      v_[2]=Path::quarterEllipse(p2,e1,p3);
      Spc q1(w1,h2);
      v_[3]=Path::quarterEllipse(p1,-e2,q1);
      Spc q2(w,h1);
      v_[4]=Path::quarterEllipse(q1,e1,q2);
      Spc q3(w1,0);
      v_[5]=Path::quarterEllipse(q2,-e2,q3);
      Spc q4(0,h1);
      v_[6]=Path::quarterEllipse(q3,-e1,q4);
   }
   else if (name=="t"){
      w=wr;
      R h1=ha, h2r=0.15;
      h_=hA;
      h=h_;
      d_=0;
      v_=V<Path>(3);
      f_=o;
      R h2=h2r*h;
      R w1=w*0.5;
      R wl=w+dA;
      l_=o+e1*wl;
      w_=w; u_=Spc(w_,h_);
      Spc p1(0,h1);
      Spc p2(w,h1);
      v_[1]=Path(p1,p2);
      Spc p3(w1,h);
      Spc p4(w1,h2);
      v_[2]=Path(p3,p4);
      Spc p5(w,0);
      v_[3]=Path::quarterEllipse(p4,-e2,p5);
   }
   else if (name=="u"){
      w=wa;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      R h1r=0.3;
      R wre=0.2;
      d_=0;
      v_=V<Path>(5);
      f_=o;
      R h1=h1r*h_;
      R w1=w*0.5;
      R we=w*(wre+1);
      R wl=we+dA;
      l_=o+e1*wl;
      Spc p1(0,h1);
      Spc p2(0,h);
      v_[1]=Path(p1,p2);
      Spc q1(w1,0);
      v_[2]=Path::quarterEllipse(p1,-e2,q1);
      Spc q2(w,h1);
      v_[3]=Path::quarterEllipse(q1,e1,q2);
      Spc q3(w,h_);
      v_[4]=Path(q3,q2);
      Spc q4(we,0);
      v_[5]=Path::quarterEllipse(q2,-e2,q4);
   }
   if (name=="uu"){
      Wrt au("u");
      w=wa;
      R w1=w*0.3;
      R w2=w*0.7;
      h=ha;
      R r=wPeriod*0.5;
      Spc pc1(w1,hA-r);
      Spc pc2(w2,hA-r);
      V<Path> add(2);
      add[1]=Path::circle(pc1,r);
      add[2]=Path::circle(pc2,r);
      au&=add;
      *this=au;
      h_=hA;
      u_=Spc(w_,h_);
   }
   else if (name=="v"){
      w=wa;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(2);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      R w1=w*0.5;
      Spc p1(0,h_);
      Spc p2(w1,0);
      v_[1]=Path(p1,p2);
      Spc p3(w,h_);
      v_[2]=Path(p2,p3);
   }
   else if (name=="w"){
      w=wm;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(4);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      R w1=w*0.25;
      R w2=w*0.5;
      R w3=w*0.75;
      Spc p1(0,h);
      Spc p2(w1,0);
      v_[1]=Path(p1,p2);
      Spc p3(w2,h);
      v_[2]=Path(p2,p3);
      Spc p4(w3,0);
      v_[3]=Path(p3,p4);
      Spc p5(w,h);
      v_[4]=Path(p4,p5);
   }
   else if (name=="x"){
      w=wa;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(2);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      Spc p1(0,0);
      Spc p2(0,h);
      Spc p3(w,0);
      Spc p4(w,h);
      v_[1]=Path(p1,p4);
      v_[2]=Path(p2,p3);
   }
   else if (name=="y"){
      w=wa;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=dg;
      v_=V<Path>(3);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;
      R w1=w*0.5;
      Spc p1(0,h);
      Spc p2(w1,0);
      v_[1]=Path(p1,p2);
      Spc p3(w,h);
      v_[2]=Path(p2,p3);
      Spc p4(0,-d_);
      v_[3]=Path(p2,p2-p3,p4,-e1);
   }
   else if (name=="z"){
      w=wa;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(3);
      f_=o;
      R wl=w+dA;
      l_=o+e1*wl;

      Spc p1(0,0);
      Spc p2(0,h);
      Spc p3(w,0);
      Spc p4(w,h);
      v_[1]=Path(p2,p4);
      v_[2]=Path(p4,p1);
      v_[3]=Path(p1,p3);
   }
   else if (name=="n1"){
      w=wj;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(2);
      R hr=0.8;
      R h1=h*hr;

      Spc p1(0,h1);
      Spc p2(w,h);
      Spc p3(w,0);
      v_[1]=Path(p1,p2);
      v_[2]=Path(p2,p3);
   }
   else if (name=="n2"){
      w=wa;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      R h1r=R(1)-uR;
      R h2r=R(1)-uR*2;
      R h3r=R(1)-uR*3;
      R h1=h1r*h, h2=h2r*h, h3=h3r*h;
      R w1=w*0.5;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(4);
      Spc p1(0,h1);
      Spc p2(w1,h);
      v_[1]=Path::quarterEllipse(p1,e2,p2);
      Spc p3(w,h2);
      v_[2]=Path::quarterEllipse(p2,e1,p3);
      Spc p4(w,h3);
      Spc p5(0,0);
      v_[3]=Path::bezier(V<Spc>{p3,p4,p5},8);
         // ideal for the present purpose
      Spc p6(w,0);
      v_[4]=Path(p5,p6);
   }
   else if (name=="n3"){
      w=wa;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      R w1=w*0.5;
      R w2=w*0.25;
      R h1=h*0.8;
      R h2=h*0.55;
      R h3=h*0.25;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(6);
      Spc p1(0,h1);
      Spc p2(w1,h);
      v_[1]=Path::quarterEllipse(p1,e2,p2);
      Spc p3(w,h1);
      v_[2]=Path::quarterEllipse(p2,e1,p3);
      Spc p4(w2,h2);
      v_[3]=Path::quarterEllipse(p3,-e2,p4);
      Spc p5(w,h3);
      v_[4]=Path::quarterEllipse(p4,-e1,p5);
      Spc p6(w1,0);
      v_[5]=Path::quarterEllipse(p5,-e2,p6);
      Spc p7(0,h3);
      v_[6]=Path::quarterEllipse(p6,-e1,p7);
   }
   else if (name=="n4"){
      w=wa;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      R h1=h*0.5, h2=h*0.25;
      R w1=w*0.75;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(3);
      Spc p1(w1,h);
      Spc p2(0,h2);
      v_[1]=Path(p1,p2);
      Spc p3(w,h2);
      v_[2]=Path(p2,p3);
      Spc q1(w1,h1);
      Spc q2(w1,0);
      v_[3]=Path(q1,q2);
   }
   else if (name=="n5"){
      w=wa;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      R h1=h*0.5 ;
      R h2=h*0.15 ;
      R w1=w*0.5;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(5);
      Spc p1(w*0.8,h);
      Spc p2(0,h);
      v_[1]=Path(p1,p2);
      Spc p3(0,h1);
      v_[2]=Path(p2,p3);
      Spc p4(w,h*0.25);
      Vec dir1=1.5*e1+e2;
      v_[3]=Path(p3,dir1,p4,-e2);
      Spc p5(w*0.5,0);
      v_[4]=Path::quarterEllipse(p4,-e2,p5);
      Spc p6(0,h2);
      v_[5]=Path::quarterEllipse(p5,-e1,p6);
   }
   else if (name=="n6"){
      w=wa;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      R h1=h*0.8 ;
      R w1=w*0.5;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(4);
      R a=w1;
      R b=ha*0.45;
      Spc pc(a,b);
      v_[1]=Path::ellipse(pc,a,b);
      Spc p1(0,b);
      Spc p2(0,h1);
      v_[2]=Path(p1,p2);
      Spc p3(w1,h);
      v_[3]=Path::quarterEllipse(p2,e2,p3);
      Spc p4(w,h1);
      v_[4]=Path::quarterEllipse(p3,e1,p4);
   }
   else if (name=="n7"){
      w=wa;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      R h1=h*0.8 ;
      R w1=w*0.5;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(3);
      Spc p1(0,h1);
      Spc p2(0,h);
      v_[1]=Path(p1,p2);
      Spc p3(w,h);
      v_[2]=Path(p2,p3);
      v_[3]=Path(p3,o);
   }
   else if (name=="n8"){
      w=wa;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      R h1=h*0.75;
      R h2=h*0.25;
      R w1=w*0.5;
      f_=o;
      l_=Spc(w+dA,0);
      R a=w*0.5;
      R b=h*0.25;
      Spc c1(w1,h1);
      v_=V<Path>(2);
      v_[1]=Path::ellipse(c1,a,b);
      Spc c2(w1,h2);
      v_[2]=Path::ellipse(c2,a,b);
   }
   else if (name=="n9"){
      w=wa;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      R h1=h*0.2 ;
      R w1=w*0.5;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(4);
      R a=w1;
      R b=ha*0.45;
      Spc pc(a,h-b);
      v_[1]=Path::ellipse(pc,a,b);
      Spc p1(w,h-b);
      Spc p2(w,h1);
      v_[2]=Path(p1,p2);
      Spc p3(w1,0);
      v_[3]=Path::quarterEllipse(p2,-e2,p3);
      Spc p4(0,h1);
      v_[4]=Path::quarterEllipse(p3,-e1,p4);
   }
   else if (name=="n0"){
      w=wa;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      R w1=w*0.5;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(1);
      R a=w1;
      R b=h*0.5;
      Spc pc(a,b);
      v_[1]=Path::ellipse(pc,a,b);
   }
   else if (name=="alpha"){
      w=wA;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(5);
      R w1=w*0.33;
      R w2=w*0.66;
      R h1=h*0.5;
      Spc p1(w,h);
      Spc p2(w1,0);
      v_[1]=Path(p1,p2,-e1,0.33);
      Spc p3(0,h1);
      v_[2]=Path::quarterEllipse(p2,-e1,p3);
      Spc p4(w1,h);
      v_[3]=Path::quarterEllipse(p3,e2,p4);
      Spc p5(w2,h1);
      Vec dir=e1-4*e2;
      v_[4]=Path(p4,e1,p5,dir);
      Spc p6(w,0);
      v_[5]=Path(p5,dir,p6,e1);
   }
   else if (name=="beta"){
      w=wa;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=dg;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(6);
      R w1=w*0.5;
      R h1=h*0.5;
      Spc p1(0,-d_);
      Spc p2(w1,h);
      v_[1]=Path(p1,p2,e1,0.8);
      Spc p3(w,h*0.75);
      v_[2]=Path::quarterEllipse(p2,e1,p3);
      Spc p4(w1,h1);
      v_[3]=Path::quarterEllipse(p3,-e2,p4);
      Spc p5(w,h*0.25);
      v_[4]=Path::quarterEllipse(p4,e1,p5);
      Spc p6(w1,0);
      v_[5]=Path::quarterEllipse(p5,-e2,p6);
      Spc p7(w1*0.4,h*0.1);
      v_[6]=Path(p6,-e1,p7,-e1+e2);
   }
   else if (name=="gamma"){
      w=wa;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=dg;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(5);
      R w1=w*0.3;
      R w2=w*0.6;
      R h1=w*0.5;
      Spc p1(0,h*0.8);
      Spc p2(w1,h);
      v_[1]=Path(p1,e1+2*e2,p2,e1);
      Spc p3(w2,0);
      v_[2]=Path(p2,e1,p3,-e2);
      Spc p4(w1,-d_);
      v_[3]=Path(p3,-e2,p4,-e1);
      Spc p5(o);
      v_[4]=Path(p4,-e1,p5,e2);
      Spc p6(w,h);
      v_[5]=Path(p6,p5,-e2);
   }
   else if (name=="delta"){
      w=wa;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(6);
      R w1=w*0.5;
      R w2=w*0.8;
      R h1=ha;
      R h2=h1*0.5;
      Spc p1(w*0.8,h1*0.9); // meeting point
      Spc p2(w,h2);
      Path pa1(p1,p2,-e2,0.5);
      v_[1]=pa1;
      Vec di=pa1.firstDir();
      Spc p0(w,h);
      Path pa2(p1,-di,p0,e1);
      v_[2]=pa2;
      Spc p3(w1,0);
      v_[3]=Path::quarterEllipse(p2,-e2,p3);
      Spc p4(0,h2);
      v_[4]=Path::quarterEllipse(p3,-e1,p4);
      Spc p5(w1,h1);
      v_[5]=Path::quarterEllipse(p4,e2,p5);
      v_[6]=Path(p5,e1,p1,di);
   }
   else if (name=="epsilon"){
      w=wa;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(6);
      R w1=w*0.5;
      R h1=h*0.5;
      Spc p1(w,h*0.75);
      Spc p2(w1,h);
      v_[1]=Path(p1,-e1+2*e2,p2,-e1);
      Spc p3(0,h*0.75);
      v_[2]=Path::quarterEllipse(p2,-e1,p3);
      Spc p4(w1,h1);
      v_[3]=Path::quarterEllipse(p3,-e2,p4);
      Spc p5(0,h*0.25);
      v_[4]=Path::quarterEllipse(p4,-e1,p5);
      Spc p6(w1,0);
      v_[5]=Path::quarterEllipse(p5,-e2,p6);
      Spc p7(w,h*0.25);
      v_[6]=Path(p6,e1,p7,e1+2*e2);
   }
   else if (name=="theta"){
      w=wA;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(6);
      R w1=w*0.5;
      R h1=h*0.5;
      R a=8;
      Vec dir1=e1+2*e2;
      Spc p1(o);
      Spc p2(w1,0);
      R g1=4,g2=1.5;
      v_[1]=Path::cubCur(p1,dir1,g1,p2,e1,g2);
      Spc p3(w1,h);
      g1=1.5; g2=1.5;
      v_[2]=Path::cubCur(p2,e1,g1,p3,-e1,g2);
      Spc p4(w,h*0.8);
      g1=2.5; g2=4;
      v_[3]=Path::cubCur(p3,-e1,g1,p4,e1*5+e2*2,g2);
   }
   else if (name=="phi"){
      w=wA;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=dg;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(6);
      R w1=w*0.5;
      R h1=h*0.5;
      Spc p1(w*0.4,h);
      Spc p2(0,h1);
      v_[1]=Path::quarterEllipse(p1,-e1,p2);
      Spc p3(w1,0);
      v_[2]=Path::quarterEllipse(p2,e2,p3);
      Spc p4(w,h1);
      v_[3]=Path::quarterEllipse(p3,e1,p4);
      Spc p5(w*0.7,h*0.9);
      v_[4]=Path::quarterEllipse(p4,e2,p5);
      Spc p6(w1,-d_);
      v_[5]=Path::quarterEllipse(p5,-e1,p6);

   }
   else if (name=="psi"){
      w=wA;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=dg;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(4);
      R w1=w*0.6;
      Spc p1(0,h*0.8);
      Spc p2(w*0.075,h);
      Vec d1=e1+e2*2.5;
      v_[1]=Path(p1,d1,p2,e1);
      Spc p3(w1,0);
      v_[2]=Path::cubCur(p2,e1,2,p3,e1,2);
      Spc p4(w,h);
      Vec d2=e1+e2*2;
      v_[3]=Path::cubCur(p3,e1,1.75,p4,d2,2);
      v_[4]=Path(Spc(w1,h),Spc(w1,-d_));
   }
   else if (name=="omega"){
      w=wA;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(6);
      R w1=w*0.2;
      R w2=w*0.5;
      R w3=w*0.8;
      R h0=h*0.5;
      R h1=h*0.75;
      Spc p1(w1,h);
      Spc p2(0,h0);
      v_[1]=Path(p1,p2,-e2);
      Spc p3(w*0.25,0);
      v_[2]=Path::quarterEllipse(p2,-e2,p3);
      Spc p4(w2,h1);
      v_[3]=Path(p3,e1,p4,e2);
      Spc p5(w*0.75,0);
      v_[4]=Path(p4,-e2,p5,e1);
      Spc p6(w,h0);
      v_[5]=Path(p5,e1,p6,e2);
      Spc p7(w3,h);
      v_[6]=Path(p7,p6,-e2);
   }
   else if (name=="rho"){
      w=wa;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=dg;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(2);
      R w1=w*0.5;
      R h1=h*0.5;
      v_[1]=Path::ellipse(Spc(w1,h1),w1,h1);
      Spc p1(0,h1);
      Spc p2(w1,-d_);
      v_[2]=Path::cubCur(p1,-e2,2.5,p2,-e1*2.5-e2,2);
   }
   else if (name=="lambda"){
      w=wa;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(3);
      R w1=w*0.5;
      R h1=h*0.5;
      Spc p1(0,h);
      Spc p2(w,0);
      Spc pc(w1,h1);
      v_[1]=Path(pc,p1,-e1,0.5);
      v_[2]=Path(pc,p2,e1,0.5);
      v_[3]=Path(o,pc);
   }
   else if (name=="chi"){
      w=wa;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=dg;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(3);
      Spc p1(0,h);
      Spc p2(0,-d_);
      Spc p3(w,h);
      Spc p4(w,-d_);
      Spc pc=p1.cen(p4);
      v_[1]=Path(pc,p1,-e1,0.5);
      v_[2]=Path(pc,p4,e1,0.5);
      v_[3]=Path(p2,p3);
   }
   else if (name=="sigma"){
      w=wa;
      h=ha*0.9;
      R h2=ha*1.2;
      h_=h2; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(3);
      R w1=w*0.5;
      R h1=h*0.5;
      Spc p1(0,h);
      Spc p2(0,0);
      Spc p3(w,h);
      Spc p4(w,0);
      Spc pc=p2.cen(p3);
      v_[1]=Path::ellipse(pc,w1,h1);
      Spc q1(0,h1);
      Spc q2(w1,h2);
      v_[2]=Path::cubCur(q1,e2,1.5,q2,e1,1.5);
      Spc q3(w,h2);
      v_[3]=Path::cubCur(q2,e1,1.5,q3,e1+e2,1.5);
   }
   else if (name=="mu"){
      w=wA;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=dg;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(6);
      R w1=w*0.5;
      R h1=h*0.5;
      Spc p1(0,-d_);
      R wr=0.20;
      R hr=0.25;
      R w0=wr*w;
      R h0=hr*h;
      R wm=w0+(w-w0)*0.5;
      Spc p2(w0,h0);
      v_[1]=Path(p1,p2,e2);
      Spc p3(w0,h);
      v_[2]=Path(p2,p3);
      Spc p4(wm,0);
      v_[3]=Path::quarterEllipse(p2,-e2,p4);
      Spc p5(w-w0,h0);
      v_[4]=Path::quarterEllipse(p4,e1,p5);
      Spc p6(w-w0,h);
      v_[5]=Path(p5,p6);
      Spc p7(w,0);
      v_[6]=Path::quarterEllipse(p5,-e2,p7);
   }
   else if (name=="nu"){
      w=wA;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(4);
      R wr=0.2;
      R hr=0.85;
      R w0=wr*w;
      R h0=hr*h;
      Spc p1(0,h0);
      Spc p2(w0,h);
      v_[1]=Path(p1,p2,e1);
      R w2=w0*2;
      Spc p3(w2,h0);
      v_[2]=Path::quarterEllipse(p2,e1,p3);
      Spc p4(w2,0);
      v_[3]=Path(p3,p4);
      v_[4]=Path(p4,Spc(w,h),e1+e2*3);
   }
   else if (name=="upsilon"){
      w=wA;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(5);
      R wr=0.2;
      R hr=0.85;
      R w0=wr*w;
      R h0=hr*h;
      Spc p1(0,h0);
      Spc p2(w0,h);
      v_[1]=Path(p1,p2,e1);
      R w2=w0*2;
      R w3=w0*3.25;
      Spc p3(w2,h0);
      v_[2]=Path::quarterEllipse(p2,e1,p3);
      Spc p4(w3,0);
      v_[3]=Path::quarterEllipse(p3,-e2,p4);
      Spc p5(w,h*0.5);
      v_[4]=Path::quarterEllipse(p4,e1,p5);
      Spc p6(w*0.9,h);
      v_[5]=Path(p6,p5,-e2);
   }
   else if (name=="eta"){
      w=wA;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=dg;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(6);
      R wr=0.2;
      R hr=0.75;
      R w0=wr*w;
      R h0=hr*h;
      Spc p1(0,h);
      Spc p2(w0,h0);
      v_[1]=Path(p1,p2,e2);
      Spc p3(w0,0);
      v_[2]=Path(p2,p3);
      R wm=w*0.5;
      Spc p4(wm,h);
      v_[3]=Path::quarterEllipse(p2,e2,p4);
      R wl=w-w0;
      Spc p5(wl,h0);
      v_[4]=Path::quarterEllipse(p4,e1,p5);
      Spc p6(wl,0);
      v_[5]=Path(p5,p6);
      Spc p7(w,-d_);
      v_[6]=Path(p6,p7,e1);
   }
   else if (name=="tau"){
      w=wA;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(4);
      R dh=h*0.15;
      Spc p1(0,h-dh);
      Spc p2(w*0.5,h);
      Spc p3(w,h+dh);
      Vec d1=e1+e2;
      Vec d2=e1;
      v_[1]=Path(p1,d1,p2,d2);
      v_[2]=Path(p2,d2,p3,d1);
      R h2=w*0.25;
      Spc q1(w*0.5,h2);
      Spc q2(w,h2);
      v_[3]=Path(p2,q1);
      //v_[4]=Path::arc(q1,q1.cen(q2),-AxVec(Angle(180,DEG),3));
      v_[4]=Path::arc(q1,q1.cen(q2),AxVec(Angle(180,DEG),3));
   }
   else if (name=="pi"){
      w=wA;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(7);
      R w1=w*0.33;
      R w2=w*0.67;
      R dh=h*0.2;
      R dw=w*0.1;
      R h1=h*0.5;
      Spc p1(0,h-dh);
      Spc p2(w1,h);
      Spc p3(w2,h);
      Spc p4(w,h+dh);
      v_[1]=Path::quarterEllipse(p1,e2,p2);
      v_[2]=Path(p2,p3);
      v_[3]=Path::quarterEllipse(p3,e1,p4);
      Spc q2(w1,h1);
      Spc q3(w2,h1);
      v_[4]=Path(p2,q2);
      v_[5]=Path(p3,q3);
      Spc r2(dw,0);
      Spc r3(w-dw,0);
      v_[6]=Path::quarterEllipse(r2,e1,q2);
      v_[7]=Path::quarterEllipse(r3,-e1,q3);
   }
   else if (name=="zeta"){
      w=wa;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=dg;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(5);
      R w1=w*0.5;
      R dh8=h/R(8);
      R dh4=h/R(4);
      Spc p1(w1,h);
      Spc p2(0,h-dh8);
      v_[1]=Path::quarterEllipse(p1,-e1,p2);
      Spc p3(w,h-dh4);
      v_[2]=Path::quarterEllipse(p2,-e2,p3);
      Spc p4(0,h*0.5);
      v_[3]=Path::quarterEllipse(p3,-e1,p4);
      Spc p5(w,0);
      v_[4]=Path::cubCur(p4,-e2,1.6,p5,-e2,1.25);
      Spc p6(0,-d_);
      v_[5]=Path::quarterEllipse(p5,-e2,p6);
   }
   else if (name=="xi"){
      w=wa;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=dg;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(7);
      R w1=w*0.5;
      R dh8=h/R(9);
      R dh4=dh8*2;
      R dh2=dh4*2;
      Spc p1(w1,h);
      Spc p2(dh8,h-dh8);
      v_[1]=Path::quarterEllipse(p1,-e1,p2);
      Spc p3(w,h-dh4);
      v_[2]=Path::quarterEllipse(p2,-e2,p3);
      Spc q1(dh8,h-dh4-dh8);
      v_[3]=Path::quarterEllipse(p3,-e1,q1);
      Spc q2(w,h-dh2);
      v_[4]=Path::quarterEllipse(q1,-e2,q2);
      Spc p4(0,h*0.5-dh8);
      v_[5]=Path::quarterEllipse(q2,-e1,p4);
      Spc p5(w,0);
      v_[6]=Path::cubCur(p4,-e2,1.6,p5,-e2,1.25);
      Spc p6(0,-d_);
      v_[7]=Path::quarterEllipse(p5,-e2,p6);
   }
   else if (name=="kappa"){
      w=wA;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(5);
      R wr=0.2;
      R hr=0.85;
      R w0=wr*w;
      R h0=hr*h;
      Spc p1(0,h0);
      Spc p2(w0,h);
      v_[1]=Path(p1,p2,e1);
      R w2=w0*2;
      Spc p3(w2,h0);
      v_[2]=Path::quarterEllipse(p2,e1,p3);
      Spc p4(w2,0);
      v_[3]=Path(p3,p4);
      Spc p5=p4.versus(p3,0.5);
      Spc p6(w,h);
      Path pa1(p5,p6,e1);
      v_[4]=pa1;
      Vec d1=pa1.firstDir();
      Spc p7(w,0);
      v_[5]=Path::cubCur(p5,d1,2,p7,e1,2);
   }
   else if (name=="omicron"){
      w=wa;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      R w1=w*0.5;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(1);
      R a=w1;
      R b=h*0.5;
      Spc pc(a,b);
      v_[1]=Path::ellipse(pc,a,b);
   }
   else if (name=="iota"){
      w=wr;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(2);
      R r=w*0.5;
      Spc q0(0,h);
      Spc q1(0,r);
      Spc q2(w,r);
      v_[1]=Path(q0,q1);
      //v_[2]=Path::arc(q1,q1.cen(q2),-AxVec(Angle(180,DEG),3));
      v_[2]=Path::arc(q1,q1.cen(q2),AxVec(Angle(180,DEG),3));
   }
   else if (name=="Gamma"){
      w=wa;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(3);
      Spc p0(o);
      Spc p1(0,h);
      Spc p2(w,h);
      Spc p3(w,h*0.9);
      v_[1]=Path(p0,p1);
      v_[2]=Path(p1,p2);
      v_[3]=Path(p2,p3);
   }
   else if (name=="Lambda"){
      w=wA;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(3);
      R dw=w*0.1;
      R w1=w*0.45;
      Spc p0(o);
      Spc p1(w1,h);
      Spc p2(w-dw,0);
      Spc p3(w,0);
      v_[1]=Path(p0,p1);
      v_[2]=Path(p1,p2);
      v_[3]=Path(p2,p3);
   }
   else if (name=="Delta"){
      w=wA;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(3);
      R dw=w*0.1;
      R w1=w*0.45;
      Spc p0(o);
      Spc p1(w1,h);
      Spc p2(w,0);
      v_[1]=Path(p0,p1);
      v_[2]=Path(p1,p2);
      v_[3]=Path(p2,p0);
   }
   else if (name=="Pi"){
      w=wa;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(3);
      R dw=w*0.1;
      R w1=w*0.45;
      Spc p0(o);
      Spc p1(0,h);
      Spc p2(w,h);
      Spc p3(w,0);
      v_[1]=Path(p0,p1);
      v_[2]=Path(p1,p2);
      v_[3]=Path(p2,p3);
   }
   else if (name=="Sigma"){
      w=wA;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(4);
      R w1=w*0.5;
      R h1=h*0.5;
      Spc p1(w,h);
      Spc p2(0,h);
      Spc p3(w1,h1);
      Spc p4(0,0);
      Spc p5(w,0);
      v_[1]=Path(p1,p2);
      v_[2]=Path(p2,p3);
      v_[3]=Path(p3,p4);
      v_[4]=Path(p4,p5);
   }
   else if (name=="Theta"){
      w=wA;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      R w1=w*0.5;
      R h1=h*0.5;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(2);
      R a=w1;
      R b=h*0.5;
      Spc pc(a,b);
      v_[1]=Path::ellipse(pc,a,b);
      v_[2]=Path(Spc(w*0.33,h1),Spc(w*0.67,h1));
   }
   else if (name=="Xi"){
      w=wA;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      R w1=w*0.5;
      R h1=h*0.5;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(3);
      v_[1]=Path(Spc(0,h),Spc(w,h));
      v_[2]=Path(Spc(w*0.33,h1),Spc(w*0.67,h1));
      v_[3]=Path(Spc(0,0),Spc(w,0));
   }
   else if (name=="Omega"){
      w=wA;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      R w1=w*0.5;
      R h1=h*0.5;
      R dw=w*0.25;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(6);
      Spc p0(o);
      Spc p1(dw,0);
      v_[1]=Path(p0,p1);
      Spc p2(0,h1);
      v_[2]=Path(p1,p2,e2);
      Spc p3(w1,h);
      v_[3]=Path::quarterEllipse(p2,e2,p3);
      Spc p4(w,h1);
      v_[4]=Path::quarterEllipse(p3,e1,p4);
      Spc p5(w-dw,0);
      v_[5]=Path(p5,p4,e2);
      Spc p6(w,0);
      v_[6]=Path(p5,p6);

   }
   else if (name=="Phi"){
      w=wA;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      R w1=w*0.5;
      R h1=h*0.8;
      R h2=h*0.5;
      R h3=h*0.2;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(5);
      Spc p1(w1,h);
      Spc p2(w1,0);
      v_[1]=Path(p1,p2);
      Spc q1(w1,h3);
      Spc q2(0,h2);
      v_[2]=Path::quarterEllipse(q1,-e1,q2);
      Spc q3(w1,h1);
      v_[3]=Path::quarterEllipse(q2,e2,q3);
      Spc q4(w,h2);
      v_[4]=Path::quarterEllipse(q3,e1,q4);
      v_[5]=Path::quarterEllipse(q4,-e2,q1);

   }
   else if (name=="Psi"){
      w=wA;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      R w1=w*0.5;
      R h1=h*0.33;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(3);
      Spc p1(w1,h);
      Spc p2(w1,0);
      v_[1]=Path(p1,p2);
      Spc q1(w1,h1);
      Spc q2(0,h);
      v_[2]=Path::quarterEllipse(q1,-e1,q2);
      Spc q3(w,h);
      v_[3]=Path::quarterEllipse(q1,e1,q3);
   }
   else if (name=="space"){
      w=wB;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      v_=V<Path>(0);
      f_=o;
      l_=Spc(w,0);
      h_=0;
      d_=0;
   }
   else if (name=="hs" || name=="halfspace"){
      w=wj;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      v_=V<Path>(0);
      f_=o;
      l_=Spc(w,0);
      h_=0;
      d_=0;
   }
   else if (name=="backspace"){
      R wSpace=wB+dA;
      v_=V<Path>(0);
      f_=o;
      l_=Spc(-wSpace,0);
      h_=0;
      d_=0;
   }
   else if (name=="backspace2"){
      R wSpace=wA+dA;
      v_=V<Path>(0);
      f_=o;
      l_=Spc(-wSpace,0);
      h_=0;
      d_=0;
   }
   else if (name=="period"){
      w=wPeriod;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      R w1=w*0.5;
      f_=o;
      l_=Spc(w+dA,0);
      d_=0;
      Spc cen(w1,w1);
      v_=V<Path>(1);
      v_[1]=Path::circle(cen,w1);
   }
   else if (name=="colon"){
      w=wPeriod;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      R w1=w*0.5;
      f_=o;
      l_=Spc(w+dA,0);
      d_=0;
      Spc cen1(w1,w1);
      Spc cen2(w1,h-w1);
      v_=V<Path>(2);
      v_[1]=Path::circle(cen1,w1);
      v_[2]=Path::circle(cen2,w1);
   }
   else if (name=="comma"){
      w=wPeriod;
      h=ha;
      R w1=w*0.5;
      f_=o;
      l_=Spc(w+dA,0);
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=dg*0.75;
      Spc cen(w1,w1);
      Spc p1(w,w1);
      Spc p2(0,-d_);
      v_=V<Path>(2);
      v_[1]=Path::circle(cen,w1);
      v_[2]=Path(p2,p1,e2);
   }
   else if (name=="semicolon"){
      w=wPeriod;
      h=ha;
      R w1=w*0.5;
      f_=o;
      l_=Spc(w+dA,0);
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=dg*0.75;
      Spc cen0(w1,h-w1);
      Spc cen(w1,w1);
      Spc p1(w,w1);
      Spc p2(0,-d_);
      v_=V<Path>(3);
      v_[1]=Path::circle(cen0,w1);
      v_[2]=Path::circle(cen,w1);
      v_[3]=Path(p2,p1,e2);
   }
   else if (name=="exclamation_mark"){
      w=wPeriod;
      h=hA;
      R w1=w*0.5;
      f_=o;
      l_=Spc(w+dA,0);
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(2);
      Spc p1(w1,h);
      Spc p2(w1,w*3);
      v_[1]=Path(p1,p2);
      Spc cen(w1,w1);
      v_[2]=Path::circle(cen,w1);
   }
   else if (name=="question_mark"){
      w=wa;
      h=hA;
      R w1=w*0.5;
      f_=o;
      l_=Spc(w+dA,0);
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(4);
      R h1=h*0.8;

      Spc p1(0,h1);
      Spc p2(w1,h);
      v_[1]=Path::quarterEllipse(p1,e2,p2);
      Spc p3(w,h1);
      v_[2]=Path::quarterEllipse(p2,e1,p3);
      R r=wPeriod*0.5;
      Spc p4(w1,r*6);
      v_[3]=Path::cubCur(p3,-e2,1.5,p4,-e2,1.5);
      Spc cen(w1,r);
      v_[4]=Path::circle(cen,r);
   }
   else if (name=="dash"){
      w=wa;
      h=ha;
      f_=o;
      l_=Spc(w+dA,0);
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(1);
      R h1=h*0.5;
      R dw=w*0.2;
      Spc p1(dw,h1);
      Spc p2(w-dw,h1);
      v_[1]=Path(p1,p2);
   }
   else if (name=="equal"){
      w=wa;
      h=ha;
      f_=o;
      l_=Spc(w+dA,0);
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(2);
      R h1=h*0.5;
      R diff_h=h1*0.25;
      R dw=w*0.15;
      Spc p1(dw,h1-diff_h);
      Spc p2(w-dw,h1-diff_h);
      Spc p3(dw,h1+diff_h);
      Spc p4(w-dw,h1+diff_h);
      v_[1]=Path(p1,p2);
      v_[2]=Path(p3,p4);
   }
   else if (name=="l_bracket"){
      w=wj;
      h=hA;
      f_=o;
      l_=Spc(w+dA,0);
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(2);
      R h1=h*0.5;
      Spc p1(w,h);
      Spc p2(0,h1);
      Spc p3(w,0);
      v_[1]=Path(p1,p2,-e2,0.6);
      v_[2]=Path(p3,p2,e2,0.6);
   }
   else if (name=="r_bracket"){
      w=wj;
      h=hA;
      f_=o;
      l_=Spc(w+dA,0);
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(2);
      R h1=h*0.5;
      Spc p1(0,h);
      Spc p2(w,h1);
      Spc p3(0,0);
      v_[1]=Path(p1,p2,-e2,0.6);
      v_[2]=Path(p3,p2,e2,0.6);
   }
   else if (name=="slash"){
      w=wr;
      h=hA;
      f_=o;
      l_=Spc(w+dA,0);
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      v_=V<Path>(1);
      Spc p1(w,h);
      v_[1]=Path(o,p1);
   }
   else if (name=="plus"){
      w=wa;
      h=ha;
      f_=o;
      l_=Spc(w+dA,0);
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      R w1=w*0.5;
      R h1=h*0.5;
      v_=V<Path>(2);
      Spc p1(w,h);
      v_[1]=Path(Spc(0,h1),Spc(w,h1));
      v_[2]=Path(Spc(w1,0),Spc(w1,h));
   }
   else if (name=="quote"){
      w=wj;
      h=hA;
      h_=h; w_=w; u_=Spc(w_,h_);
      v_=V<Path>(1);
      f_=o;
      l_=Spc(dA,0);
      d_=0;
      Spc t1(0,hA);
      Spc t2(0,hA-w);
      v_[1]=Path(t1,t2);
   }
   else if (name=="dquote"){
      w=wj;
      h=hA;
      v_=V<Path>(2);
      f_=o;
      l_=Spc(dA+w,0);
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      Spc p1(0,h);
      Spc p2(0,h-w);
      v_[1]=Path(p1,p2);
      Spc p3(w,h);
      Spc p4(w,h-w);
      v_[2]=Path(p3,p4);
   }
   else if (name=="tilde"){
      w=wA;
      R wh=w*0.5;
      v_=V<Path>(2);
      f_=o;
      l_=Spc(w+dA,0);
      h_=hA*1.25;
      d_=0;
      w_=w; u_=Spc(w_,h_);
      Spc t1(0,h_);
      Spc t2(wh,h_);
      Spc t3(w,h_);
      Vec e1(1,0);
      Angle a(40,DEG);
      Vec v1=e1*AxVec(a,3);
      Vec v2=e1*AxVec(-a,3);
      v_[1]=Path(t1,v1,t2,v2);
      v_[2]=Path(t2,v2,t3,v1);
   }
   else if (name=="schlipf"){
      // Professor Engelbert Schlipf uses this symbol in
      // his lectures on the theory of gears at the FH Ulm.
      w=wr;
      h_=hA;
      R h1=h_*0.8;
      v_=V<Path>(2);
      f_=o;
      l_=Spc(w+dA,0);
      d_=0;
      w_=w; u_=Spc(w_,h_);
      Spc p1(0,h_);
      Spc p2(w,h_);
      Spc p3(w,h1);
      v_[1]=Path(p1,p2);
      v_[2]=Path(p2,p3);
   }
   else if (name=="ast"){
      w=wa;
      h=ha;
      h_=h; w_=w; u_=Spc(w_,h_);
      d_=0;
      f_=o;
      l_=Spc(w+dA,0);
      v_=V<Path>(4);
      Spc p1(w,h);
      Spc pc=o.cen(p1);
      R r=CpmRootX::inf<R>(w,h)*0.5;
      r*=0.9;
      Vec v1=e1*r;
      Vec v2=v1*rot45;
      Vec v3=v2*rot45;
      Vec v4=v3*rot45;
      v_[1]=Path(pc+v1,pc-v1);
      v_[2]=Path(pc+v2,pc-v2);
      v_[3]=Path(pc+v3,pc-v3);
      v_[4]=Path(pc+v4,pc-v4);
   }
}

V<Word> Wrt::allNam()
{
   V<Word> res(123);
   res[1]="A"; res[2]="Au"; res[3]="B"; res[4]="C"; res[5]="D";
   res[6]="E"; res[7]="F"; res[8]="G"; res[9]="H"; res[10]="I";
   res[11]="J"; res[12]="K"; res[13]="L"; res[14]="M"; res[15]="N";
   res[16]="O"; res[17]="Ou"; res[18]="P"; res[19]="Q"; res[20]="R";
   res[21]="S"; res[22]="T"; res[23]="U"; res[24]="Uu"; res[25]="V";
   res[26]="W"; res[27]="X"; res[28]="Y"; res[29]="Z"; res[30]="a";
   res[31]="au"; res[32]="b"; res[33]="c"; res[34]="d";res[35]="e";
   res[36]="f"; res[37]="g"; res[38]="h"; res[39]="i";res[40]="j";
   res[41]="k"; res[42]="l"; res[43]="m"; res[44]="n";res[45]="o";
   res[46]="ou"; res[47]="p"; res[48]="q"; res[49]="r";res[50]="s";
   res[51]="t"; res[52]="u"; res[53]="uu"; res[54]="v"; res[55]="w";
   res[56]="x"; res[57]="y"; res[58]="z"; res[59]="n1"; res[60]="n2";
   res[61]="n3"; res[62]="n4"; res[63]="n5"; res[64]="n6"; res[65]="n7";
   res[66]="n8"; res[67]="n9"; res[68]="n0"; res[69]="alpha";
   res[70]="beta";
   res[71]="gamma"; res[72]="delta"; res[73]="epsilon"; res[74]="theta";
   res[75]="phi";
   res[76]="psi"; res[77]="omega"; res[78]="rho"; res[79]="lambda";
   res[80]="chi";
   res[81]="sigma"; res[82]="mu"; res[83]="nu"; res[84]="upsilon";
   res[85]="eta";
   res[86]="tau"; res[87]="pi"; res[88]="zeta"; res[89]="xi";
   res[90]="kappa";
   res[91]="omicron"; res[92]="iota"; res[93]="Gamma"; res[94]="Lambda";
   res[95]="Delta";
   res[96]="Pi"; res[97]="Sigma"; res[98]="Theta"; res[99]="Xi";
   res[100]="Omega";
  // res[101]="Phi"; res[102]="Psi"; res[103]="ast"; res[104]="ast";
  // res[105]="ast";
  // res[106]="ast"; res[107]="period"; res[108]="colon";
   res[101]="Phi"; res[102]="Psi"; res[103]="space"; res[104]="halfspace";
   res[105]="backspace";
   res[106]="backspace2"; res[107]="period"; res[108]="colon";
   res[109]="comma"; res[110]="semicolon";
   res[111]="exclamation_mark"; res[112]="question_mark"; res[113]="dash";
   res[114]="equal"; res[115]="l_bracket";
   res[116]="r_bracket"; res[117]="slash"; res[118]="plus";
   res[119]="quote"; res[120]="dquote";
   res[121]="tilde"; res[122]="schlipf"; res[123]="ast";
   return res;
}

void Wrt::showAll(Camera const& ca, Color const& color, bool points,
   R tWait)
{
   using namespace CpmImaging;
   Z mL=2;
   Word loc("Wrt::showAll(Camera,Color,bool)");
   CPM_MA
   ca.clr();
   R sclText=0.005;
   Spc o;
   V<Vec> b=Vec::canBas();
   Vec e1=b[1];
   Vec e2=b[2];
   Vec e3=b(3);
   Vec tv1=e1;
   Vec tv2=e2;
   Spc t1=o;
   Spc t2=t1+tv1;
   Spc t3=t2+tv2;
   Spc t4=t1+tv2;
   Spc c0=t1.cen(t3);
   Spc cen=c0.cen(t4).cen(t4);
   V<Spc> vtb{t1,t2,t3,t4};
   Skeleton tb1(vtb,true);
      // table 1
   tb1.clsPath_();
   LineArt la;
   la&=tb1;
   V<Word> vw=Wrt::allNam();
   Z vwd=vw.dim();
   Z i,j,nSec=22;
   Z nLines=vwd/nSec;
   Z rest=vwd%nSec;
   if (rest>0) nLines++;
   VV<Wrt> lines(nLines,nSec);
   Z k=1;
   for (i=1;i<=nLines;++i){
      for (j=1;j<=nSec;++j){
         lines[i][j]=Wrt(vw[k]);
         if (k<vwd) k++;
      }
   }
   R si=0;
   for (i=1;i<=nLines;++i){
      LineArt lai(lines[i],true);
      lai.to_(cen);
      lai.scl_(cen,sclText);
      Vec shift=e2*si;
      R hi=-lai.h();
      si+=hi*2;
      lai+=shift;
      la&=lai;
   }
   la.mark(ca,color,points);
   ca.vis();
   cpmwait(tWait,2);
   CPM_MZ
}

//////////////////////////// class LineArt ///////////////////////////////

LineArt::LineArt( V<Wrt> const& vw, bool chainIt):s_(),t_(),p_(),w_()
{
   if (!chainIt){
      w_=vw;
      return;
   }
   Z vwd=vw.dim();
   if (vwd<1) return;
   Z mL=3;
   Word loc("LineArt(V<Wrt>,bool)");
   CPM_MA
   V<Wrt> res(vwd);
   for (Z i=1;i<=vwd;++i){
      Wrt wi(vw[i]);
    //  wi.ddot_();
    //  wi.vec_();
      if (i>1) wi.to_(res[i-1].last());
      res[i]=wi;
   }
   w_=res;
   CPM_MZ
}

LineArt::LineArt( V<Word> const&  vw, R r, Z n):s_(),t_(),p_(),w_()
{
   Z vwd=vw.dim();
   if (vwd<1) return;
   Z mL=3;
   Word loc("LineArt(V<Word>,R,Z)");
   CPM_MA
   V<Wrt> res(vwd);
   for (Z i=1;i<=vwd;++i){
      Wrt ai(vw[i]);
      ai.thc_(r,n);
      if (i>1) ai.to_(res[i-1].last());
      res[i]=ai;
   }
   w_=res;
   CPM_MZ
}

LineArt LineArt::text(V<Wrt> const& vw)
{
   Z vwd=vw.dim();
   V<Wrt> res(vwd);
   for (Z i=1;i<=vwd;++i){
      Wrt ai=vw[i];
      if (i>1) ai.to_(res[i-1].last());
      res[i]=ai;
   }
   return LineArt(res);
}

R LineArt::h()const
{
   Z n=w_.dim();
   R_Vector r(n);
   for (Z i=1;i<=n;++i) r[i]=w_[i].h();
   return r.sup();
}

LineArt& LineArt::operator*=(Group const& g)
{
   Z i;
   for (i=1;i<=dimS();i++) s_[i]*=g;
   for (i=1;i<=dimT();i++) t_[i]*=g;
   for (i=1;i<=dimP();i++) p_[i]*=g;
   for (i=1;i<=dimW();i++) w_[i]*=g;
   return *this;
}

LineArt& LineArt::operator+=(Vec const& v)
{
   Z i;
   for (i=1;i<=dimS();i++) s_[i]+=v;
   for (i=1;i<=dimT();i++) t_[i]+=v;
   for (i=1;i<=dimP();i++) p_[i]+=v;
   for (i=1;i<=dimW();i++) w_[i]+=v;
   return *this;
}

LineArt& LineArt::scl_(Spc const& fix, R fac)
{
   Z i;
   for (i=1;i<=dimS();i++) s_[i].scl_(fix,fac);
   for (i=1;i<=dimT();i++) t_[i].scl_(fix,fac);
   for (i=1;i<=dimP();i++) p_[i].scl_(fix,fac);
   for (i=1;i<=dimW();i++) w_[i].scl_(fix,fac);
   return *this;
}

LineArt& LineArt::thc_(R r, Z n)
{
   Z i;
   V<Path> coll;
   for (i=1;i<=dimP();i++) coll&=p_[i].thc(r,n);
   for (i=1;i<=dimW();i++) w_[i].thc_(r,n);
   p_=coll;
   return *this;
}

bool LineArt::prnOn(ostream& str)const
{
   cpmwat;
   cpmp(s_);
   cpmp(t_);
   cpmp(p_);
   cpmp(w_);
   return true;
}

bool LineArt::scanFrom(istream& str)
{
   cpms(s_);
   cpms(t_);
   cpms(p_);
   cpms(w_);
   return true;
}

LineArt LineArt::disk(Spc const& p, Vec const& h, R r, Z n)
{
   Z mL=3;
   Word loc("LineArt::disk(Spc,Vec,R,Z)");
   CPM_MA
   cpmassert(n>2,loc);
   cpmassert(r>0,loc);
   Angle phi(360./n,DEG);
   AxVec rot(phi);
   Vec rho(r,0);
   if (Spc::dimension()>2){
      // only then h enters the computation
      R ha=h.abs();
      cpmassert(ha>0,loc);
      V<Vec> trip=h.base();
      Vec ax=trip[1];
      rho=trip[2]*r; // transversal vector
      rot=AxVec(phi,ax);
   }
   AxVec ri=rot;
   V<Vec> vv(n);
   vv[1]=rho;
   Z i;
   for (i=2;i<=n;++i){
      vv[i]=vv[i-1]*rot;
   }
   V<Triangle> bas(n);
   for (i=1;i<=n;++i){
      Spc pi=p+vv[i];
      Spc qi=p+vv(i+1,CpmArrays::CYCLIC);
      bas[i]=Triangle(pi,qi,p);
   }
   CPM_MZ
   return LineArt(bas);
}

LineArt LineArt::ellDisk(Spc const& p, Vec const& h, Vec const& h1,
   R r1, R r2, Z n)
{
   Z mL=3;
   Word loc("LineArt::ellDisk(Spc,Vec,Vec,R,R,Z)");
   CPM_MA
   cpmassert(n>2,loc);
   cpmassert(r1>0,loc);
   cpmassert(r2>0,loc);
   V<Vec> vv(n);
   Vec b1,b2;
   if (Spc::dimension()==2){ // h is ignored
      V<Vec> bas=h1.base();
      b1=bas[1]; b2=bas[2];
   }
   else{
      V<Vec> bas=h1.base(h.crossExc(h1));
      b1=bas[1]; b2=bas[2];
   }
   R ai=0;
   R phi=cpm2pi/n;
   Z i;
   for (i=1;i<=n;++i){
      vv[i]=b1*r1*cpmcos(ai)+b2*r2*cpmsin(ai);
      ai+=phi;
   }
   V<Triangle> bas(n);
   for (i=1;i<=n;++i){
      Spc pi=p+vv[i];
      Spc qi=p+vv(i+1,CpmArrays::CYCLIC);
      bas[i]=Triangle(pi,qi,p);
   }
   CPM_MZ
   return LineArt(bas);
}

LineArt LineArt::sph(Spc const& p, R r, Z n)
{
   const Z points=16;
   const Z steps=40;
   Z mL=3;
   Word loc("LineArt::sph(Spc,R,Z)");
   CPM_MA
   Vec e1(1,0,0);
   LineArt res=disk(p,e1,r,n);
   if (e1.dim()==2){
      CPM_MZ
      return res;
   }
   V<Vec> vv=Vec::homUnitVec(points,steps);
   for (Z i=1;i<=points;++i){
      res&=disk(p,vv[i],r,n);
   }
   CPM_MZ
   return res;
}

LineArt LineArt::cone(Spc const& p, Vec const& h, R r, Z n)
{
   Z mL=3;
   Word loc("LineArt::cone(Spc,Vec,R,Z)");
   CPM_MA
   cpmassert(n>2,loc);
   cpmassert(r>0,loc);
   if (Spc::dimension()==2){
      Spc c=p+h;
      Vec e=h;
      e.nor_();
      Vec n=e.rot90();
      Vec lat=n*r;
      Spc a=p-lat;
      Spc b=p+lat;
      Triangle tri(a,b,c);
      CPM_MZ
      return LineArt(tri);
   }
   R ha=h.abs();
   Vec hCor=h; //experiment
   if (!(ha>R(0))) {ha=1e-9; hCor=Vec(ha,ha);}
   cpmassert(ha>0,loc);
   Spc top=p+hCor;
   V<Vec> trip=hCor.base();
   Vec ax=trip[1];
   Vec rho=trip[2]*r; // transversal vector
   Angle phi(360./n,DEG);
   AxVec rot(phi,ax);
   AxVec ri=rot;
   V<Vec> vv(n);
   vv[1]=rho;
   Z i;
   for (i=2;i<=n;++i){
      vv[i]=vv[i-1]*rot;
   }
   V<Triangle> sur(n);
   V<Triangle> bas(n);
   for (i=1;i<=n;++i){
      Spc pi=p+vv[i];
      Spc qi=p+vv(i+1,CpmArrays::CYCLIC);
      sur[i]=Triangle(pi,qi,top);
      bas[i]=Triangle(pi,qi,p);
   }
   V<Triangle> all=sur&bas;
   CPM_MZ
   return LineArt(all);
}

LineArt LineArt::arr(Spc const& p1, Spc const& p2, R h, R r, Z n)
{
   Vec e=p2-p1;
   e.nor_();
   Spc p2Graph=p2-e*(h*0.5);
      // to avoid dismatches on pixel basis of
      // the end of the line and the top of the cone
   Vec hh=e*h;
   Spc pb=p2-hh;
   LineArt res=cone(pb,hh,r,n);
   return res&=Path(p1,p2Graph);
}

void LineArt::mark(Camera const& ca)const
{
   Z i;
   Color c=ca.isStereo() ? INVISIBLE : BLACK;
   for (i=1;i<=dimS();i++) ca.mark(s_[i],c);
   for (i=1;i<=dimT();i++) ca.mark(t_[i],c);
   for (i=1;i<=dimP();i++) ca.mark(p_[i],c);
   for (i=1;i<=dimW();i++) w_[i].mark(ca,c);
}

void LineArt::mark(Camera const& ca, Color const& col, bool points)const
{
   Z i;
   for (i=1;i<=dimS();i++) ca.mark(s_[i],col);
   for (i=1;i<=dimT();i++) ca.mark(t_[i],col);
   for (i=1;i<=dimP();i++) ca.mark(p_[i],col);
   for (i=1;i<=dimW();i++) w_[i].mark(ca,col,points);
}
