//? cpmmathfound.h
//? C+- by Ulrich Mutze. Status of work 2022-03-21.
//? Copyright (c) 2020 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//? 
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//? 
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for 
//? more details.


#ifndef CPM_MATHFOUND_H_
#define CPM_MATHFOUND_H_
/*
   Description: Some basic classes useful for foundation of
      discrete math.
      NatNum is the class of natural numbers. Implementation
      is based solely on the typographics of character strings.
*/

#include <cpminterfaces.h>
#include <vector>

namespace CpmMathFound{ // foundation of mathematics

const char conChr='l'; // counting character
   // natural numbers are represented by strings made out of
   // this character only (according to what sometimes is called
   // the 'stone age representation' of numbers).

using std::vector;
   
using namespace CpmStd;   

///////////////// class NatNum ///////////////////////////////////////////

class NatNum{  
// natural numbers {1,2,3, ...} in 'stone age representation'

   string rep;
   NatNum(const string& x, bool b):rep(x){}
      // private since it assumes that x is a string of conChr's and
      // of no other characters
public:
   typedef NatNum Type;
   CPM_IO
   CPM_ORDER
   CPM_SUM_C
      // summation is based on concatenation of strings
   CPM_PRODUCT_C
   static NatNum unit;
      // number 1
   bool isUnit()const{ return rep.size()==1;}
      // returns true iff *this==unit
   explicit NatNum(const string& x):rep(x.size(),conChr){}
      // construction from the basic typographical representation
      // of a number as a string of n conChr's. Actually arbitrary
      // characteres are allowed, but will be translated to conChr
      // for storage in string rep.
   NatNum():rep(1,conChr){}
      // constructs number one

   explicit NatNum(int j):rep((j>0 ? j : 1),conChr){}
      // Creation from system provided numbers.
      // if j is not a natural number we create unit
   int toInt()const{ return (int)rep.size();}
      // conversion to system provided numbers

   NatNum next()const{
      string repNew=rep;
      return NatNum(repNew.append(1,conChr),true);
   }
      // next number in natural order; obtained by appending
      // the character conChr
   NatNum prev()const;
      // previous number in natural order; if *this==unit, the
      // result is unit too in order to have the function defined
      // everywhere and with homogeneous type (as the declaration says).
};

} // namespace

#endif
