//? cpmq.h
//? C+- by Ulrich Mutze. Status of work 2022-03-21.
//? Copyright (c) 2020 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//? 
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//? 
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for 
//? more details.


#ifndef CPM_Q_H_
#define CPM_Q_H_
/*

   Description: Declaration of a class of rational numbers

*/
#include <cpmtypes.h>

namespace CpmRoot{

class Q { // rational numbers
   // Mainly used as descriptors for dimensions of physical quantities
   // (see class CpmPhysics::Si)

   Z qZ, qN;
   void normalize();
   Q(const Word& w, Z a, Z b):qZ(a),qN(b){}
      // not safe, but efficient, w has no influence
public:

   Q(void):qZ(0),qN(1){}

   explicit Q(Z a):qZ(a),qN(1){}

   Q(Z a, Z b):qZ(a),qN(b){normalize();}
      // gives a/b in standard form

   explicit Q(R x, R eps=1e-4, Z nOK=20);
      // constructor for q=m/n such that |x-q|<=eps and n minimal

   typedef Q Type;
   typedef Z ScalarType;
   CPM_IO
   CPM_ORDER
   CPM_TEST_MOD
   CPM_DESCRIPTORS
   CPM_CONJUGATION
   CPM_SUM_M
   CPM_PRODUCT_M
   CPM_SCALAR_M
      // this incudes Type + ScalarType, not only Type * ScalarType
   CPM_DIFFERENCE
   CPM_DIVISION

   operator R(void) const{ return ((R)qZ)/qN;}
      // automatic cast for elementary functions
      // conversion from Q to R

   Z numerator()const{ return qZ;}
      // evident, natural type is Z (integer)
      // German association: Z = Zaehler

   Z denominator()const{ return qN;}
      // natural type is N (natural number)
      // German association: N = Nenner
};

} // namespace

#endif
