//? cpmmaxw1d.cpp
//? C+- by Ulrich Mutze. Status of work 2022-03-21.
//? Copyright (c) 2020 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//? 
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//? 
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for 
//? more details.


#include <cpmmaxw1d.h>
#include <cpmvectormore.h>
using namespace CpmFunctions;
using namespace CpmStd;

CpmWaves::TEM::TEM(Z n, R L, R_Func const& fE, R_Func const& fH):
   n_(n),d_(2*n_+1),L_(L),x_(d_),X_(d_)
{
   ini_();
   R dxi=1./(d_-1);
   bool putE=true;
   R x=0,xi=0;
   for (Z i=0;i<d_;++i){
      x_[i]=x;
      x+=s_;
      if (putE){
         X_[i]=fE(xi);
         putE=false;
      }
      else{
         X_[i]=fH(xi);
         putE=true;
      }
      xi+=dxi;
   }
   dX_=der(X_);
}

CpmWaves::TEM::TEM(Z n, R L, Iv const& iv):
   n_(n),d_(2*n_+1),L_(L),x_(d_),X_(0.,d_)
{
   ini_();
   bool putE=true;
   R x=0;
   for (Z i=0;i<d_;++i){
      x_[i]=x;
      x+=s_;
      if (putE){
         X_[i]=((i==0 || i==d_-1) ? R(0) : iv.ran());
         putE=false;
      }
      else putE=true;
   }
   dX_=der(X_);
}

void CpmWaves::TEM::showAsCurves(Graph& g)const
{
   using namespace CpmImaging;
   V<Color> col(2);
   col[1]=RED;
   col[2]=GREEN;
   g.addText("TEM-1D: E-field red, H-field green");
   g.mark(getEH(),col);
   g.vis();
}

//////////////////////////////////////////////////////////
CpmWaves::TEM2::TEM2(Z n, R L, R_Func const& fE, R_Func const& fH):
   n_(n),L_(L),x_(n_),X_(n_)
{
   ini_();
   R x=0;
   R xi=0;
   R dxi=1./(n_-1);
   for (Z i=1;i<=n_;++i){
      x_[i]=x;
      x+=s_;
      X_.setE_(i,fE(xi));
      X_.setH_(i,fH(xi));
      xi+=dxi;
   }
   dX_=X_.dot(hsInv_);
}

CpmWaves::TEM2::TEM2(Z n, R L, Iv const& iv):
   n_(n),L_(L),x_(n_),X_(n_)
{
   ini_();
   R x=0;
   for (Z i=1;i<=n_;++i){
      x_[i]=x;
      x+=s_;
      R val=(i==1 || i==n_) ? R(0) : iv.ran();
      X_.setE_(i,val);
   }
   dX_=X_.dot(hsInv_);
}

R_Vector fAux(R const& t, Iv const& iv, R_Vector const& v, R const& omega\
)
{
   R ft=cpmsin(omega*(t-iv[1]));
   return v*ft;
}

CpmWaves::TEM2::TEM2(Z n, R L, R xJ, Iv tJ, R omegaJ,Z nSmoothJ):
n_(n),L_(L),x_(n_),X_(n_),tci_(tJ[1]),tcf_(tJ[2]),withCurrent_(true)
{
   ini_();
   R x=0;
   R vE=1e-10;
   R vH=R(-0.5)*vE;
   R xi=0;
   R dxi=1./(n_-1);
   for (Z i=1;i<=n_;++i){
      x_[i]=x;
      x+=s_;
      R y=xi*(R(1)-xi);
      X_.setE_(i,y*vE);
      X_.setH_(i,y*vH);
      xi+=dxi;
   }
   Z iJ=cpmrnd(xJ*sInv_+1);
   R_Vector vJ(n_);
   vJ[iJ]=1;
   vJ=CpmArrays::smoothen(vJ,nSmoothJ);
#ifdef CPM_Fn
   fJ_=F3<R,Iv,R_Vector,R,R_Vector>(tJ,vJ,omegaJ)(fAux);
#else
   fJ_=F<R,R_Vector>(bind(fAux,_1,tJ,vJ,omegaJ));
#endif
   dX_=X_.dot(hsInv_);
}

void CpmWaves::TEM2::showAsCurves(Graph& g)const
{
   using namespace CpmImaging;
   V<Color> col(2);
   col[1]=RED;
   col[2]=BLUE;
   g.addText("TEM2-1D: E-field red, H-field blue");
   g.mark(getEH(),col);
   g.vis();
}

