//? cpmqp1.cpp
//? C+- by Ulrich Mutze. Status of work 2022-03-21.
//? Copyright (c) 2020 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//? 
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//? 
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for 
//? more details.


#include <cpmqp1.h>

using namespace CpmRoot;
using namespace CpmRootX;
using namespace CpmFunctions;
using namespace CpmSystem;
using namespace CpmArrays;
using namespace CpmGraphics;
using namespace CpmImaging;
using namespace CpmApplication;
using namespace CpmQuantumMechanics;
using namespace std;

/////////////////// Wf implementation /////////////////////////////////////

R Wf::opNor(F<Wf,Wf> const& op, IvZ const& ivz)
{
//   cpmmessage("opNor started");
   Z jRan=137;
   Z nSteps=100;
   Wf psi = Wf::ran(ivz,jRan);
   Wf chi = op(psi);
 //  Graph gr;
   R eta=chi.abs();
   R_Vector etaVec(nSteps);
   for (Z i=1;i<=nSteps;++i){
      psi=chi*cpminv(eta);
//      psi.mark(gr);
//      gr.vis();
//      cpmwait(0.5);
      chi=op(psi);
      eta=chi.abs();
      etaVec[i]=eta;
   }
//   gr.mark(etaVec);
//   gr.vis();
//   cpmwait(10,2);
   return etaVec.sup();
}

Wf Wf::con()const
{
   map<Z,C> res;
   for (auto ix=rep_.begin(); ix!=rep_.end(); ++ix ){
      Z i=(*ix).first;
      res[i]=(*ix).second.con();
   }
   return res;
}

Wf Wf::re()const
{
   map<Z,C> res;
   for (auto ix=rep_.begin(); ix!=rep_.end(); ++ix ){
      Z i=(*ix).first;
      res[i]=(*ix).second.real();
   }
   return res;
}

Wf Wf::im()const
{
   map<Z,C> res;
   for (auto ix=rep_.begin(); ix!=rep_.end(); ++ix ){
      Z i=(*ix).first;
      res[i]=(*ix).second.imag();
   }
   return res;
}

C Wf::operator|(Wf const& psi)const
{
   C sum;
   for (auto ix=rep_.begin(); ix!=rep_.end(); ++ix ){
      Z i=(*ix).first;
      auto iy=psi.rep_.find(i);
      if(iy!=psi.rep_.end()) sum+=(*ix).second.con()*(*iy).second;
   }
   return sum;
}

Wf Wf::mult( F<R,R> const& f, R const& d)const
{
   map<Z,C> res;
   for (auto ix=rep_.begin(); ix!=rep_.end(); ++ix ){
      Z i=(*ix).first;
      C z=(*ix).second;
      res[i]=z*f(d*i);
   }
   return res;
}

Wf Wf::operator + (Wf const& psi)const
{
   map<Z,C> res;
   for (auto p=rep_.begin(); p!=rep_.end(); ++p ){
      Z i=(*p).first;
      res[i]+=(*p).second;
   }
   for (auto q=psi.rep_.begin(); q!=psi.rep_.end(); ++q ){
      Z i=(*q).first;
      res[i]+=(*q).second;
   }
   return res;
}

Wf Wf::operator - (Wf const& psi)const
{
   map<Z,C> res;
   for (auto p=rep_.begin(); p!=rep_.end(); ++p ){
      Z i=(*p).first;
      res[i]+=(*p).second;
   }
   for (auto q=psi.rep_.begin(); q!=psi.rep_.end(); ++q ){
      Z i=(*q).first;
      res[i]-=(*q).second;
   }
   return res;
}

Wf Wf::operator * (C const& z)const
{
   map<Z,C> res;
   for (auto ix=rep_.begin(); ix!=rep_.end(); ++ix ){
      Z i=(*ix).first;
      res[i]=(*ix).second * z;
   }
   return res;
}

Wf Wf::operator * (R const& r)const
{
   map<Z,C> res;
   for (auto ix=rep_.begin(); ix!=rep_.end(); ++ix ){
      Z i=(*ix).first;
      res[i]=(*ix).second * C(r);
   }
   return res;
}

Wf Wf::operator + (Z const& i)const
{
   map<Z,C> res;
   for (auto ix=rep_.begin(); ix!=rep_.end(); ++ix ){
      Z j=(*ix).first;
      res[j+i]=(*ix).second ;
   }
   return res;
}

// shorter code for '+=' than for '+' and similar
// operations.
Wf& Wf::operator += (Wf const& psi)
{
   for (auto q=psi.rep_.begin(); q!=psi.rep_.end(); ++q ){
      Z i=(*q).first;
      rep_[i]+=(*q).second;
   }
   return *this;
}

Wf& Wf::operator -= (Wf const& psi)
{
   for (auto q=psi.rep_.begin(); q!=psi.rep_.end(); ++q ){
      Z i=(*q).first;
      rep_[i]-=(*q).second;
   }
   return *this;
}

Wf& Wf::operator *= (C const& z)
{
   for (auto ix=rep_.begin(); ix!=rep_.end(); ++ix ){
      (*ix).second *= z;
   }
   return *this;
}

Wf& Wf::operator *= (R const& r)
{
   for (auto ix=rep_.begin(); ix!=rep_.end(); ++ix ){
      (*ix).second *= r;
   }
   return *this;
}

R Wf::nor_()
{
   R r=abs();
   R rInv=cpminv(r);
   (*this)*=rInv;
   return r;
}

void Wf::mark(Graph& g, R const& d)const
{
   Word loc("Wf::mark");
   Z mL=3;
   CPM_MA
   using namespace CpmImaging;
   using namespace CpmArrays;
   V<Color> cl{BLACK, RED, GREEN};

   Z iB = (*(rep_.begin())).first;
   Z iE = (*(--rep_.end())).first;
   Z dim=iE-iB+1;
   R_Vector x(dim);
   R_Matrix y(3,dim);
   for (auto ix=rep_.begin(); ix!=rep_.end(); ++ix ){
      Z i=(*ix).first;
      Z is=i-iB+1;
      C val=(*ix).second;
      x[is]=d*R(i);
      y[1][is]=val.abs();
      y[2][is]=val[1];
      y[3][is]=val[2];
   }
   g.clear();
   g.setAutoScl_(true);
   g.addText("Wf: Real part red, Imaginary part green, Abs black");
   g.mark(x,y,cl);
   CPM_MZ
}

M<Z,C> Wf::toM()const
{
   M<Z,C> res;
   for (auto ix=rep_.begin(); ix!=rep_.end(); ++ix ){
      Z i=(*ix).first;
      C v=(*ix).second;
      res[i]=v;
   }
   return res;
}

bool Wf::prnOn( ostream& str )const
{
   return toM().prnOn(str);
}

bool Wf::scanFrom( istream& str )
{
   return toM().scanFrom(str);
}

Wf Wf::dot(R const& d, R const& m )const
{
   C g=C::I*cpminv(d*d*m*2.);
   Wf_ res;
   for (auto p = rep_.begin(); p != rep_.end(); ++p){
      Z i=(*p).first;
      Z ip=i+1;
      Z im=i-1;
      C wi=(*p).second;
      res[i] -= wi*2.;
      res[ip]+=wi;
      res[im]+=wi;
   }
   return Wf(res)*g;
}

Wf Wf::der(R const& d)const
{
   R g=cpminv(d*2.);
   Wf_ res;
   for (auto p = rep_.begin(); p != rep_.end(); ++p){
      Z i=(*p).first;
      C wi=(*p).second;
      res[i+1]+=wi;
      res[i-1]-=wi;
   }
   return Wf(res)*g;
}

Wf Wf::ran(IvZ const& ivz, Z j)
{
   C z(1.,1.);
   Wf res;
   for (Z i=ivz.b();i<=ivz.e();++i){
      res.rep_[i]=z.ran(j++);
   }
   res.nor_();
   return res;
}

void Wf::cut_(R const& eps)
{
   if (eps<=0.) return;
   R sup=-1.;
   for (auto ix=rep_.begin(); ix!=rep_.end(); ++ix ){
 //     Z i=(*ix).first;
      C v=(*ix).second;
      R s=v.abs();
      if (s > sup) sup=s;
   }
   R er=eps*sup;
   C zL = (*rep_.begin()).second;
   Z iL = (*rep_.begin()).first;
   C zU = (*(--rep_.end())).second;
   Z iU = (*(--rep_.end())).first;
   if (zU.abs()<er) rep_.erase(iU);
   if (zL.abs()<er) rep_.erase(iL);
 }

////////////////////// class Qp1 //////////////////////////////////////////

void Qp1::step_(R dt){

   R tau=dt*0.5;
   R tauh=tau*0.5;
   t_+=tauh;
   psi_+= phi_*tauh;
   phi_= psi_.dot(d_,m_)*2.-phi_;
   phi_.cut_(eps_);
   psi_+= phi_*C(tau);
   t_+=tau;
   phi_= psi_.dot(d_,m_)*2.-phi_;
   phi_.cut_(eps_);
   psi_+= phi_*tauh;
   t_+=tauh;
   norm_=psi_.abs();
   avr_=(C::I*(psi_|phi_)).real();
}

void Qp1::shift_(R dt){
   R tau=dt*0.5;
   R tauh=tau*0.5;
   t_+=tauh;
   psi_+= phi_*tauh;
   phi_= psi_.der(d_)*2.-phi_;
   phi_.cut_(eps_);
   psi_+= phi_*C(tau);
   t_+=tau;
   phi_= psi_.der(d_)*2.-phi_;
   phi_.cut_(eps_);
   psi_+= phi_*tauh;
   t_+=tauh;
   norm_=psi_.abs();
   avr_=(C::I*(psi_|phi_)).real();
}

void Qp1::step_(R dt, F<Wf,Wf> const& op){
   R tau=dt*0.5;
   R tauh=tau*0.5;
   t_+=tauh;
   psi_+= phi_*tauh;
   phi_= op(psi_)*C(0.,2.)-phi_;
   phi_.cut_(eps_);
   psi_+= phi_*C(tau);
   t_+=tau;
   phi_= op(psi_)*C(0.,2.)-phi_;
   phi_.cut_(eps_);
   psi_+= phi_*tauh;
   t_+=tauh;
   norm_=psi_.abs();
   avr_=(C::I*(psi_|phi_)).real();
}

C fFunc(R const& x, R const& x0, R const& w, R const& p){
   R xr=(x-x0)/w;
   return C::expi(x*p)*exp(-0.5*xr*xr);
}

Wf hamFunc(Wf const& psi, R const& d, R const& m){
   return psi.dot(d,m)*C::I;
}

Wf pFunc(Wf const& psi, R const& d){
   return psi.der(d)*C::I;
}

