//? cpmsys2datcpp.h
//? C+- by Ulrich Mutze. Status of work 2022-03-21.
//? Copyright (c) 2020 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//? 
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//? 
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for 
//? more details.


#include <cpmunits.h>
#include <cpmroot.h>
#include <cpmperm.h>
#include <cpms.h>

namespace CPM_TOP{

using CPM_DIM::Spc;
using CPM_DIM::Vec;
using CPM_DIM::AxVec;
using CPM_DIM::Group;
using CPM_DIM::RefSys;
using CPM_DIM::dimVec;
using CPM_DIM::Rdim;
using CpmGeo::Angle;
using CpmGeo::RAD;

using namespace CpmRoot;
using namespace CpmRootX;
using namespace CpmLinAlg;

R Sys2Dat::getArea()const
{
   return get_xLength()*get_yLength();
}

V<Rdim> Sys2Dat::testPoints(Z n)const
{
   Iv iv(xL,xU);
   V<R> vx=iv.centers(n);
   V<Rdim> res(n);
   for (Z i=1;i<=n;i++) res[i]=Rdim(vx[i],getH());
   return res;
}

bool Sys2Dat::scanFrom(std::istream& str)
{
   cpms(dim_);
   cpms(xL);
   cpms(xU);
   cpms(yL);
   cpms(yU);
   cpms(zL);
   cpms(zU);
   adjust(); // here the assertion in in adjust()
      // may point out inconsitencies if SysViewerAppl
      // processes a inputfile which was created within
      // a dimension context inconsistent with the value dim_
   return true;
}

bool Sys2Dat::prnOn(std::ostream& str)const
{
   CpmRoot::writeTitle("Sys2Dat",str);
   cpmp(dim_);
   cpmp(xL);
   cpmp(xU);
   cpmp(yL);
   cpmp(yU);
   cpmp(zL);
   cpmp(zU);
   return true;
}

void Sys2Dat::adjust()
{
   Z mL=3;
   Word loc("Sys2Dat::adjust():");
   CPM_MA
   if (dim_!=dimVec){
      ostringstream ost;
      ost<<loc<<"We are reading a "<<dim()<<"D movie with the "
      <<dimVec<<"D version of the viewer program."<<endl
      <<"One needs to set mode="<<dim()<<
      " in section selection of the ini-file";
      cpmwarning(ost); 
   }
   cpmassert(dim_==dimVec,loc);
   di1=DiIv(xL,xU);
   di2=DiIv(yL,yU);
   if (dimVec>2) di3=DiIv(zL,zU);
   CPM_MZ
}

Sys2Dat::Sys2Dat(R xL_,R xU_, R yL_,R yU_, R zL_, R zU_):
   xL(xL_),xU(xU_),yL(yL_),yU(yU_),zL(zL_),zU(zU_)
{
   if (xL>xU) cpmerror("Sys2Dat(...): xL>xU");
   if (yL>yU) cpmerror("Sys2Dat(...): yL>yU");
   if (zL>zU) cpmerror("Sys2Dat(...): zL>zU");
   dim_=dimVec;
   Sys2Dat::adjust(); // in this case the assertion in 
      // adjust holds true
}

Rdim Sys2Dat::f(const Rdim& w)const
{ 
   Rdim res;
   res[1]=di1.p(w[1]);
   res[2]=di2.p(w[2]);
   if (dimVec>2) res[3]=di3.p(w[3]);
   return res;
}

Rdim Sys2Dat::fInv(const Rdim& w)const
{
   Rdim res;
   res[1]=di1.x(w[1]);
   res[2]=di2.x(w[2]);
   if (dimVec>2) res[3]=di3.x(w[3]);
   return res;
}

} // namespace
