//? cpmsys2datx.h
//? C+- by Ulrich Mutze. Status of work 2022-03-21.
//? Copyright (c) 2020 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//? 
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//? 
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for 
//? more details.


   using CPM_DIM::Rdim;
   using CPM_DIM::ParOfSpc;
      // partition of space, see cpm2/cpmpartition.h

   using CpmRoot::Word;
   using CpmRoot::R;
   using CpmRoot::Z;

   using CpmRootX::B;

   using CpmGeo::Angle;
   using CpmGeo::DiIv;
   using CpmGeo::RAD;
   using CpmGeo::Iv;

   using CpmArrays::V;

//doc namespace CpmTOP{

///////////////////////// class Sys2Dat /////////////////////////////
// data that determine a rectangular box and data relevant to
// partitioning particles into groups to be handled by different processes
// in parallel computing. Here we have a rectangular box as the living
// room for the and not the complicated geometry as in PaDyAn

class Sys2Dat { // sys2 data

   Z dim_{0};
      // 2 or 3; info on dimension which does
      // not depend on the reading context

   R xL{0.};
      // boundary in x-direction, smallest x-value
   R xU{0.};
      // boundary in x-direction, largest x-value
   R yL{0.};
      // boundary in y-direction, smallest y-value
   R yU{0.};
      // boundary in y-direction, largest y-value
   R zL{0.};
      // boundary in z-direction, smallest z-value
   R zU{0.};
      // boundary in z-direction, largest z-value

// dependent data for efficiency

   DiIv di1,di2,di3;

   void adjust();
      // setting the values of the dependent quantities
public:
   typedef Sys2Dat Type;
   Sys2Dat(){}
   Sys2Dat(R xL, R xU, R yL, R yU, R zL=0, R zU=0);
      // constructor from independent data
   CPM_IO
   Rdim f(const Rdim&)const;
      // maps the region of interest into the unit cube
   Rdim fInv(const Rdim&)const;
      // is an inverse function only on the unit cube
   Z dim()const{ return dim_;}

   R getArea()const;
      // area in the (1,2)-plane

   R get_xL()const{ return xL;}
   R get_xU()const{ return xU;}
   R get_yL()const{ return yL;}
   R get_yU()const{ return yU;}
   R get_zL()const{ return zL;}
   R get_zU()const{ return zU;}
   R get_xLength()const{ return xU-xL;}
   R get_yLength()const{ return yU-yL;}
   R getH()const{ return yU-yL;}
   R get_zLength()const{ return zU-zL;}
   V<Rdim> testPoints(Z n)const;
   Rdim corL()const{ return Rdim(xL,yL,zL);}
      // lower corner
   Rdim corU()const{ return Rdim(xU,yU,zU);}
      // upper corner
   Iv iv1()const{ return Iv(xL,xU);}
      // interval on 1-axis
   Iv iv2()const{ return Iv(yL,yU);}
      // interval on 2-axis
   Iv iv3()const{ return Iv(zL,zU);}
      // interval on 3-axis
};

typedef ParOfSpc<Sys2Dat> Partition;
   // partition with a simple rectangular box geometry
