//? cpmsysviewer.cpp
//? C+- by Ulrich Mutze. Status of work 2022-03-21.
//? Copyright (c) 2020 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//? 
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//? 
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for 
//? more details.


#include <cpmsysviewer.h>
#include <cpmalgorithms.h>
#include <cpmsys2.h>
#include <cpmgreg.h>
#include <cpmframes.h>
#include <iomanip>

namespace CpmPalaAux{

using CpmArrays::V;
using CpmSystem::IFileStream;
using CpmTime::Greg;

using namespace CpmSystem;
using namespace CpmRoot;
using namespace CpmRootX;
using namespace CpmApplication;
using namespace CpmAlgorithms;
using namespace CpmProperties;
using namespace CpmGraphics;
using namespace CpmImaging;

V<Z> SysViewerAppl::showMovie2()
// all cameras get set to newColor=false so since otherwise the display
// is too slow and not all images get refreshed on equal footing.
{
#if defined(CPM_DIMDEF2)
   using namespace CpmPhysics2;
   using namespace CpmRigidBody2;
   using namespace CpmDim2;
   using namespace CpmPala2_2;
   Z mL=1;
   Word loc("showMovie2(...)");

   V<CpmCamera2::Camera> ca=ca2_;
   #include <cpmsysviewercpp.h>
#endif
   return V<Z>();
}

V<Z> SysViewerAppl::showMovie3()

{
#if defined(CPM_DIMDEF3)
   using namespace CpmPhysics3;
   using namespace CpmRigidBody3;
   using namespace CpmDim3;
   using namespace CpmPala2_3;
   Z mL=1;
   Word loc("showMovie3(...)");
   V<CpmCamera3::Camera> ca=ca3_;
   #include <cpmsysviewercpp.h>
#endif
   return V<Z>();
}

////////////////////// SysViewerAppl /////////////////////////////////

SysViewerAppl::SysViewerAppl(const Word& w):
   IniFileBasedApplication(w),sc_()
{
   const  Z mL=1;
   Word loc("SysViewerAppl("&w&")");
   cpmmessage(mL,loc&" started");
   Z i, ped=1, n=inFiles.dim();
 
   Word sec="syscontrol";
   SysControl::read(rch,sec,ped);

   sec="runcontrol";

   Z clearPeriod, imagingPeriod, viewingPeriod;
   rch.read(sec,"clearPeriod",clearPeriod);
   rch.read(sec,"imagingPeriod",imagingPeriod);
   rch.read(sec,"viewingPeriod",viewingPeriod);
   rch.read(sec,"firstFrame",sc_.offSet);
   rch.read(sec,"boxSel",sc_.boxSel);

   V<R> xRange,yRange,zRange;
   rch.read(sec,"xRange",xRange);
   rch.read(sec,"yRange",yRange);
   rch.read(sec,"zRange",zRange);
   sc_.ivx=Iv(xRange[1],xRange[2]);
   sc_.ivy=Iv(yRange[1],yRange[2]);
   sc_.ivz=Iv(zRange[1],zRange[2]);

   sc_.cyalClear=CyclicAlarm(clearPeriod);
   sc_.cyalImg=CyclicAlarm(imagingPeriod);
   sc_.cyalView=CyclicAlarm(viewingPeriod);

   rch.read(sec,"tWait",sc_.tWait);
   rch.read(sec,"modality",sc_.modality);

   V<Z> frameNumbers;
   rch.read(sec,"frameNumbers",frameNumbers);
   sc_.nocyalImg=NonCyclicAlarm(frameNumbers);

   rch.read(sec,"maxNumberImages",sc_.maxNumberImages);

// reading two Zs-lists in a way that they get ignored if they contain
// a single 0. This is OK since valid values have to be >0.
   sc_.invisibleParticles=V<Z>(0);
   V<Z> invisibleParticles(0);
   bool succ=rch.read(sec,"invisibleParticles",invisibleParticles,ped);
   if (succ){ 
      if (!(invisibleParticles.dim()==1&&invisibleParticles[1]==0))
         sc_.invisibleParticles=invisibleParticles;
   }

   sc_.listColorize=V<Z>(0);
   V<Z> listColorize(0);
   succ=rch.read(sec,"listColorize",listColorize,ped);
   if (succ){
      if (!(listColorize.dim()==1&&listColorize[1]==0))
         sc_.listColorize=listColorize;
   }
 
// now the cameras get initialized from the ini-file
   sec="cameracontrol"; 
   rch.read(sec,"autoFrameFactor",sc_.autoFrameFactor,ped);
   Frame f0;
   R fill=(n==1 ? 1.0 : 0.98);
   Frames frs(f0,n,fill);
  // setModeFromInFiles(); experiment
   if (mode_==2){
#if defined(CPM_DIMDEF2)
   ca2_=V<CpmCamera2::Camera>(n);
   for (i=1;i<=n;i++) ca2_[i]=CpmCamera2::Camera(rch,sec,frs(i));
#endif
   }
   else if (mode_==3){
#if defined(CPM_DIMDEF3)
   ca3_=V<CpmCamera3::Camera>(n);
   for (i=1;i<=n;i++) ca3_[i]=CpmCamera3::Camera(rch,sec,frs(i));
#endif
   }
   else{
      cpmerror("CpmViewer::SysViewerAppl(...): unvalid mode_"); 
   }
   cpmmessage(mL,loc&" done");
}

void SysViewerAppl::doTheWork()
{
   const  Z mL=1;
   cpmmessage(mL,"SysViewerAppl::doTheWork() started");
   V<Z> count(0);

   if (mode_==2){
#if defined(CPM_DIMDEF2)
      count=showMovie2();
#endif
   }
   else if (mode_==3) {
#if defined(CPM_DIMDEF3)
      count=showMovie3();
#endif 
   }
   else{
      cpmerror("CpmViewer::SysViewerAppl::doTheWork(): unvalid mode_"); 
   }
   cpmdata<<"movie frames found: "<<endl;
   count.prnOn(cpmdata);
   cpmmessage(mL,"SysViewerAppl::doTheWork() done");
}

} // CpmPalaAux
