//? cpmbasictypesasclass.h
//? C+- by Ulrich Mutze. Status of work 2022-03-21.
//? Copyright (c) 2020 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//? 
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//? 
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for 
//? more details.


#ifndef CPM_BASIC_TYPES_AS_CLASS_H_
#define CPM_BASIC_TYPES_AS_CLASS_H_
/*

   Description: Introduces mathematics style named
      aliases for the built-in types that the C+- class 
      system will use.

   History: made 2007-08-16 
      for experimentation with class definition of fundamental types
      Could work, but some modifications needed avoid ambiguous 
      conversions. One should probably start with ensuring that 
      the basic files such as cpmword.cpp and cpmintervalz.cpp compile 
 
*/
#include <cpmdefinitions.h> // for CPM_LONG 

namespace CpmRoot{

// These are the basic types on which most of C+- class hierarchy
// will be built
# if defined(CPM_LONG)   
   typedef long int Z;
      // integer numbers
   typedef long double R_;
      // real numbers
#else
   typedef int Z;
      // integer numbers
   typedef double R_;
      // real numbers   
#endif 

   typedef float Rh;
 
   class R{
      R_ x_;
   public:
      R():x_(0.){}
      R(R_ const& x):x_(x){}
      explicit R(Z const& z):x_((Z_)z){}
      R& operator+=(R const& x){ x_+=x.x_; return *this;}
      R& operator-=(R const& x){ x_-=x.x_; return *this;}
      R& operator*=(R const& x){ x_*=x.x_; return *this;}
      R& operator/=(R const& x){ x_/=x.x_; return *this;}

      R& operator++(void){++x_; return *this;}
/// prefix decrement (subtracting 1)      
      R& operator--(void){--x_; return *this;}
/// postfix increment (adding 1)       
      R operator++(int){ R res=*this; x_++; return res;}
/// prefix decrement (subtracting 1)  
      R operator--(int){ R res=*this; x_--; return res;}

      R operator-(void)const{ return R(-x_;}
    //  operator bool()const{ return x_!=0;}
 //     operator Z(void)const{ return Z((Z_) x_);}
 //     operator Rh()const;
      friend Z R2Z(R const& x)
         { return (Z)x.x_;}
      friend R operator+(R const& x1, R const& x2)
         { return R(x1.x_+x2.x_);}
      friend R operator+(Z const& x1, R const& x2)
         { return R(x1+x2.x_);}
      friend R operator+(R const& x1, Z const& x2)
         { return R(x1.x_+x2);}
      friend R operator-(R const& x1, R const& x2)
         { return R(x1.x_-x2.x_);}
      friend R operator-(Z const& x1, R const& x2)
         { return R(x1-x2.x_);}
      friend R operator-(R const& x1, Z const& x2)
         { return R(x1.x_-x2);}
      friend R operator*(R const& x1, R const& x2)
         { return R(x1.x_*x2.x_);}
      friend R operator*(Z const& x1, R const& x2)
         { return R(x1*x2.x_);}
      friend R operator*(R const& x1, Z const& x2)
         { return R(x1.x_*x2);}
      friend R operator/(R const& x1, R const& x2)
         { return R(x1.x_/x2.x_);}
      friend R operator/(Z const& x1, R const& x2)
         { return R(x1/x2.x_);}
      friend R operator/(R const& x1, Z const& x2)
         { return R(x1.x_/x2);}
      friend bool operator==(R const& x1, R const& x2)
         { return x1.x_==x2.x_ ;}
      friend bool operator!=(R const& x1, R const& x2)
         { return x1.x_!=x2.x_;}
      friend bool operator<=(R const& x1, R const& x2)
         { return x1.x_<=x2.x_;}
      friend bool operator>=(R const& x1, R const& x2)
         { return x1.x_>=x2.x_;}
      friend bool operator<(R const& x1, R const& x2)
         { return x1.x_<x2.x_;}
      friend bool operator>(R const& x1, R const& x2)
         { return x1.x_>x2.x_;}
   };
    
// Much less important types:

 //  typedef float Rh;
      // Real numbers of reduced ('half') accuracy, mainly to be
      // used in floating point image processing, where reducing the
      // size of large image matrix data by a factor 2 might be 
      // desirable (less important today).

   typedef unsigned char L;
      // 'L' for letter represents a byte which
      // after 'integral promotion' is a value between 0 and 255.
      // Mainly needed for storing pixel values in image matrices.
}

#endif
