//? cpmguiapplication.h
//? C+- by Ulrich Mutze. Status of work 2022-03-21.
//? Copyright (c) 2020 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//? 
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//? 
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for 
//? more details.


#ifndef CPM_GUIAPPLICATION_H
#define CPM_GUIAPPLICATION_H

/*
Purpose: Offer tools for building interactive application programs
   No experience available. Experimental!

  History: Started 2005-04-26
*/
#include <cpmapplication.h>

namespace CpmApplication{

   using CpmRoot::Word;
   using CpmRoot::Z;
   using CpmRoot::R;
   using CpmRoot2::RecordHandler;
   using CpmArrays::V;
   using CpmArrays::Vp;
   using CpmFunctions::F;
   using CpmFunctions::F1;
   
template <class Sys>
class GuiApp: public IniFileBasedApplication{
   R runTimeSpan;
   Z nSteps,filingPeriod,filingBurstPeriod,filingBurstLength,
      viewingPeriod;
   R dtStep;
   Timer tm;
   F<R,Z> f;
   Frame fr0;   
   B initialized;
   static Sys sys;
   Camera cam;
   public:
   GuiApp(const Word& w):IniFileBasedApplication(w){}
   void doTheWork();
}; 


//Station* sp;
// needed already here since the next function needs it

template <class Sys>
void GuiApp<Sys>::doTheWork()
{
//   cpmmessage("StationAppl::doTheWork() started");

   bool earlyStop=false;
 
   if (!initialized){ // not yet done, preparation necessary
      // notice that we prepare for calling this function many times in
      // an
      // event-controlled program

      Word sec("selection");
      rch.read(sec,"cpmdbg",CpmSystem::Message::debug);
      rch.read(sec,"runTimeSpan",runTimeSpan);
      sec="syscontrol";
      SysControl(rch,sec);
      // Some of SysControl's data will be overwritten soon.
      sec="runcontrol";
      rch.read(sec,"dtStep",dtStep);
      rch.read(sec,"nSteps",nSteps);
      rch.read(sec,"filingPeriod",filingPeriod);
      rch.read(sec,"filingBurstPeriod",filingBurstPeriod);
      rch.read(sec,"filingBurstLength",filingBurstLength);
      rch.read(sec,"viewingPeriod",viewingPeriod);
      tm=Timer(runTimeSpan*3600);
     // runTimeSpan is in hours, and Timer() expects seconds      
      f=F<R,Z>(0);
      fr0=Frame();
      cam=Camera(rch,"cameracontrol",fr0);
      sys= Sys(rch);
      initialized=true;      
   } 
   earlyStop=sys.run(dtStep,nSteps,filingPeriod,filingBurstPeriod,
      filingBurstLength,viewingPeriod,0,outFiles[1],cam,tm,f);
}

void stationRun()
{
   static StationAppl a("station"); 
      // will be called only with the first call to station()
      // When station gets called several times (as in interactive
      // programs
      // where it
      // gets called in idleFunc, it knows always its status
   a.run();
}

}

Word loc;
bool prepared=false;

void displayFunc(void)
// everything is done by direct graphical commands
{
}   

void glToglut(int w, int h)
{
  glViewport(0,0,w,h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(0,w,0,h);
  glScalef(1.0,-1.0,1.0);   /* this flips the coordinate system so
         that y increases going down the
         screen (see below) */
  glTranslatef(0.0,-h,0.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glTranslatef(0.375,0.375,0.0);/* so we can draw using integer
          coordinates (see above) */
   glPixelStorei(GL_UNPACK_ALIGNMENT,1);
   // in order to intoduce no alignement errors depending on pixel
   // matrices having sizes that are a multiple of 4 or 8.; 
   // value 4 (instead of our 1) is default
}

void reshapeFunc(int w, int h)  
   Word wrs("reshapeFunc(int w, int h)");
   cpmmessage(wrs&" called, w="&cpm(w)&" h="&cpm(h));
   glToglut(w, h);
}

void keyFunc( unsigned char key, int x, int y )
{
   static R n2=2;
   static R fac=::sqrt(n2);
   static R facInv=1./fac;
   using namespace CpmGuiPadyan;
   if (key=='q'|| key=='Q'){
      cout<<"run canceled via key q"<<endl;
      exit(0);
   }
   else if (key=='T'){ // increase time step
      // t and T stand for timestep
      // T for making larger, t for making smaller
      R dt=sp->get_dt();
      dt*=fac;
      sp->set_dt(dt);
   }
   else if (key=='t'){ // decrease time step
      R dt=sp->get_dt();
      dt*=facInv;
      sp->set_dt(dt);      
   }  
   else if (key=='V'){ // increase vCrit
      SysControl::vCrit*=fac;
   }
   else if (key=='v'){ // decrease vCrit
      SysControl::vCrit*=facInv;
   }  
   else if (key=='A'){ // increase aCrit
      SysControl::aCrit*=fac;
   }
   else if (key=='a'){ // decrease aCrit
      SysControl::aCrit*=facInv;
   }     
   else if (key=='s'){ // stop: set particle velocities to zero
      sp->stopParticles();
   } 
   else if (key=='w'){ // wait
      cpmwait(5.);
   } 
   else ;
}

void idleFunc( void )
{
   using namespace CpmGuiPadyan;   
   if (!prepared){
      cpmmessage("idleFunc() started");
      Z nx=glutGet(GLUT_WINDOW_WIDTH);
      Z ny=glutGet(GLUT_WINDOW_HEIGHT);
      Word mes0=loc&" nx="&cpm(nx)&", ny="&cpm(ny);
      cpmmessage(mes0);       
      glToglut(nx, ny);
      CpmGraphics::Viewport(nx,ny,Cpm_args[1]);
      bool vi=CpmGraphics::Viewport::isInitialized();
      if (!vi) cpmerror(loc&"not able to initialize Viewport");
      prepared=true;
   }
   stationRun();   
}

int main(Z argc, char *argv[])
   // notice C-native argument types in order to not irritate the compiler
   // who in this case
   // even does not allow typedef aliases
{
   cout<<"function main() started for program "<<argv[0]<<endl;
   Cpm_args=V<Word>(argc);
   for (Z i=1;i<=argc;i++) Cpm_args[i]=argv[i-1];
   loc=Word("main()");
   loc=loc&cpm(argc)&", "&Cpm_args[1]&"): ";
   glutInit( &argc, argv );
   glutInitDisplayMode( GLUT_SINGLE | GLUT_RGB );   
   glutCreateWindow(argv[0]);
   int gdb=glutGet(GLUT_WINDOW_DOUBLEBUFFER);
   cout<<"GLUT_WINDOW_DOUBLEBUFFER="<<gdb<<endl;
   int dmp=glutGet(GLUT_DISPLAY_MODE_POSSIBLE);
   cout<<"GLUT_DISPLAY_MODE_POSSIBLE="<<dmp<<endl;
   int wbs=glutGet(GLUT_WINDOW_BUFFER_SIZE);
   cout<<"GLUT_WINDOW_BUFFER_SIZE="<<wbs<<endl;
   glutFullScreen();
   glutReshapeFunc(reshapeFunc);
   glutDisplayFunc( displayFunc );
   glutIdleFunc( idleFunc );   
   glutKeyboardFunc( keyFunc );
   glutMainLoop();
}

} // namespace

#endif
