//? cpmlatex.cpp
//? C+- by Ulrich Mutze. Status of work 2022-03-21.
//? Copyright (c) 2020 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//? 
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//? 
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for 
//? more details.


/*
Purpose: Implementation of graphic classes Frame and Figure
version which uses Microsoft Visual C++
2001-3-21: standard vector used to improve the free the
implementation of polygonclip on constant N_EXCESS since this
created an unexpected misbehavior
2002-2-11:
bool Figure::setXY(const R_Vector& x, const R_Matrix& y)
re-implemented based on function R_Vector::infSup(). Number
IMMENSE no longer needed for that.
*/
#include <cpmv.h>
#include <cpmsystemdependencies.h>
#include <cpmsystem.h>
#include <cpmlatex.h>

#include <vector>

using CpmLinAlg::R2;
using CpmLinAlg::Z2;
using std::vector;

using namespace CpmLatex;
using namespace CpmRoot;
using namespace CpmSystem;
using namespace CpmArrays;
using namespace CpmFunctions;

// some badly treated method parameters

/////////////////////////// class Figure ///////////////////////////////


bool Figure::printOn( ostream& str) const
// return preliminary
//notice that content is not copied and retrieved !!!!
{
   CpmRoot::writeTitle("Figure",str);
   cpmp(rx_);
   cpmp(ry_);
   cpmp(nx_);
   cpmp(ny_);
   cpmp(tit_);
   cpmp(withOri_);
   cpmp(newGrid_);
   cpmp(autoScl_);
   cpmp(gridText_);
   cpmp(gridTextPos_);
   return true;
}

bool Figure::scanFrom (istream& str)
{
   cpms(rx_);
   cpms(ry_);
   cpms(nx_);
   cpms(ny_);
   cpms(tit_);
   cpms(withOri_);
   cpms(newGrid_);
   cpms(autoScl_);
   cpms(gridText_);
   cpms(gridTextPos_);
   return true;
}


void Figure::setWithOrigin(Z c)
{
   withOri_=c;
}

void Figure::setNewGrid(Z c)
{
   newGrid_=c;
}

void Figure::setAutoScale(Z c)
{
   autoScl_=c;
}

void Figure::setGridText(Word a)
{
   gridText_=a;
}

void Figure::setGridTextPosition(const R2& pos)
// pos is understood in the Frame coordinate system (xF,yF)
{
   gridTextPos_=pos;
}

void Figure::setGridTextPosition(R xText, R yText)
// pos is understood in the Frame coordinate system (xF,yF)
{
   R2 pos(xText,yText);
   gridTextPos_=pos;
}

// Setting the window according to the size of objects to be
// represented graphically. Overloading with differently
// typed arguments may be added later according to need

namespace{

void splitVal(R& a, R& b)
// functionality considerably changed 2003-09-05
// in order not to collapse histograms of tiny quantities such as
// the magnetic moments of developer particles (in SI units !)
{
// precondition: a<=b
// postcondition: a<b
   const R smallest=1e-16; // changed from 1e-6 to 1e-16
   if (a>b) cpmerror("splitVal(R& a, R& b), a > b not allowed");
   if (a==b){ // changed from cpabs(a-b)<smallest to a==b
      a-=smallest;
      b+=smallest;
   }
}

}

void Figure::setXY(const R_Vector& x, const R_Matrix& y)
/*
   (x,y) represents a family of curves. x contains the x-values
   and y[i][j] are the y-values belonging to x=x[i].
   The values of CL.x1 CU.x1 CL.x2 CU.x2 are set such that these curves
    fill the window exactly
*/
{
   const Z mL=4;
   Word loc("Figure::setXY(R_Vector,R_Matrix)");
   CPM_MA
   Z n=x.dim(),  m=y.dim1(), n1=y.dim2(), i;
   if (n!=n1) cpmerror("Figure::setXY(R_Vector,R_Matrix): n!=n1");
   cpmassert(n>0,loc);

   Iv ivx=x.infSup();
   R x_min=ivx.inf(), x_max=ivx.sup();
   splitVal(x_min,x_max);

   R_Vector yMin(m);
   R_Vector yMax(m);
   for (i=1;i<=m;i++){
      Iv ivi=y[i].infSup();
      yMin[i]=ivi.inf();
      yMax[i]=ivi.sup();
   }
   R y_min=yMin.inf();
   R y_max=yMax.sup();
   splitVal(y_min,y_max);

   if (withOri_==1){ // This allows make the point y=0 appear in
                  // the graph
      if (y_min>0.){
          y_min=0. ;
      }
      else if (y_max<0.){
         y_max=0. ;
      }
   }
   rx_=Iv(x_min,x_max);
   ry_=Iv(y_min,y_max);

   CPM_MZ
}

void Figure::setXY(const R_Vector& x, const R_Vector& y)
{
   R_Matrix yM(1);
   yM[1]=y;
   setXY(x,yM);
}

void Figure::setXY(const R_Matrix& y)
{
   R_Vector x=y[1];
   Z m=y.dim1()-1;
   if (m==1){ // y has only two lines and represents one curve
      R_Vector yy;
      yy=y[2];
      setXY(x,yy);
   }
   else{   // y has more than two lines and represents a family
      // of curves
      R_Matrix yy(m);
      for (Z i=1;i<=m;i++) yy[i]=y[i+1];
      setXY(x,yy);
   }
}

bool Figure::indFunc(const R2& p)const
{
   return rx_.contains(p[1])&&ry_.contains(p[2]);
}

/*
void Figure::grid(Z n_x, Z n_y)
// Draws a grid by dividing x-range in n_x equally sized parts, and
// y-range into n_y such parts.
{
   if (cpmverbose>4) cpmmessage("grid() started");
   if (n_x<1) n_x=1;
   if (n_y<1) n_y=1;

   R h_x=1./n_x;
   R h_y=1./n_y;
   Z i,k;

   for (i=1; i<n_x; i++){ // vertical lines, lines of constant x
      k=ix(i*h_x);
      line(k, pL.x2, k, pU.x2, gridColor);
   }

   for (i=1; i<n_y; i++){ // horizontal lines, lines of constant y
      k=jy(i*h_y);
      line(pL.x1, k, pU.x1, k, gridColor);
   }

// vertical border lines
   line(pL.x1, pL.x2, pL.x1, pU.x2, gridColor);
   line(pU.x1, pL.x2, pU.x1, pU.x2, gridColor);

// horizontal border lines
   line(pL.x1, pL.x2, pU.x1, pL.x2, gridColor);
   line(pL.x1, pU.x2, pU.x1, pU.x2, gridColor);
   // here it is evident that we have a clean grid with no
   // 1-pixel gaps possible if lines actually fill their starting
   // and end pixel

   writeGridText();
}

void Figure::adjustXY(Z divX, Z divY)
// Enlarges the window such that the boundaries correspond to values
// which in scientific number format give short and convenient
// numbers. Also a grid is drawn which subdivides the window
// conveniently into approximately divX subintervals in the x-axis
// and divY subintervals in the y-axis. The actual grid may deviate
// from these numbers since only convenient values for the grid line
// coordinates are accepted (for instance, the unit interval will
// be devided into 2 or 4 subintervals but never in 3).
{
   R2 a;
   Z nX=ladder(CL.x1,CU.x1,divX,a);
   CL.x1=a.x1;
   CU.x1=a.x2;
   Z nY=ladder(CL.x2,CU.x2,divY,a);
   CL.x2=a.x1;
   CU.x2=a.x2;
   updatedXdY();
    // nX and nY are assured not to be 0
   R xStep=(CU.x1-CL.x1)/nX;
   R yStep=(CU.x2-CL.x2)/nY;
   ostringstream ost;
   ost<<"x:"<<CL.x1<<"("<<xStep<<")"<<CU.x1<<
       " y:"<<CL.x2<<"("<<yStep<<")"<<CU.x2;
   gridText=Word(ost.str());
   if (newGrid){
      paint(colorActive);
      grid(nX,nY);
      writeGridText();
   }
}

*/

bool Figure::show(Word const& fileName, R_Vector const& x,
   R_Vector const& y, Word const& title, R xPosRel)
{
   Word loc("Figure::show(...)");
   OFileStream of(fileName);
   if (autoScl_){
      setXY(x,y);
   }

      Z n=x.dim();
      cpmassert(n==y.dim(),loc);
      bool thick=true;
      bool limited=true;
      for (Z i=1;i<=n;i++){
         R2 p1(x[i],0.);
         R2 p2(x[i],y[i]);
       //  drawClip(p1,p2,cFinal,thick);
            // drawClip sometimes misplaces a bar that should
            // coincide with the rightmost border line into a line that
            // is by 1 pixel off frame, the normal draw-function
            // works OK, also for withOrigin=0
         draw(p1,p2,cFinal,limited,thick);
      }
   if (newGrid) writeGridText();
   addText(title,xPosRel);
   display(writeOnShow);
   return true;
}

void Figure::writeGridText(void)
// writes the grid text
{
   const Z sizeLimitUpper=16;
   const Z sizeLimitLower=9;
   const Z font=1;
   Z ti=ix(gridTextPosition.x1);
   Z tj=jy(gridTextPosition.x2);
   Z tchar;
   tchar=(Z)(0.15*cpmabs((R)(jy(1)-jy(0))));
   if (tchar>sizeLimitUpper) tchar=sizeLimitUpper;
   if (tchar<sizeLimitLower) tchar=sizeLimitLower;
   outtextxy(ti,tj,-gridText,tchar,font);
   setFigureColor(graphColor);
}

void Figure::addText(const Word& text, R xText, R yText, Z font,
   Z charSize)
{
   const Z sizeLimitUpper=20;
   const Z sizeLimitLower=11;
   R2 pos(xText,yText);
   Z ti=ix(pos.x1);
   Z tj=jy(pos.x2);
   Z tchar;
   if (charSize==0){
      tchar=(Z)(0.15*cpmabs((R)(jy(1)-jy(0))));
      if (tchar>sizeLimitUpper) tchar=sizeLimitUpper;
      if (tchar<sizeLimitLower) tchar=sizeLimitLower;
   }
   else{
      tchar=charSize;
   }
   outtextxy(ti,tj,-text,tchar,font);
}

void Figure::addFrame(void)
{
   Z k=pL.x1;
   line(k,pL.x2,k,pU.x2,graphColor);
   k=pU.x1;
   line(k,pL.x2,k,pU.x2,graphColor);
   k=pL.x2;
   line(pL.x1,k,pU.x1,k,graphColor);
   k=pU.x2;
   line(pL.x1,k,pU.x1,k,graphColor);
}
