//? cpmregzah.cpp
//? C+- by Ulrich Mutze. Status of work 2022-03-21.
//? Copyright (c) 2020 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//? 
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//? 
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for 
//? more details.


/*
   cpmregzah.cpp
*/
#include <cpmregzah.h>
#include <cpmalgorithms.h>
#include <cpmrfunc.h>
#include <cpmrecord.h>

using namespace CpmRoot;
using namespace CpmGeld;
using CpmAlgorithms::rootByBisection;
using namespace CpmStd;
using CpmFunctions::F;
#ifdef CPM_Fn
using CpmFunctions::F3;
using CpmFunctions::F5;
#endif
using CpmFunctions::R_Func;
using CpmRootX::Text;
using CpmArrays::V;

R RegZah::monZah()const
{ return sum/per;}

bool RegZah::prnOn(ostream& str)const
{
  // cpmwt("RegZah");
   str<<endl;
   cpmp(name);
   cpmp(typ);
   cpmp(sch);
   cpmp(sum);
   cpmp(per);
   return true;
}

bool RegZah::scanFrom(istream& str)
{
   Word::setScanLine(true);
   cpms(name);
   cpms(typ);
   cpms(sch);
   Word::setScanLine(false);
   cpms(sum);
   cpms(per);
   return true;
}

Z RegZah::com(const Type& rz)const
// we don't use the key words for comparison
{
   if (name<rz.name) return 1;
   else if (name>rz.name) return -1;
   else if (typ<rz.typ) return 1;
   else if (typ>rz.typ) return -1;
   else if (sum<rz.sum) return 1;
   else if (sum>rz.sum) return -1;
   else if (per<rz.per) return 1;
   else if (per>rz.per) return -1;
   else return 0;
}

bool RegZah::writeFormatted(ostream& str)const
      // writes the content of *this  in such a
      // way to a stream, that the following read function can
      // recover it from there.
{
   return true;
}

bool RegZah::readFormatted(istream& str)
{
   Z mL=1;
   Word loc="RegZah::readFormatted(istream)";
   Word::setScanLine(true);
   bool suc=name.scanFrom(str);
   cpmmessage(mL,loc&": name read as "&name);
   if (!suc){
      cpmwarning("did not read a name");
      return false; // only for avoiding a warning
   }
   suc=typ.scanFrom(str);
   cpmmessage(mL,loc&": typ read as "&typ);
   if (!suc){
      cpmwarning("did not read a typ");
      return false; // only for avoiding a warning
   }
   V<Word> sw=Text::getLine(str).rep();
   Z i,n=sw.dim();
   sch=S<Word>();
   for (i=1;i<=n;++i){
      sch.add_(sw[i]);
   }
  // for (i=1;i<=n;++i) sch.add_(sw[i]);
   Root<R> sumIn;
   suc=sumIn.scanFrom(str);
   if (!suc){
      cpmwarning("did not read a sum");
      return false; // only for avoiding a warning
   }
   else{
      sum=sumIn();
      cpmmessage(mL,loc&": sum read as "&cpm(sum));
   }
   Root<R> perIn;
   suc=perIn.scanFrom(str);
   if (!suc){
      cpmwarning("did not read a period");
      return false; // only for avoiding a warning
   }
   else{
      per=perIn();
      cpmmessage(mL,loc&": per read as "&cpm(per));
   }
   return true;
}

R CpmGeld::zins(R k, R p, R t)
{
   return k*p*0.01*zinsJahrInv*t;
}

R CpmGeld::zinsPlus(R k, R p, R t)
{
   return k+zins(k,p,t);
}

R CpmGeld::zinsesZins(R k, R p, R t)
{
   Z mL=3;
   Word loc("CpmGeld::zinsesZins(R,R,R)");
   CPM_MA
   cpmassert(t>=0,loc);
   R res=k;
   R tJ=t*zinsJahrInv;
   R tn=cpmfloor(tJ);
   R tr=tJ-tn;
   R fp=p*0.01;
   R fr=fp*tr+1;
   res*=fr;
   if (tn>0) res*=cpmpow(fp+1,tn);
   CPM_MZ
   return res;
}

R CpmGeld::anLage(R k0, R p, R anl, R dtAnl, R t)
{
   R res=k0;
   R tAnl=dtAnl;
   while (tAnl<=t){
      res+=zins(res,p,dtAnl);
      res+=anl;
      tAnl+=dtAnl;
   }
   return res;
}

R CpmGeld::anLageV(V< X2<Greg,R> > const& v, R p, Greg const& tEnd)
{
   Z i,n=v.dim();
   R tf=tEnd.getMJD();
   R sum=0;
   for (i=1;i<=n;++i){
      R ti=(v[i]).c1().getMJD();
      R tfi=tf-ti;
      if (tfi<0) continue;
         // Wir beruecksichtigen natuerlich nur die Einzahlungen
         // die zum Zeitpunkt tEnd schon gemacht sind. Naehmen
         // wir die Zeitpunkte als aufsteigend geordnet an, so
         // koennte continue durch break ersetzt werden.
      R ki=(v[i]).c2();
      sum+=zinsesZins(ki,p,tfi);
   }
   return sum;
}

namespace{
   R diffFunc(const R& p, const R& k1, const R& k2, const R& anl,
      const R& dtAnl, const R& t)
   {
      R res=k2-anLage(k1,p,anl,dtAnl,t);
      return res;
   }

   R diffFuncV(R const& p, V< X2<Greg,R> > const& v, R const& k,
      Greg const& t)
   {
      return k-anLageV(v,p,t);
   }
}

R CpmGeld::rendite(const Greg& t1, const Greg& t2, R k1, R k2, R anl)
{
  R t=t2-t1;
  R dtAnl=30;
#ifdef CPM_Fn
  R_Func f=F5<R,R,R,R,R,R,R>(k1,k2,anl,dtAnl,t)(diffFunc);
#else
  R_Func f=F<R,R>(bind(diffFunc,_1,k1,k2,anl,dtAnl,t));
#endif
  Z iterMax=50;
  R acc=1e-3;
  R pMin=-200; // 0 als schlechtesten Fall anzunehmen waere
      // sehr optimistisch!
  R pMax=200;
  R p=rootByBisection(f,pMin,pMax,acc,iterMax);
  return p;
}

R CpmGeld::renditeV(V< X2<Greg,R> > const& v, R k, Greg const& t)
{
#ifdef CPM_Fn
  R_Func f=F3<R,V< X2<Greg,R> >,R,Greg,R>(v,k,t)(diffFuncV);
#else
  R_Func f=F<R,R>(bind(diffFuncV,_1,v,k,t));
#endif
  Z iterMax=100;
  R acc=1e-2;
  R pMin=-200; // 0 als schlechtesten Fall anzunehmen waere
      // sehr optimistisch!
  R pMax=200;
  return rootByBisection(f,pMin,pMax,acc,iterMax);
}
