//? tut1.cpp
//? C+- by Ulrich Mutze. Status of work 2022-0nw-06.
//? Copyright (c) 2022 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//?
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//?
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for
//? more details.


/*************************************************************************
   Comparing std::vector and CpmArays::V with respect to speed
   of copy-construction and assignment. Notice that this drastic
   advantage of CpmArays::V over std::vector still exists in C++11, where
   moving instead of copying is said to be done where appropriate.
*********************************************************************/
#include <cpmbas.h>
#include <cpmv_.h>
#include <cpmv__.h>
#include <vector>
#include <valarray>
#include <quadmath.h>
/*************************************************************************
1. C+- filenames start with 'cpm' and contain neither underscores nor
   blanks. Therefore, even projects employing files from many sources are
   not likely to run into problems with non-unique file names.

This directive includes all C+- header files needed here;
'bas' stands for 'basics'. Notice that we have not written
#include "cpmbas.h"
and thus have assumed that our C++ project - let it be named tut1 -
has a list of include directories defined.
Who wants to use C+- for more than a single project, should hold
C+- files in directories separate from his project files.
The general purpose C+- files to be used in the project tut1 are on my
computer in xxx/cpm/cpm0/include and xxx/cpm/cpm0/source.
The files that are specific for the tut1 application are in
yyy/tut1/include and yyy/tut1/source.
In yyy/tut1/include there are the project specific headers
among which there need to be two C+- related configuration files:
cpmdefinitions.h and cpmsystemdependencies.h. Customizing these files
allows us to control the behavior of the C+- classes in our project, as
will be explained later.
In yyy/tut1/source the present project-defining main source tut1.cpp is to
be placed.
Now we are in a position to define the file content of project
tut1: Its include directories have to be set as
yyy/tut1/include and xxx/cpm/cpm0/include
and the files to be compiled (translation units) have to be chosen as
yyy/tut1/source/tut1.cpp and xxx/cpm/cpm0/source/cpmbas.cpp.

1.1 Details (skip on first reading ?):
The file cpmbas.cpp is actually a collection of all other files
in xxx/cpm/cpm0/source. Its content is

#include "cpmc.cpp"
#include "cpmangle.cpp"
#include "cpmv.cpp"
#include "cpmgreg.cpp"
#include "cpmsystem.cpp"
#include "cpmtypes.cpp"
#include "cpmuc.cpp"
#include "cpmzinterval.cpp"
#include "cpmnumbers.cpp"
#include "cpmword.cpp"
#include "cpmviewport.cpp"
End of 1.1
*********************************************************************/
using namespace CpmRoot;
/*************************************************************************
2. C+- namespace names start with 'Cpm' followed by an identifier
   which starts with a capital letters.

The namespace CpmRoot is rather small, so that the present using directive
should be applicable also in projects in which classes from various
sources are being used.

3. The C+- names for integer, real, and complex numbers are
   CpmRoot::Z, CpmRoot::R, Cpmoot::C. CpmRoot::Word wraps std::string and
   adds functionality to it.

The main effect of this using directive is that we may use these names
simply as Z, R, C, Word.

3.1 Details (skip on first reading ?):
There are the less ubiquitous types L, and N in CpmRoot:
L for unsigned characters ('L' for 'letter', after 'integer promotion'
the values range from 0 to 2nwnw), N for 'natural numbers', i.e. unsigned
integers. Types L,Z, and N are typedefs for unsigned char, int,
unsigned int. If we add in file cpmdefinition.h the macro
#define CPM_LONG, we change Z to long int, and N to unsigned long int.
If in cpmdefinitions.h the macro CPM_MP is defined, R has the meaning of
mpreal as defined by the wrapper library MPFRC++ to the multiple precision
library mpfr. See comments to CPM_MP in file cpmdefinitionswrc.h. If
CPM_MP is not defined R is also a typedef, long double if CPM_LONG and
double else. The types bool and string from standard C++ are useful as
they stand. Nevertheless they will be wrapped into classes - CpmRootX::B
in cpmtypes.h and class CpmRoot::Word in file cpmword.h.
End of 3.1

There is a namespace CpmRootX for the 'less essential essentials':

4. There are classes CpmRootX::B, CpmRootX::Z1, CpmRootX::R1,  for
   Boolean values, integer, and real numbers, which may replace the
   non-class types bool, Z, and R in situations where class func-
   tionality, such as automatic initialization, is indispensable.
   Note 2017-02-18: With C++11 automatic initialization can be achieved
   also for the built-in types. For instance a data member of a class X
         bool b {false};
   can be handeled as if we had declared it as
         B b;

4.1 Details (skip on first reading ?):
Namespace CpmRoot in mainly layed out in file cpmnumbers.h
and CpmRootX in cpmtypes.h. The vigilant reader may argue that we
have not available the declarations of CpmRoot, since we have no
   #include <cpmnumbers.h>
seen so far. Actually, it is there since file cpmbas.h is the following
collection of include directives
#ifndef CPM_BAS_H_
#define CPM_BAS_H_
   #include <cpmtypes.h>
   #include <cpmfr.h>
   #include <cpmvr.h>
   #include <cpmsr.h>
   #include <cpmm.h>
   #include <cpmp.h>
   #include <cpmc.h>
   #include <cpmangle.h>
   #include <cpmgreg.h>
#endif
Also here we don't see
   #include <cpmnumbers.h>
but inspecting cpmtypes.h, we find the commented directive
   #include <cpmsystem.h> // includes cpmwords.h and
      // thus also cpmnumbers.h
which shows the steps which finally include cpmnumbers.h. Acually the
directory cpm/cpm0/include contains 32 header files. This set of files
(as any set of C++ header files) is a directed graph in a natural manner:
There is an edge leading from file1 to file2 iff file2 contains the
directive  #include <file1> .
The files that can be reached from file1 along a path of that graph depend
on file1 and every translation unit which includes such a file1-dependent
header has to be re-compiled upon some change in file1. So, any system
capable of generating proper make files has to do this dependency
analysis and thus has to work with this  d e p e n d e n c y  g r a p h.
I made a tool consisting of a Ruby program for file analysis
and a C+- program for graph analysis and representation which
creates a  pictorial representation of this dependency graph of an
arbitrary collecton of C/C++ header files. This tool turned out to be very
useful.
It is an obvious logical requirement that the dependency graph is free of
cycles. It is a non-trivial task to organize the header dependencies in a
way that each translation unit gets the declarations which it needs to
know by inclusion of only a few header files. For instance, cpmword.cpp
and cpmtypes.cpp need only cpmtypes.h; cpmc.cpp needs cpmc.h and
cpmtypes.h. The present header file hierarchy is the result of many
simplifying re-organizations. However, many attempts of a simplification
failed since they entailed unacceptable complications elsewhere. I'm not
aware of tools which would automatize relevant parts of this organization
process.
End of 4.1
***********************************************************************/
using namespace CpmArrays;
/*************************************************************************
nw. The general-purpose array in C+- is the template class CpmArrays::V
   Most constructors set the first valid index of a non-void V is 1 and
   not 0 as for std::vector. There is, however, a member function which
   shifts all indexes and so allows indexing to start with 0. This allows
   identical code to be used for these two types of arrays.

nw.1 Details on C+- namespaces (skip on first reading ?):
The C+- class system introduces many namespaces. Their belonging
togeher under the Cpm banner is not expressed by making them
sub-namespaces of a single Cpm-namespace. Instead, their names all
start with 'Cpm' and continue with a capital letter.
If the name contains a digit, this is 2 or 3 and refers to the space
dimension under consideration. So the namespace CpmDim2 contains the
2D ('flatland') analogs of the geometric classes defined in
namespace CpmDim3.
The namespaces which are declared in the present scope by including
cpmbas.h are
CpmRoot, CpmRootX, CpmSystem, CpmArrays, CpmFunctions, CpmGeo,
CpmGraphics, CpmMPI, CpmStd, CpmTests, CpmTime.
End of nw.1
*********************************************************************/

void info();
N tutorial1(N,N,N);
   // Declaration of two functions, the definition of which will
   // determine the functionality of the program.#
//N tutorial2(N,N);

int main(int argc, char* argv[])
   // Traditional main function with types of arguments and return value
   // according to C/C++. Not all compilers accept using Z instead of
   // int here.
{
  //  N m=100, n=100;  //default values to be used if no input from the
   N m=2000, n=2000, diff=1; // default values to be used if no input from the
      //  command line can be found
      // nw000, nw000 works fast for CPM_R but not for CPM_RLONG
      // 2022-10-10 timing series for m=2000, n=2000 with O3 optimizing
      // with the new fixed precisions FLOAT and QUAD
      // CPM_FLOAT  t12+t23 V = 0.439032
      // CPM_DOUBLE t12+t23 V= 0.438204
      // CPM_LONG   t12+t23 V= 0.nw67039
      // CPM_QUAD   t12+t23 V= nw.336nw3
      // CPM_MP 32  t12+t23 V= 8.80929


   V<Word> arg=comLine(argc,argv); // CpmRoot::Word is similar to
      // std::string. CpmArrays::V is the C+- array type. There are two
      // 'schools' about indexing arrays. The 'C school' lets valid
      // indexes start with 0 and the 'Fortran school' which lets valid
      // valid indexes start with 1. Till September 2010 C+- had a
      // 'C school'-array Vl (l for 'lean') and a 'Fortran school'-array
      // V , where the implementation of V was such that any instance of
      // V had a data member of type Vl. The type Vl was used mostly
      // internally in situation where efficiency was considered to be
      // of utmost importance. Finally it became clear that the decision
      // to have these two types of arrays was a mistake. It creates a
      // permanent uncertainty for the programmer whether he should use
      // the default type or should strive for utmost efficiency by
      // using the lean version. Now we have only a single array type V
      // for which the valid indexes may form any contiguous set of
      // integers. This index range is set to start with 1 in most
      // of the available constructors but can be very conveniently reset
      // to any other start index such as 0.
      // CpmArrays::comLine is a function which transforms the
      // traditional argument of main into the more functional data type
      // of C+-. The function name 'comLine' results from
      // the  d e s c r i p t i v e  f u l l  n a m e (DFN)
      // 'command line' by a simple deterministic rule: The rule is
      // to leave unchanged all words with up to four letters and shorten
      // all longer words to 3 by taking the first two letters as they are
      // and take the first of the following consonants as the third
      // letter. Thus 'ooops' gets converted to 'oop'. If there is no
      // such consonant, the vowel at place 3 has to be taken: 'hooiii'
      // gets converted to 'hoo'. In a chain of words, the contracted
      // components get concateneted in a style which is evident from
      // the example that 'descriptive full name' goes to
      // 'desFullName'. In most cases the names come out quite nice, but
      // there are exceptions (not to the rules!): I consider it ugly to
      // see 'field' abbreviated to 'fil' (notice that 'file', as having
      // only four letters, remains 'file')
      // The simplicity of the rule allows us to use the the function name
      // fluently whenever we remember the full name correctly. Of course,
      // the descriptive full name of a function has to be stated in the
      // declaration of this function. In our case the declaration is in
      // file cpmv.h:
      // inline V<Word> comLine(int argc, char* argv[])
      //    //: command line
      // The colon hints at the fact that here a descriptive full name
      // is being communicated.

      N na=arg.size();

      if(na==1){ return tutorial1(m,n,diff);}
      else if(na==2){
         Word w2=arg[2];
         if(w2=="?" ){
            info(); return 0;
         }
         else{
            m=w2.toZ();
            n=m;
         }
      }
      else if(na==3){
         Word w3=arg[3];
         m=arg[2].toZ();
         n=w3.toZ();
      }
      else if(na==4){
         Word w4=arg[4];
         m=arg[2].toZ();
         diff=w4.toZ();
      }
      else{
         ; // m and n have there initial values
      }
      cout<<"next line calls tutorial1(m,n,diff)"<<endl;
      return tutorial1(m,n,diff);
  }

using namespace std;

template<typename C>
C cz(N i) // an ad-hoc function N --> C
{
   static R pi=cpmpi;
   static R u=R(1.);
   R ir=i+1.;
   //R pi=std::numbers::pi_v<R>;
   //R pi=3.141nw926nw3nw89793238462643;
   return C(ir/(u+ir*ir),ir*pi);
      // calling constructor C(R,R)
}

template<typename T, template<typename> class Vec, N fi>
V<R> perMes(N m, N n)
//: performance measure
// Intent: We want to evaluate how efficient the copy and assign operations for
// array ('vector') classes work for large objects (long arrays and large
// objects that make up the components of the arrays. I suggests itself to
// choose the large objects again as large arrays. The  components of these
// 'second order arrays' are not chosen as a primary builtin type such as double
// or int. Instead we use as the complex numbers for which there are
// implementations in namespaces std and CpmRoot.
// The various objects mentioned above can all treated by a function which takes
// types as arguments i.e. is a function template in C++ parlance.
// For these template arguments T, Vec, N we will finally (i.e. in function tutorial1)
// use the following concrete realisations:
//    T: std::complex<R>, CpmRoot::C
//    Vec: std::vector, std::valarray, CpmArrays::V, CpmArrays::V_, CpmArrays::V__,
//    N: 0 for Vec=vector,valarray,V_,V__ and 1 for V
// The function argument m sets the length of the tested arrays, and n sets the
// length of the arrays which figure as the components of the tested arrays.
{
   Z mL=1;
/*
   This line, together with the following
   Word loc(...);
   CPM_MA
   CPM_MZ
   consitute a convenient idiom for signaling
   entry to and exit from a function block to the log file
   cpmcerr.txt. This writing to the log file takes place only if
   mL is larger or equal to the static data member
   CpmSystem::Message::verbose. This bulky quantity has a
   macro alias 'cpmverbose' (i.e. we have, in file cpmsystem.h
       #define cpmverbose      CpmSystem::Message::verbose
   ) and it can be set by a statement like
       cpmverbose=10;
   Its initial value is 2.
   Soon we will encounter macros cpmtime, cpmwait, cpmdebug.
  */
   Word loc("perMes(Z,Z)"); // messages use this as name of the function
   CPM_MA // defined in cpmmacros.h, assumes that 'mL' and 'loc' are
      // defined
   R t1=cpmtime(); /* time of function call in seconds from some
       system-defined 'point-zero in time'.
       cpmtime is a short name for function CpmSystem::time defined
       in file cpmsystem.h as follows:
           #define cpmtime         CpmSystem::time
   */
   N i,j;
   N ni=fi;
   N nf=fi+n-1;
   N mi=fi;
   N mf=fi+m-1;

   Vec<Vec<T>> vi, vf;
   Vec<T> v1(m);
   Vec<T> v2(m);
   for (i=mi;i<=mf;++i) v1[i]=cz<T>(i-mi);
   for (i=mi;i<=mf;++i) v2[i]=v1[i]*v1[i];
         // The preferred form of this statement in C+- is
         // for (i=v1.b();i<=v1.e();++i) v1[i]=cz(i);
         // Written in this form, it is alo correct if v1 is of type V<C>,
         // the C+- array for which indexing starts with 1 instead of 0.
   Vec<Vec<T>> w1(n); for(i=ni;i<=nf;++i) w1[i]=v1;
   Vec<Vec<T>> w2(n); for(i=ni;i<=nf;++i) w2[i]=v2;

   Vec<Vec<T>> temp1=w1; // copy constructor
   Vec<Vec<T>> temp2=temp1; // copy constructor
   Vec<Vec<T>> temp3=temp2; // copy constructor
   Vec<Vec<T>> temp4=temp3; // copy constructor
   Vec<Vec<T>> temp5=temp4; // copy constructor
   Vec<Vec<T>> temp6=temp5; // copy constructor
   Vec<Vec<T>> temp7=temp6; // copy constructor
   Vec<Vec<T>> temp8=temp7; // copy constructor
   Vec<Vec<T>> temp9=temp8; // copy constructor
   Vec<Vec<T>> temp10=temp9; // copy constructor
   temp1=Vec<Vec<T>>(); // assignment
   temp2=w2; // assignment
   temp3=temp2; // assignment
   temp4=w2; // assignment
   temp5=temp4; // assignment
   temp6=w2; // assignment
   temp7=temp3; // assignment
   temp8=temp2; // assignment
   temp9=temp1; // assignment
      // these 9 assignments should neither influence w, nor temp10
   vi=w1; // assignment
   vf=temp10; // assignment
   R t2=cpmtime();
   R t12=t2-t1; // time needed for copy and assignment
   R err=0,sum_v=0;
   for (i=ni;i<=nf;++i){
      Vec<T> vii=vi[i];
      Vec<T> vfi=vf[i];
      for (j=mi;j<=mf;++j){
         T viij=vii[j];
         T vfij=vfi[j];
         err+=abs(viij-vfij);
         sum_v+=abs(vfij);
      }
   } // making sure that copying and assignment worked correctly
   // unfortunately one has to stagger many copy/assign steps
   // make this part of the function the dominant one timewise.
   R t3=cpmtime();
   R t23=t3-t2;
   V<R> res{t12,t23,t12+t23,err,sum_v};
   /*
      Convenient  l i s t  c o n s t r u c t o r, requires C++11
   */
   //bool b=std::movable<Vec<T>>;
   //cout<<"std::movable<Vec<T>> = "<<b<<endl;

  CPM_MZ
  return res;
}


template<typename T, template<typename> class Vec, N fi>
V<R> perMesM(N m, N n)
//: performance measure
// Intent: We want to evaluate how efficient the copy and assign operations for
// array ('vector') classes work for large objects (long arrays and large
// objects that make up the components of the arrays. I suggests itself to
// choose the large objects again as large arrays. The  components of these
// 'second order arrays' are not chosen as a primary builtin type such as double
// or int. Instead we use as the complex numbers for which there are
// implementations in namespaces std and CpmRoot.
// The various objects mentioned above can all treated by a function which takes
// types as arguments i.e. is a function template in C++ parlance.
// For these template arguments T, Vec, N we will finally (i.e. in function tutorial1)
// use the following concrete realisations:
//    T: std::complex<R>, CpmRoot::C
//    Vec: std::vector, std::valarray, CpmArrays::V, CpmArrays::V_, CpmArrays::V__,
//    N: 0 for Vec=vector,valarray,V_,V__ and 1 for V
// The function argument m sets the length of the tested arrays, and n sets the
// length of the arrays which figure as the components of the tested arrays.
{
   Z mL=1;
/*
   This line, together with the following
   Word loc(...);
   CPM_MA
   CPM_MZ
   consitute a convenient idiom for signaling
   entry to and exit from a function block to the log file
   cpmcerr.txt. This writing to the log file takes place only if
   mL is larger or equal to the static data member
   CpmSystem::Message::verbose. This bulky quantity has a
   macro alias 'cpmverbose' (i.e. we have, in file cpmsystem.h
       #define cpmverbose      CpmSystem::Message::verbose
   ) and it can be set by a statement like
       cpmverbose=10;
   Its initial value is 2.
   Soon we will encounter macros cpmtime, cpmwait, cpmdebug.
  */
   Word loc("perMesM(Z,Z)"); // messages use this as name of the function
   CPM_MA // defined in cpmmacros.h, assumes that 'mL' and 'loc' are
      // defined
   R t1=cpmtime(); /* time of function call in seconds from some
       system-defined 'point-zero in time'.
       cpmtime is a short name for function CpmSystem::time defined
       in file cpmsystem.h as follows:
           #define cpmtime         CpmSystem::time
   */
   N i,j;
   N ni=fi;
   N nf=fi+n-1;
   N mi=fi;
   N mf=fi+m-1;

   Vec<Vec<T>> vi, vf;
   Vec<T> v1(m);
   Vec<T> v2(m);
   for (i=mi;i<=mf;++i) v1[i]=cz<T>(i-mi);
   for (i=mi;i<=mf;++i) v2[i]=v1[i]*v1[i];
         // The preferred form of this statement in C+- is
         // for (i=v1.b();i<=v1.e();++i) v1[i]=cz(i);
         // Written in this form, it is alo correct if v1 is of type V<C>,
         // the C+- array for which indexing starts with 1 instead of 0.
   Vec<Vec<T>> w1(n); for(i=ni;i<=nf;++i) w1[i]=v1;
   Vec<Vec<T>> w2(n); for(i=ni;i<=nf;++i) w2[i]=v2;

   Vec<Vec<T>> w1Orig=w1;

   Vec<Vec<T>> temp1=std::move(w1); // copy constructor
   Vec<Vec<T>> temp2=std::move(temp1); // copy constructor
   Vec<Vec<T>> temp3=std::move(temp2); // copy constructor
   Vec<Vec<T>> temp4=std::move(temp3); // copy constructor
   Vec<Vec<T>> temp5=std::move(temp4); // copy constructor
   Vec<Vec<T>> temp6=std::move(temp5); // copy constructor
   Vec<Vec<T>> temp7=std::move(temp6); // copy constructor
   Vec<Vec<T>> temp8=std::move(temp7); // copy constructor
   Vec<Vec<T>> temp9=std::move(temp8); // copy constructor
   Vec<Vec<T>> temp10=std::move(temp9); // copy constructor
   temp1=std::move(Vec<Vec<T>>()); // assignment
   temp2=std::move(w2); // assignment
   temp3=std::move(temp2); // assignment
   temp4=std::move(w2); // assignment
   temp5=std::move(temp4); // assignment
   temp6=std::move(w2); // assignment
   temp7=std::move(temp3); // assignment
   temp8=std::move(temp2); // assignment
   temp9=std::move(temp1); // assignment
      // these 9 assignments should neither influence w, nor temp10
   vi=std::move(w1Orig); // assignment
   vf=std::move(temp10); // assignment
   R t2=cpmtime();
   R t12=t2-t1; // time needed for copy and assignment
   R err=0,sum_v=0;
   for (i=ni;i<=nf;++i){
      Vec<T> vii=std::move(vi[i]);
      Vec<T> vfi=std::move(vf[i]);
      for (j=mi;j<=mf;++j){
         T viij=vii[j];
         T vfij=vfi[j];
         err+=abs(viij-vfij);
         sum_v+=abs(vfij);
      }
   } // making sure that copying and assignment worked correctly
   // unfortunately one has to stagger many copy/assign steps
   // make this part of the function the dominant one timewise.
   R t3=cpmtime();
   R t23=t3-t2;
   V<R> res{t12,t23,t12+t23,err,sum_v};
   /*
      Convenient  l i s t  c o n s t r u c t o r, requires C++11
   */
   //bool b=std::movable<Vec<T>>;
   //cout<<"std::movable<Vec<T>> = "<<b<<endl;

  CPM_MZ
  return res;
}

template<typename T, template<typename> class Vec, N fi>
V<R> perMesMove(N m, N n)
//: performance measure
// Intent: We want to evaluate how efficient the copy and assign operations for
// array ('vector') classes work for large objects (long arrays and large
// objects that make up the components of the arrays. I suggests itself to
// choose the large objects again as large arrays. The  components of these
// 'second order arrays' are not chosen as a primary builtin type such as double
// or int. Instead we use as the complex numbers for which there are
// implementations in namespaces std and CpmRoot.
// The various objects mentioned above can all treated by a function which takes
// types as arguments i.e. is a function template in C++ parlance.
// For these template arguments T, Vec, N we will finally (i.e. in function tutorial1)
// use the following concrete realisations:
//    T: std::complex<R>, CpmRoot::C
//    Vec: std::vector, std::valarray, CpmArrays::V, CpmArrays::V_, CpmArrays::V__,
//    N: 0 for Vec=vector,valarray,V_,V__ and 1 for V
// The function argument m sets the length of the tested arrays, and n sets the
// length of the arrays which figure as the components of the tested arrays.
{
   Z mL=1;
/*
   This line, together with the following
   Word loc(...);
   CPM_MA
   CPM_MZ
   consitute a convenient idiom for signaling
   entry to and exit from a function block to the log file
   cpmcerr.txt. This writing to the log file takes place only if
   mL is larger or equal to the static data member
   CpmSystem::Message::verbose. This bulky quantity has a
   macro alias 'cpmverbose' (i.e. we have, in file cpmsystem.h
       #define cpmverbose      CpmSystem::Message::verbose
   ) and it can be set by a statement like
       cpmverbose=10;
   Its initial value is 2.
   Soon we will encounter macros cpmtime, cpmwait, cpmdebug.
  */
   Word loc("perMesMove(Z,Z)"); // messages use this as name of the function
   CPM_MA // defined in cpmmacros.h, assumes that 'mL' and 'loc' are
      // defined
   R t1=cpmtime(); /* time of function call in seconds from some
       system-defined 'point-zero in time'.
       cpmtime is a short name for function CpmSystem::time defined
       in file cpmsystem.h as follows:
           #define cpmtime         CpmSystem::time
   */
   cout<<"perMesMove entered"<<endl;
   N i,j;
   N ni=fi;
   N nf=fi+n-1;
   N mi=fi;
   N mf=fi+m-1;

   Vec<T> vi, vf;
   Vec<T> v1(m);
   Vec<T> v2(m);
   for (i=mi;i<=mf;++i) v1[i]=cz<T>(i-mi);
   for (i=mi;i<=mf;++i) v2[i]=v1[i]*v1[i];
         // The preferred form of this statement in C+- is
         // for (i=v1.b();i<=v1.e();++i) v1[i]=cz(i);
         // Written in this form, it is alo correct if v1 is of type V<C>,
         // the C+- array for which indexing starts with 1 instead of 0.
   cout<<"first simple block done"<<endl;

   Vec<T> w1(n); for(i=ni;i<=nf;++i) w1[i]=std::move(v1);
   Vec<T> w2(n); for(i=ni;i<=nf;++i) w2[i]=std::move(v2);

   cout<<"second block done"<<endl;

   Vec<Vec<T>> temp1=std::move(w1); // copy constructor
   Vec<Vec<T>> temp2=std::move(temp1); // copy constructor
   Vec<Vec<T>> temp3=std::move(temp2); // copy constructor
   Vec<Vec<T>> temp4=std::move(temp3); // copy constructor
   Vec<Vec<T>> temp5=std::move(temp4); // copy constructor
   Vec<Vec<T>> temp6=std::move(temp5); // copy constructor
   Vec<Vec<T>> temp7=std::move(temp6); // copy constructor
   Vec<Vec<T>> temp8=std::move(temp7); // copy constructor
   Vec<Vec<T>> temp9=std::move(temp8); // copy constructor
   Vec<Vec<T>> temp10=std::move(temp9); // copy constructor
   cout<<"third block done"<<endl;
   temp1=std::move(Vec<Vec<T>>()); // assignment
   temp2=std::move(w2); // assignment
   temp3=std::move(temp2); // assignment
   temp4=std::move(w2); // assignment
   temp5=std::move(temp4); // assignment
   temp6=std::move(w2); // assignment
   temp7=std::move(temp3); // assignment
   temp8=std::move(temp2); // assignment
   temp9=std::move(temp1); // assignment
   cout<<"block 4 done"<<endl;
      // these 9 assignments should neither influence w, nor temp10
   vi=std::move(w1); // assignment
   vf=std::move(temp10); // assignment
   cout<<"block 5 done"<<endl;
   R t2=cpmtime();
   R t12=t2-t1; // time needed for copy and assignment
   R err=0.,sum_v=0.;
   for (N i=ni;i<=nf;++i){
      cout<<"i-loop "<<i<<" started"<<endl;
      Vec<T> vii=std::move(vi[i]);
      Vec<T> vfi=std::move(vf[i]);

      for (N j=mi;j<=mf;++j){
         T viij=vii[j];
         T vfij=vfi[j];
         err+=abs(viij-vfij);
         sum_v+=abs(vfij);
      }
   } // making sure that copying and assignment worked correctly
   // unfortunately one has to stagger many copy/assign steps
   // make this part of the function the dominant one timewise.
   cout<<"block 6 done"<<endl;
   R t3=cpmtime();
   R t23=t3-t2;
   V<R> res{t12,t23,t12+t23,err,sum_v};
   /*
      Convenient  l i s t  c o n s t r u c t o r, requires C++11
   */
   //bool b=std::movable<Vec<T>>;
   //cout<<"std::movable<Vec<T>> = "<<b<<endl;

  CPM_MZ
  return res;
}

template<typename T, template<typename> class Vec, N fi>
V<R> perMesMove0(N m, N n)
{
   Z mL=1;

   Word loc("perMesMove0(Z,Z)"); // messages use this as name of the function
   CPM_MA
   R t1=cpmtime();
 //  cout<<"perMesMove1 entered"<<endl;
   N i,j;
   N ni=fi;
   N nf=fi+n-1;
   N mi=fi;
   N mf=fi+m-1;

   Vec<T> vi, vf;
   Vec<T> w1(m);
   Vec<T> w2(m);
   for (i=mi;i<=mf;++i) w1[i]=cz<T>(i-mi);
   for (i=mi;i<=mf;++i) w2[i]=w1[i]*w1[i];
         // The preferred form of this statement in C+- is
         // for (i=v1.b();i<=v1.e();++i) v1[i]=cz(i);
         // Written in this form, it is alo correct if v1 is of type V<C>,
         // the C+- array for which indexing starts with 1 instead of 0.
   Vec<T> wOrig(m);
   for (i=mi;i<=mf;++i) wOrig[i]=w1[i];

   Vec<T> temp1=w1; // copy constructor
 //  for (i=mi;i<=mf;++i) cout<<"temp1[i] = "<<temp1[i]<<endl;
   //cout<<"temp1 = "<<temp1<<endl;
   Vec<T> temp2=temp1; // copy constructor
   Vec<T> temp3=temp2; // copy constructor
   Vec<T> temp4=temp3; // copy constructor
   Vec<T> temp5=temp4; // copy constructor
   Vec<T> temp6=temp5; // copy constructor
   Vec<T> temp7=temp6; // copy constructor
//   for (i=mi;i<=mf;++i) cout<<"temp7[i] = "<<temp7[i]<<endl;
   Vec<T> temp8=temp7; // copy constructor
   Vec<T> temp9=temp8; // copy constructor
   Vec<T> temp10=temp9; // copy constructor
//   cout<<"third block done"<<endl;
   temp1=w2; // assignment
   temp2=w2; // assignment
   temp3=temp2; // assignment
   temp4=w2; // assignment
   temp5=temp4; // assignment
   temp6=w2; // assignment
   temp7=temp3; // assignment
   temp8=temp2; // assignment
   temp9=temp1; // assignment
 //  cout<<"block 4 done"<<endl;
      // these 9 assignments should neither influence w, nor temp10
   vi=wOrig; // assignment
   vf=temp10; // assignment
 //  for (i=mi;i<=mf;++i) cout<<"vi[i] = "<<vi[i]<<endl;
 //  for (i=mi;i<=mf;++i) cout<<"vf[i] = "<<vf[i]<<endl;
//   cout<<"block 5 done"<<endl;
   R t2=cpmtime();
   R t12=t2-t1; // time needed for copy and assignment
   R err=0.,sum_v=0.;
   for (N i=mi;i<=mf;++i){
     // cout<<"i-loop "<<i<<" started"<<endl;
     // cout<<"vi[i] = "<<vi[i]<<" vf[i] = "<<vf[i]<<endl;
      err+=abs(vi[i]-vf[i]);
      sum_v+=abs(vf[i]);
   }
//   cout<<"block 6 done"<<endl;
   R t3=cpmtime();
   R t23=t3-t2;
   V<R> res{t12,t23,t12+t23,err,sum_v};
   /*
      Convenient  l i s t  c o n s t r u c t o r, requires C++11
   */
   //bool b=std::movable<Vec<T>>;
   //cout<<"std::movable<Vec<T>> = "<<b<<endl;

  CPM_MZ
  return res;
}


template<typename T, template<typename> class Vec, N fi>
V<R> perMesMove1(N m, N n)
{
   Z mL=1;

   Word loc("perMesMove1(Z,Z)"); // messages use this as name of the function
   CPM_MA
   R t1=cpmtime();
 //  cout<<"perMesMove1 entered"<<endl;
   N i,j;
   N ni=fi;
   N nf=fi+n-1;
   N mi=fi;
   N mf=fi+m-1;

   Vec<T> vi, vf;
   Vec<T> w1(m);
   Vec<T> w2(m);
   for (i=mi;i<=mf;++i) w1[i]=cz<T>(i-mi);
   for (i=mi;i<=mf;++i) w2[i]=w1[i]*w1[i];
         // The preferred form of this statement in C+- is
         // for (i=v1.b();i<=v1.e();++i) v1[i]=cz(i);
         // Written in this form, it is alo correct if v1 is of type V<C>,
         // the C+- array for which indexing starts with 1 instead of 0.
   Vec<T> wOrig(m);
   for (i=mi;i<=mf;++i) wOrig[i]=w1[i];

   Vec<T> temp1=std::move(w1); // copy constructor
 //  for (i=mi;i<=mf;++i) cout<<"temp1[i] = "<<temp1[i]<<endl;
   //cout<<"temp1 = "<<temp1<<endl;
   Vec<T> temp2=std::move(temp1); // copy constructor
   Vec<T> temp3=std::move(temp2); // copy constructor
   Vec<T> temp4=std::move(temp3); // copy constructor
   Vec<T> temp5=std::move(temp4); // copy constructor
   Vec<T> temp6=std::move(temp5); // copy constructor
   Vec<T> temp7=std::move(temp6); // copy constructor
//   for (i=mi;i<=mf;++i) cout<<"temp7[i] = "<<temp7[i]<<endl;
   Vec<T> temp8=std::move(temp7); // copy constructor
   Vec<T> temp9=std::move(temp8); // copy constructor
   Vec<T> temp10=std::move(temp9); // copy constructor
//   cout<<"third block done"<<endl;
   temp1=std::move(w2); // assignment
   temp2=std::move(w2); // assignment
   temp3=std::move(temp2); // assignment
   temp4=std::move(w2); // assignment
   temp5=std::move(temp4); // assignment
   temp6=std::move(w2); // assignment
   temp7=std::move(temp3); // assignment
   temp8=std::move(temp2); // assignment
   temp9=std::move(temp1); // assignment
 //  cout<<"block 4 done"<<endl;
      // these 9 assignments should neither influence w, nor temp10
   vi=std::move(wOrig); // assignment
   vf=std::move(temp10); // assignment
 //  for (i=mi;i<=mf;++i) cout<<"vi[i] = "<<vi[i]<<endl;
 //  for (i=mi;i<=mf;++i) cout<<"vf[i] = "<<vf[i]<<endl;
//   cout<<"block 5 done"<<endl;
   R t2=cpmtime();
   R t12=t2-t1; // time needed for copy and assignment
   R err=0.,sum_v=0.;
   for (N i=mi;i<=mf;++i){
     // cout<<"i-loop "<<i<<" started"<<endl;
     // cout<<"vi[i] = "<<vi[i]<<" vf[i] = "<<vf[i]<<endl;
      err+=abs(vi[i]-vf[i]);
      sum_v+=abs(vf[i]);
   }
//   cout<<"block 6 done"<<endl;
   R t3=cpmtime();
   R t23=t3-t2;
   V<R> res{t12,t23,t12+t23,err,sum_v};
   /*
      Convenient  l i s t  c o n s t r u c t o r, requires C++11
   */
   //bool b=std::movable<Vec<T>>;
   //cout<<"std::movable<Vec<T>> = "<<b<<endl;

  CPM_MZ
  return res;
}

template<typename T, template<typename> class Vec, N fi>
V<R> perMesMove2(N m, N n)
{
   Z mL=1;

   Word loc("perMesMove2(Z,Z)"); // messages use this as name of the function
   CPM_MA
   R t1=cpmtime();
 //  cout<<"perMesMove1 entered"<<endl;
   N i,j;
   N ni=fi;
   N nf=fi+n-1;
   N mi=fi;
   N mf=fi+m-1;

   Vec<T> vi, vf;
   Vec<T> w1(m);
   Vec<T> w2(m);
 //  cout<<"got to loc 1"<<endl;
   for (i=mi;i<=mf;++i){
   //   cout<<" index in loop = "<<i<<endl;
      T yi=cz<T>(i-mi);
  //    cout<<" yi in loop = "<<yi<<endl;
      w1[i]=yi;
    //  cout<<" w1[i] in loop done "<<w1[i]<<endl;
      w2[i]=yi*yi;
   }
 //  cout<<"got to loc 2"<<endl;
         // The preferred form of this statement in C+- is
         // for (i=v1.b();i<=v1.e();++i) v1[i]=cz(i);
         // Written in this form, it is alo correct if v1 is of type V<C>,
         // the C+- array for which indexing starts with 1 instead of 0.
   Vec<T> wOrig=w1;
  // Vec<T> wOrig(m);
  // for (i=mi;i<=mf;++i) wOrig[i]=w1[i];
 // cout<<"got to loc 3"<<endl;

   Vec<T> temp1=w1; // copy constructor
 //  for (i=mi;i<=mf;++i) cout<<"temp1[i] = "<<temp1[i]<<endl;
   //cout<<"temp1 = "<<temp1<<endl;
   Vec<T> temp2=temp1; // copy constructor
   Vec<T> temp3=temp2; // copy constructor
   Vec<T> temp4=temp3; // copy constructor
   Vec<T> temp5=temp4; // copy constructor
   Vec<T> temp6=temp5; // copy constructor
   Vec<T> temp7=temp6; // copy constructor
//   for (i=mi;i<=mf;++i) cout<<"temp7[i] = "<<temp7[i]<<endl;
   Vec<T> temp8=temp7; // copy constructor
   Vec<T> temp9=temp8; // copy constructor
   Vec<T> temp10=temp9; // copy constructor
//   cout<<"third block done"<<endl;
   cout<<"got to loc 4"<<endl;
   temp1=w2; // assignment
   temp2=w2; // assignment
   temp3=temp2; // assignment
   temp4=w2; // assignment
   temp5=temp4; // assignment
   temp6=w2; // assignment
   temp7=temp3; // assignment
   temp8=temp2; // assignment
   temp9=temp1; // assignment
 //  cout<<"block 4 done"<<endl;
      // these 9 assignments should neither influence w, nor temp10
 //  cout<<"got to loc 5"<<endl;
   vi=wOrig; // assignment
   vf=temp10; // assignment
 //  for (i=mi;i<=mf;++i) cout<<"vi[i] = "<<vi[i]<<endl;
 //  for (i=mi;i<=mf;++i) cout<<"vf[i] = "<<vf[i]<<endl;
//   cout<<"block 5 done"<<endl;
   R t2=cpmtime();
   R t12=t2-t1; // time needed for copy and assignment
   R err=0.,sum_v=0.;
   for (N i=mi;i<=mf;++i){
     // cout<<"i-loop "<<i<<" started"<<endl;
     // cout<<"vi[i] = "<<vi[i]<<" vf[i] = "<<vf[i]<<endl;
      err+=abs(vi[i]-vf[i]);
      sum_v+=abs(vf[i]);
   }
//   cout<<"block 6 done"<<endl;
   R t3=cpmtime();
   R t23=t3-t2;
   V<R> res{t12,t23,t12+t23,err,sum_v};
   /*
      Convenient  l i s t  c o n s t r u c t o r, requires C++11
   */
   //bool b=std::movable<Vec<T>>;
   //cout<<"std::movable<Vec<T>> = "<<b<<endl;

  CPM_MZ
  return res;
}

template<typename T, template<typename> class Vec, N fi>
V<R> perMesMove3(N m, N n)
{
   Z mL=1;

   Word loc("perMesMove3(Z,Z)"); // messages use this as name of the function
   CPM_MA
   R t1=cpmtime();
 //  cout<<"perMesMove1 entered"<<endl;
   N i,j;
   N ni=fi;
   N nf=fi+n-1;
   N mi=fi;
   N mf=fi+m-1;
   cout<<"m = "<<m<<endl;

   Vec<T> vi, vf;
   Vec<T> w1(m);
   cout<<"we write the components of w1:  "<<endl;
   for (i=mi;i<=mf;++i) cout<<"w1["<<i<<"] = "<<w1[i]<<endl;
   Vec<T> w2(m);
   cout<<"got to loc 1"<<endl;
   for (i=mi;i<=mf;++i){
      cout<<" index in loop = "<<i<<endl;
      T yi=cz<T>(i-mi);
      cout<<" yi in loop = "<<yi<<endl;
      T zi=yi*yi;
      cout<<" zi in loop = "<<zi<<endl;
      w1.set_(i,yi);
      cout<<"w1.set_ done"<<endl;
      w2.set_(i,zi);
      cout<<"w2.set_ done"<<endl;
   }
   cout<<"got to loc 2"<<endl;
         // The preferred form of this statement in C+- is
         // for (i=v1.b();i<=v1.e();++i) v1[i]=cz(i);
         // Written in this form, it is alo correct if v1 is of type V<C>,
         // the C+- array for which indexing starts with 1 instead of 0.
   Vec<T> wOrig=w1;
  // Vec<T> wOrig(m);
  // for (i=mi;i<=mf;++i) wOrig[i]=w1[i];
  cout<<"got to loc 3"<<endl;

   Vec<T> temp1=w1; // copy constructor
 //  for (i=mi;i<=mf;++i) cout<<"temp1[i] = "<<temp1[i]<<endl;
   //cout<<"temp1 = "<<temp1<<endl;
   Vec<T> temp2=temp1; // copy constructor
   Vec<T> temp3=temp2; // copy constructor
   Vec<T> temp4=temp3; // copy constructor
   Vec<T> temp5=temp4; // copy constructor
   Vec<T> temp6=temp5; // copy constructor
   Vec<T> temp7=temp6; // copy constructor
//   for (i=mi;i<=mf;++i) cout<<"temp7[i] = "<<temp7[i]<<endl;
   Vec<T> temp8=temp7; // copy constructor
   Vec<T> temp9=temp8; // copy constructor
   Vec<T> temp10=temp9; // copy constructor
//   cout<<"third block done"<<endl;
   cout<<"got to loc 4"<<endl;
   temp1=w2; // assignment
   temp2=w2; // assignment
   temp3=temp2; // assignment
   temp4=w2; // assignment
   temp5=temp4; // assignment
   temp6=w2; // assignment
   temp7=temp3; // assignment
   temp8=temp2; // assignment
   temp9=temp1; // assignment
 //  cout<<"block 4 done"<<endl;
      // these 9 assignments should neither influence w, nor temp10
   cout<<"got to loc 5"<<endl;
   vi=wOrig; // assignment
   vf=temp10; // assignment
 //  for (i=mi;i<=mf;++i) cout<<"vi[i] = "<<vi[i]<<endl;
 //  for (i=mi;i<=mf;++i) cout<<"vf[i] = "<<vf[i]<<endl;
//   cout<<"block 5 done"<<endl;
   R t2=cpmtime();
   R t12=t2-t1; // time needed for copy and assignment
   R err=0.,sum_v=0.;
   for (N i=mi;i<=mf;++i){
     // cout<<"i-loop "<<i<<" started"<<endl;
     // cout<<"vi[i] = "<<vi[i]<<" vf[i] = "<<vf[i]<<endl;
      err+=abs(vi[i]-vf[i]);
      sum_v+=abs(vf[i]);
   }
//   cout<<"block 6 done"<<endl;
   R t3=cpmtime();
   R t23=t3-t2;
   V<R> res{t12,t23,t12+t23,err,sum_v};
   /*
      Convenient  l i s t  c o n s t r u c t o r, requires C++11
   */
   //bool b=std::movable<Vec<T>>;
   //cout<<"std::movable<Vec<T>> = "<<b<<endl;

  CPM_MZ
  return res;
}



N tutorial1(N m, N n, N diff)
{
   Z mL=1;
   Word loc("tutorial1(Z,Z)");
   cout<<"tutorial1 entered"<<endl;

   CPM_MA
   V<V<R>> resL;
   if (diff ==0){
      resL<<perMesMove0<std::complex<R>,std::vector,0>(m,n);
      resL<<perMesMove0<std::complex<R>,std::valarray,0>(m,n);
      resL<<perMesMove0<CpmRoot::C,CpmArrays::V,1>(m,n);
      resL<<perMesMove0<CpmRoot::C,CpmArrays::V_,0>(m,n);
      resL<<perMesMove0<CpmRoot::C,CpmArrays::V__,0>(m,n);
      resL<<perMesMove0<CpmRoot::C,CpmArrays::Vv,0>(m,n);
   }
   else{
      resL<<perMes<std::complex<R>,std::vector,0>(m,n);
      resL<<perMes<std::complex<R>,std::valarray,0>(m,n);
      resL<<perMes<CpmRoot::C,CpmArrays::V,1>(m,n);
      resL<<perMes<CpmRoot::C,CpmArrays::V_,0>(m,n);
      resL<<perMes<CpmRoot::C,CpmArrays::V__,0>(m,n);
      resL<<perMes<CpmRoot::C,CpmArrays::Vv,0>(m,n);
   }
   V<Word> res1{"vector","valarray","V","V_","V__","Vv"};
   N nw=res1.size();
   Vo<R> dat1(nw);
   Vo<R> dat2(nw);
   Vo<R> dat3(nw);

   N i;
   for (i=1;i<=nw;++i) dat1[i]=resL[i][1];
   for (i=1;i<=nw;++i) dat2[i]=resL[i][2];
   for (i=1;i<=nw;++i) dat3[i]=resL[i][3];

   V<Z> per1=dat1.permutationForIncreasingOrder();
   V<Z> per2=dat2.permutationForIncreasingOrder();
   V<Z> per3=dat3.permutationForIncreasingOrder();
   R err=0.;
   for (i=1;i<=nw;++i) err+=resL[i][4]; // err>=0. is obvious from the definition

   V<Word> res1o=res1.permute(per1);
   V<R> dat1o=dat1.permute(per1);

   V<Word> res2o=res1.permute(per2);
   V<R> dat2o=dat2.permute(per2);

   V<Word> res3o=res1.permute(per3);
   V<R> dat3o=dat3.permute(per3);

   cout<<endl<<"Output of function tutorial1("<<m<<", "<<n<<")"<<endl;
   cout<<"total error ="<<err<<endl;
   cout<<"The lists are ordered for increasing times."<<endl;
   cout<<endl<<"execution times for copy and assignment:"<<endl;
   for (i=1;i<=nw;++i){
      cout<<res1o[i].std()<<": "<<dat1o[i]<<endl;
   }
   cout<<endl<<"execution times for verification:"<<endl;
   for (i=1;i<=nw;++i){
      cout<<res2o[i].std()<<": "<<dat2o[i]<<endl;
   }
   cout<<endl<<"total execution times:"<<endl;
   for (i=1;i<=nw;++i){
      cout<<res3o[i].std()<<": "<<dat2o[i]<<endl;
   }
   CPM_MZ
   return 0;
}


void info(){
   cout<<endl<<"takes zero, one, or two integer argument";
   cout<<endl<<"compares the performance or std::vector and CpmArays::V";
   cout<<endl<<"in copy construction and assignment operations"<<endl;
}
// end of tut1.cpp

