//? cpmdefinitions.h
//? C+- by Ulrich Mutze. Status of work 2022-03-21.
//? Copyright (c) 2020 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//?
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//?
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for
//? more details.


#ifndef CPM_DEFINITIONS_H_
#define CPM_DEFINITIONS_H_
/*************************************************************************
   cpmdefinitions.h
   Description: In this file one can place defines that are valid in all
   translation units working with Cpm classes
   Cpm = Classes for Physics and Mathematics || C+- (C plus minus)
         -           -           -                   - -    -
************************************************************************/
// This file will never be included by a C+- program since its name is
// cpmdefinitionswrc.h instead of cpmdefinitions.h. Upon renaming, it can
// function as cpmdefinitions.h.
// The name appendix 'wrc' stands for 'with reference comments'.
// We dont't use the more readable form cpmdefinitions_wrc.h in order
// not to disturb pdflatex-based Rugy script listing generation.
// The role of this file is to hold valid and complete explanations of the
// macros which can be set in any file cpmdefinitions.h.
// A file with similar scope is cpmsystemdependencieswrc.h.

#define CPM_NAMEOF
   // If this is defined, the C+- naming scheme is used instead of
   // C++ typeinfo. Recommended since for template types, native
   // typeinfo creates inconveniently long names.

//#define CPM_LONG
   // If this is defined, the types Z and N are long int and unsigned
   // long int, otherwise simply int and unsigned int.
   // If this is defined and CPM_MP is not defined the type R is
   // long double (which, unfortunately, is not always longer than
   // double).
   // Was not found to make much of a difference.

#define CPM_DOUBLE

#define CPM_RANGE_CHECK
   // If this is defined, all indexed access to arrays is index-
   // checked.
   // Recommended, increases safty.

#define CPM_USECOUNT

#define CPM_Fn
//#undef CPM_Fn

//#define CPM_MP 20
   // This macro enables usage of multiple precision
   // mathematics and assumes that the libraries gmp and
   // mpfr are available for linking (and, of course, their headers
   // for inclusion). Also the files mpreal.cpp, mpreal.h (by Pavel
   // Holoborodko) need to be available for compilation and for
   // inclusion repectively. There is, however also a mode of installation
   // of mpfr and mpreal in which it is sufficient to have a
   // '#include <mpreal.h> once in the user code.
   // The number of decimal places may also be set in cpmconfig.ini.
   // This line has only than an effect if CPM_MP is already defined.

#include <cpmcompdef.h>
   // This allows the defines given to the compiler via options of type
   // -D to override the definitions of CPM_LONG and CPM_MP given
   // previously in this file. Although, for instance, a compiler option
   // -DCPM_R_MP_E asks for real numbers of 256 decimal places, a line
   // CPM_MP 100
   // in file config.ini will override this to 100 decimal places.

#endif
