//? cpmsystemdependencies.h
//? C+- by Ulrich Mutze. Status of work 2022-03-21.
//? Copyright (c) 2020 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//?
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//?
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for
//? more details.


#ifndef CPM_SYSTEMDEPENDENCIES_H_
#define CPM_SYSTEMDEPENDENCIES_H_
/*************************************************************************
   cpmsystemdependencies.h
   Description: By this file one can place defines in implementation
    files in cases that the proper implementation is dependent on the
    computer system, e.g. on the access to a display or a file system.
    These defines may also modify the way how we use such facilities;
    e.g. using or not using the graphical display. Or, creating or
    not creating log files. Especially such changes of usage should
    not entail too much recompilation. So the present file should
    never be included in header files. Presently it is only included
    in six files: cpmnumbers.cpp, cpmsystem.cpp, cpmapplication.cpp
    cpmgraph.cpp, cpmimg24.cpp, cpminifilebasapp.cpp.

    Notice that changes in file cpmdefinitions.h trigger recompilation
    of virtually all C+- translation units.

    See also cpmconfig.ini, where most data can be overwritten.

************************************************************************/

#define CPM_REL_POS1 "."
#define CPM_REL_POS2 "./control"
#define CPM_REL_POS3 ".."
#define CPM_REL_POS4 "../control"

#define CPM_INP_DIR "./control"
#define CPM_OUT_DIR "."

#define CPM_FONT_DIR "../fonts"

#define CPM_HEIGHT 800
#define CPM_WIDTH  1200

#define CPM_PANE1 350
#define CPM_PANE2 250
#define CPM_PANE3 250
#define CPM_PANE4 100

#define CPM_STEREOLEFT_R 255
#define CPM_STEREOLEFT_G 0
#define CPM_STEREOLEFT_B 0

#define CPM_STEREORIGHT_R 0
#define CPM_STEREORIGHT_G 255
#define CPM_STEREORIGHT_B 255

#define CPM_RUN_ID_LENGTH 4

#define CPM_WRITE_PRECISION 8

#define CPM_WRITE_TITLE

#define CPM_CERR_EXTEND


//#define CPM_M
   // use M<> instead of map<> in quantum mechanical creation and
   // destruction operators
#undef CPM_M

//#define CPM_USE_EIGEN

#if defined(CPM_USE_EIGEN)
   // using the Eigen library for fast matrix and vector algorithms
   // for real and for complex quantities. The algorithms for
   // eigenvalues/eigenvectors and SVD do not cooperate perfectly with
   // multiple. The many resulting warnings can be disabled by "-w".
   // If multiple precision is based on boost instead of mpreal.h
   // one has even to compile with "-fpermissive" to avoid errors.
   #include <eigen3/Eigen/Dense>
#endif

//#define CPM_NOLOG

//#define CPM_NOGRAPHICS

#if !defined(CPM_NOGRAPHICS)
   #include <GL/glut.h>
#endif

#endif
