//? vqm2app2.cpp
//? C+- by Ulrich Mutze. Status of work 2022-03-21.
//? Copyright (c) 2020 Ulrich Mutze
//? contact: see contact-info at www.ulrichmutze.de
//? 
//? This program is free software: you can redistribute it and/or
//? modify it under the terms of the GNU General Public License as
//? published by the Free Software Foundation, either version 3 of
//? the License, or (at your option) any later version.
//? 
//? This program is distributed in the hope that it will be useful,
//? but WITHOUT ANY WARRANTY; without even the implied warranty of
//? MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//? GNU General Public License <http://www.gnu.org/licenses/> for 
//? more details.


/************************************************************************
   vqm2app2.cpp
***********************************************************************/
#include <vqm2app2.h>
#include <vqm2.h>

#include <cpmframes.h>
#include <cpmgreg.h>
#include <cpmrfuncrec.h>
#include <cpmoperator.h>
#include <cpmtrj.h>
#include <cpmfieldoperator.h>
#include <cpmstate2.h>
#include <cpmfft.h>
#include <cpmmaxw1d.h>
#include <cpmqmtemplates.h>

using namespace CpmSystem;
using namespace CpmRoot;
using namespace CpmRootX;
using namespace CpmArrays;
using namespace CpmFunctions;
using namespace CpmGraphics;
using namespace CpmImaging;
using namespace CpmQuantumMechanics;

using CpmAlgorithms::CyclicAlarm;
using CpmTime::Greg;
using CpmWaves::TEM;
using CpmWaves::TEM2;

////////////////////// App11 /////////////////////////////

void CpmVQM2::App11::doTheWork()
// 1-dimensional case: Euler formla exp(-i omega t)
// studying the difference between the exact solution and the
// solution obtained by the direct midpoint integrator
// visualizing splining
// Made from App10 (2006-10-13)
{
   Z mL=1;
   Word loc("CpmVQM2::App11::doTheWork()");
   CPM_MA
//////////////////////////////////////////////////////////////////////
   Word sec("run control");
//////////////////////////////////////////////////////////////////////
   Z nSteps,nSubSteps;
   R tInspectCurves0,tInspectCurves1,tInspectCurves2,tInspectCurves3;
   B antiAlias;
   cpmrh(nSteps);
   cpmrh(nSubSteps);
   cpmrh(tInspectCurves0);
   cpmrh(tInspectCurves1);
   cpmrh(tInspectCurves2);
   cpmrh(tInspectCurves3);
   cpmrh(antiAlias);
 ////////////////////////////////////////////////////////////////////
   sec="system data";
//////////////////////////////////////////////////////////////////////
   R omega=1,dt,phase0,chi_x,chi_y;
   cpmrh(omega);
   cpmrh(chi_x);
   cpmrh(chi_y);
   cpmrh(dt);
   cpmrh(phase0);
 // start working
   OFileStream out1(outFiles[1]);
   Greg date;
   date.now();
   out1()<<"// ";
   date.writeFormatted(out1());
   out1()<<"// output of CpmVQM2::App11::doTheWork()"<<endl;
   C psi0=C::expi(phase0);
   C psi=psi0;
   C dot=-C::I*omega;
   C dot2=dot*dot;
   C phi=dot*psi;
   phi+=C(chi_x,chi_y);
   R tau=0.5*dt;
   C psiMid=psi+phi*tau;
   C alpha=dot*dot*psiMid;
      // initial condition is ready
   Graph gr(0.05,0.05,0.9,0.9);
   gr.clr();
   gr.setAntiAliasLines(antiAlias);

   R t=0;
   R tSteps=0;

   R dtSub=dt/nSubSteps;
   vector<R> tVal;
   vector<R> rpsiEx;
   vector<R> ipsiEx;
   vector<R> rpsiRep;
   vector<R> ipsiRep;
   vector<R> rspEx;
   vector<R> ispEx;
   vector<R> rspRep;
   vector<R> ispRep;
   vector<R> da;
   R tTot=nSteps*dt;
   while (t<tTot){
      R phase=-omega*t;
      C psiEx=C::expi(phase)*psi0;
      C phiEx=dot*psiEx;
      R aPsiEx=psiEx.abs();
      R rPsiEx=psiEx.real();
      R iPsiEx=psiEx.imag();
      C spEx=phiEx.con()*psiEx;
      R rSpEx=spEx.real();
      R iSpEx=spEx.imag();
      rpsiEx.push_back(rPsiEx);
      ipsiEx.push_back(iPsiEx);
      rspEx.push_back(rSpEx);
      ispEx.push_back(iSpEx);

      R h=t-tSteps;
      if (h>=dt){
         psiMid=psi+phi*tau;
       //  alpha=dot2*psiMid; // alpha has valid value yet
         phi+=alpha*dt;
         psi=psiMid+phi*tau;
         tSteps+=dt;
         h=t-tSteps;
         psiMid=psi+phi*tau;
         alpha=dot2*psiMid; // this is alpha to be
         // used in the next statements
      }
      C psiRep=psi+phi*h+alpha*(h*h*0.5);
      C phiRep=phi+alpha*h;
      C spRep=phiRep.con()*psiRep;
      R aPsiRep=psiRep.abs();
      R rPsiRep=psiRep.real();
      R iPsiRep=psiRep.imag();
      rpsiRep.push_back(rPsiRep);
      ipsiRep.push_back(iPsiRep);
      R rSpRep=spRep.real();
      R iSpRep=spRep.imag();
      rspRep.push_back(rSpRep);
      ispRep.push_back(iSpRep);
      da.push_back(aPsiRep-aPsiEx);
      tVal.push_back(t);
      t+=dtSub;
   }// time loop
   R_Vector t_val(tVal);
   R_Vector rpsiEx_(rpsiEx);
   R_Vector ipsiEx_(ipsiEx);
   R_Vector rpsiRep_(rpsiRep);
   R_Vector ipsiRep_(ipsiRep);
   R_Vector rspEx_(rspEx);
   R_Vector ispEx_(ispEx);
   R_Vector rspRep_(rspRep);
   R_Vector ispRep_(ispRep);
   R_Vector da_(da);

   if (tInspectCurves0>0){
      R_Matrix show(3);
      show[1]=t_val;
      show[2]=rpsiEx_;
      show[3]=rpsiRep_;
      gr.addText("psiExact Real, psiRep Real");
      gr.show(show);
      cpmwait(tInspectCurves0);
   }
   if (tInspectCurves1>0){
      R_Matrix show(5);
      show[1]=t_val;
      show[2]=rpsiEx_;
      show[3]=ipsiEx_;
      show[4]=rpsiRep_;
      show[5]=ipsiRep_;
      gr.addText("psiExact Real and Img, psiRep Real and Img");
      gr.show(show);
      cpmwait(tInspectCurves1);
   }
   if (tInspectCurves2>0){
      R_Matrix show(5);
      show[1]=t_val;
      show[2]=rspEx_;
      show[3]=ispEx_;
      show[4]=rspRep_;
      show[5]=ispRep_;
      gr.addText("scalar product phi times psi, Real Img Exact Rep");
      gr.show(show);
      cpmwait(tInspectCurves2);
   }
   if (tInspectCurves3>0){
      R_Matrix show(2);
      show[1]=t_val;
      show[2]=da_;
      gr.addText("norm Rep - norm Exact");
      gr.show(show);
      cpmwait(tInspectCurves3);
   }
   CPM_MZ
}

////////////////////// App12 /////////////////////////////

void CpmVQM2::App12::doTheWork()
// 1-dimensional case: Euler formla exp(-i omega t)
// studying the difference between the exact solution and the
// solution obtained by the direct midpoint integrator
// Made from App11 (2006-11-15)
{
   Z mL=1;
   Word loc("CpmVQM2::App12::doTheWork()");
   CPM_MA
//////////////////////////////////////////////////////////////////////
   Word sec("run control");
//////////////////////////////////////////////////////////////////////
   R nPeriods;
   R tInspectCurves0,tInspectCurves1,tInspectCurves2,tInspectCurves3;
   B antiAlias;
   cpmrh(nPeriods);
   cpmrh(tInspectCurves0);
   cpmrh(tInspectCurves1);
   cpmrh(tInspectCurves2);
   cpmrh(tInspectCurves3);
   cpmrh(antiAlias);
 ////////////////////////////////////////////////////////////////////
   sec="system data";
//////////////////////////////////////////////////////////////////////
   R omega=1,dt,phase0;
   cpmrh(omega);
   cpmrh(dt);
   cpmrh(phase0);
 // start working
   OFileStream out1(outFiles[1]);
   Greg date;
   date.now();
   out1()<<"// ";
   date.writeFormatted(out1());
   out1()<<"// output of CpmVQM2::App11::doTheWork()"<<endl;
   C psi0=C::expi(phase0);
   C dot=-C::I*omega;
   C dot2=dot*dot;
   C phi0=dot*psi0;

   C psiEx=psi0;
   C phiEx=phi0;

   C psiDM=psi0;
   C phiDM=phi0;

   Graph gr(0.05,0.05,0.9,0.9);
   gr.clr();
   gr.setAntiAliasLines(antiAlias);

   R t=0;
   R tau=0.5*dt;
   R tSteps=0;

   vector<R> tVal;
   vector<C> vPsiEx;
      // Ex: exact
   vector<C> vPsiDM;
      // DM: direct midpoint
   vector<C> vPhiEx;
   vector<C> vPhiDM;
   vector<R> vnu;

   R tTot=nPeriods*cpm2pi/omega;
   while (t<tTot){
      tVal.push_back(t);
      vPsiEx.push_back(psiEx);
      vPhiEx.push_back(phiEx);
      vPsiDM.push_back(psiDM);
      vPhiDM.push_back(phiDM);
      R vu=psiDM.absSqr()-1;
      vnu.push_back(vu);
      t+=dt;
      R phase=-omega*t;
      psiEx=C::expi(phase)*psi0;
      phiEx=dot*psiEx;

      psiDM+=phiDM*tau;
      C alpha=dot2*psiDM;
      phiDM+=alpha*dt;
      psiDM+=phiDM*tau;

   }// time loop
   R_Vector t_val(tVal);
   R_Vector v_nu(vnu);
   V<C> v_psiDM(1,vPsiDM);
   V<C> v_psiEx(1,vPsiEx);
   V<C> v_phiDM(1,vPhiDM);
   V<C> v_phiEx(1,vPhiEx);
   Z i,n=t_val.dim();
   R_Vector vrpsiDM(n);
   R_Vector vrpsiEx(n);
   R_Vector vrphiDM(n);
   R_Vector vrphiEx(n);
   for (i=1;i<=n;++i){
      vrpsiDM[i]=v_psiDM[i][1];
      vrpsiEx[i]=v_psiEx[i][1];
      vrphiDM[i]=v_phiDM[i][1];
      vrphiEx[i]=v_phiEx[i][1];
   }
   /*R_Vector rpsiEx_(rpsiEx);
   R_Vector ipsiEx_(ipsiEx);
   R_Vector rpsiRep_(rpsiRep);
   R_Vector ipsiRep_(ipsiRep);
   R_Vector rspEx_(rspEx);
   R_Vector ispEx_(ispEx);
   R_Vector rspRep_(rspRep);
   R_Vector ispRep_(ispRep);
   R_Vector da_(da);*/

   if (tInspectCurves0>0){
      R_Matrix show(2);
      show[1]=t_val;
      show[2]=v_nu;
    //  show[3]=rpsiRep_;
      gr.addText("nu");
      gr.show(show);
      cpmwait(tInspectCurves0);
   }
   if (tInspectCurves1>0){
      R_Matrix show(3);
      show[1]=t_val;
      show[2]=vrpsiEx;
      show[3]=vrpsiDM;
      gr.addText("psiExact Real, psiDM Real");
      gr.show(show);
      cpmwait(tInspectCurves1);
   }
   if (tInspectCurves2>0){
      R_Matrix show(3);
      show[1]=t_val;
      show[2]=vrphiEx;
      show[3]=vrphiDM;
      gr.addText("phiExact Real, phiDM Real");
      gr.show(show);
      cpmwait(tInspectCurves2);
   }
/***************
   if (tInspectCurves2>0){
      R_Matrix show(5);
      show[1]=t_val;
      show[2]=rspEx_;
      show[3]=ispEx_;
      show[4]=rspRep_;
      show[5]=ispRep_;
      gr.addText("scalar product phi times psi, Real Img Exact Rep");
      gr.show(show);
      cpmwait(tInspectCurves2);
   }
   if (tInspectCurves3>0){
      R_Matrix show(2);
      show[1]=t_val;
      show[2]=da_;
      gr.addText("norm Rep - norm Exact");
      gr.show(show);
      cpmwait(tInspectCurves3);
   }
*******************************/
   CPM_MZ
}


////////////////////// App13 /////////////////////////////

C fp(R const& x){
   R y=6*x;
   return C(cpmsin(y),cpmcos(y))*cpmexp(-y*y)*cpmexp(-1./(0.25-x*x));
}

/*******************

void CpmVQM2::App13::doTheWork()
{
   Z mL=1;
   Word loc("CpmVQM2::App13::doTheWork()");
   CPM_MA
   static R fac=-QMBase::get_hDiracInv();
   Z d=128;
   Z i,n=4;
   R tWait=60;
   R tWaitFrame=0.;
  // F<R,C> f_psi(fp);
  // State psi(f_psi,d);
   Observable ob("eKinF",d);
   SpectralObject so=ob.spectralRepresentation();
   Z im=115;
   State psi_1=so[im];
   State psi_2=so[im+3]*C(0.5);
   State psi_3=so[im-5]*C(0,0.8);

   State psi=psi_1+psi_2+psi_3;
   psi.normalize_();
 //  State stF=so.represent(psi);
   R dt=1e-3;
   so.expi(dt*fac);
   Graph g;
   Z nf=256;
   psi.showAsCurves(g);
   g.setAutoScale(0);
   State psi_t=psi;
   V<C> v(nf);
   R nfInv=1./(nf-1);
   for (i=1;i<=nf;++i){
      cpmprogress("i-loop",(i-1)*nfInv);
      psi_t=so*psi_t;
      v[i]=(psi|psi_t);
      psi_t.showAsCurves(g);
      cpmwait(tWaitFrame);
   }
   V<C> e=CpmFourier::fft(v,-1);
   State st("",e);
 //  State st("",v);
   g.setAutoScale(1);
   st.showAsCurves(g);
   cpmwait(tWait);
  // stF.showAsCurves(g);
 //  cpmwait(tWait);
   CPM_MZ
}
*********************************/

void CpmVQM2::App13::doTheWork()
{
   Z mL=1;
   Word loc("CpmVQM2::App13::doTheWork()");
   CPM_MA
   static R fac=-QMBase::get_hDiracInv();
   Z d=128;
   Z i,n=4;
   R tWait=60;
   R tWaitFrame=0.;
  // F<R,C> f_psi(fp);
  // State psi(f_psi,d);
   Observable ob(Word("eKinF"),d);
   SpectralObject so=ob.spectralRepresentation();
   Z im=115;
   State psi_1=so[im];
   State psi_2=so[im+3]*C(0.5);
   State psi_3=so[im-20]*C(0,0.8);
   State psi_4=so[im-30];

   State psi=psi_1+psi_2+psi_3+psi_4;
   psi.normalize_();

   R dt=1e-1;
   R dx=dt*fac;
   so.expi(dx);
   Graph g;
   Z nf=500;
   psi.showAsCurves(g);
   g.setAutoScale(0);
   State psi_t=psi;
   R nfInv=1./(nf-1);
   R beta=0;
   R alpha=(psi_t|ob*psi_t).real();
   R_Vector va(nf+1);
   Z j0=200;
   va[1]=alpha;
   for (i=1;i<=nf;++i){
      cpmprogress("i-loop",(i-1)*nfInv);
      State psi_d=so*psi_t;
      State psi_next=psi_d*C::expi(-dx*alpha);
      R r=CpmRoot::randomR(j0++);
      Z sig=r<0.5 ? 1 : -1;
      beta=(psi_next-psi_t).absSqr()*0.25*sig;
      R delta=cpmsqrt(cpmabs(psi_d.absSqr()-alpha*alpha));
      psi_t=psi_next;
      alpha+=beta*delta;
     // alpha=(psi_t|ob*psi_t).real();
      va[i+1]=alpha;
      psi_t.showAsCurves(g);
     // cpmwait(tWaitFrame);
   }
   psi_t.showAsCurves(g);
   cpmwait(10);
   g.setAutoScale(1);
   g.show(va);
   cpmwait(tWait);
   CPM_MZ
}


////////////////////// App14 /////////////////////////////
// Maxwell 1D system, inspired by Hans De Raedt

namespace{
   R gauss14(R const& x, R const& x0, R const& sigma)
   {
      R a=(x-x0)/sigma;
      return cpmexp(-a*a);
   }

   R sin1(R const& x){ return cpmsin(x);}
   R cos1(R const& x){ return cpmcos(x);}
}

//#undef USE_TEM2
#define USE_TEM2

void CpmVQM2::App14::doTheWork()
{
#if defined(USE_TEM2)
   typedef TEM2 AppSys;
   Word app("TEM2");
#else
   typedef TEM AppSys;
   Word app("TEM");
#endif
   Z mL=2;
   Word loc("CpmVQM2::App14::doTheWork()");
   CPM_MA
// reading control data from a text file
   Word sec("system definition");
   Z n, nTrigE=1, nTrigH=1;
   Word typeOfIniState="gauss";
   R L,x0,sigma,xJ,tiJ,tfJ,omegaJ;
   Z nSmoothJ;
   cpmrh(n);
   cpmrh(L);
   cpmrh(x0);
   cpmrh(xJ);
   cpmrh(tiJ);
   cpmrh(tfJ);
   cpmrh(omegaJ);
   cpmrh(nSmoothJ);
   cpmrh(sigma);
   cpmrh(nTrigE);
   cpmrh(nTrigH);
   cpmrh(typeOfIniState);
///////////////////////////////////////
   sec="run control";
   Z nSteps=10,viewingPeriod=1,nRuns=1;
   R dt,tWait=0.5;
   R xViewRelLower=0, xViewRelUpper=1, yView=-1;
   B autoScale=B(true);
   cpmrh(nSteps);
   cpmrh(dt);
   cpmrh(nRuns);
   cpmrh(viewingPeriod);
   cpmrh(tWait);
   cpmrh(autoScale);
   cpmrh(xViewRelLower);
   cpmrh(xViewRelUpper);
   cpmrh(yView);
// Now all data known and we build the initial state
   R_Func fE,fH;
   R sca=1./(dt*dt);
   AppSys tem;
   if (typeOfIniState=="gauss"){
      // proposed by Hans De Raedt
      R Linv=1./L;
      x0*=Linv;
      sigma*=Linv;
#ifdef CPM_Fn
      fE=F2<R,R,R,R>(x0,sigma)(gauss14);
#else
      fE=F<R,R>(bind(gauss14,_1,x0,sigma));
#endif
      tem=AppSys(n,L,fE,fH);
   }
   else if (typeOfIniState=="trigonometric"){
      // simple smooth functions, satisfying the boundary
      // conditions. Not proposed by Hans De Raedt
      R sE=1./(cpmpi*nTrigE), sH=1./(cpmpi*nTrigH);
      fE=R_Func(sin1);
      fH=R_Func(cos1);
      fE.stretch_(sE);
      fH.stretch_(sH);
      tem=AppSys(n,L,fE,fH);
   }
   else if (typeOfIniState=="stochastic")
      // proposed by Hans De Raedt
      tem=AppSys(n,L,Iv(-1,1));
#if defined(USE_TEM2)
   else if (typeOfIniState=="current"){
      // proposed by Hans De Raedt
      tem=TEM2(n, L, xJ, Iv(tiJ,tfJ),omegaJ,nSmoothJ);
   }
#endif
   else
      cpmerror("bad selection for typeOfIniState");
   AppSys tem0=tem;
   R en0=tem.energy();
   R dis0=tem.dissipation();
   R nStepsInv=1./nSteps;
   R t0=0,t;
   Z i,j;
// creating resources for storing results from nRuns
// consecutive program runs. These runs cover a given span of
// time with more and more time steps. Actually, from run
// to run the number of time steps doubles.
   V< R_Vector > vRes(nRuns);
   R_Vector vdt(nRuns);
   V< V<R2> > vE(nRuns);
   V< V<R2> > vH(nRuns);
   V< V<R2> > nu_scaled(nRuns);
   V< V<R2> > eps_scaled(nRuns);
   Graph g;
   Z order=2;
   Z facSteps=2;
   R facStepsInv=1./facSteps;
   R facSca(CpmRootX::pow(facSteps,order));
   CyclicAlarm show(viewingPeriod);
   for (j=1;j<=nRuns;++j){ // loop of runs
      tem=tem0;
      if (j==1){
         g.setAutoScale(1);
         tem.showAsCurves(g);
         g.write(Word("initialFrame_")&app);
      }
      t=t0; // each run starts with the same field and the same model
      // time
      cpmvalue("run index ",j,3);
      nu_scaled[j]=V<R2>(nSteps);
      eps_scaled[j]=V<R2>(nSteps);
      vdt[j]=dt;
      for (i=1;i<=nSteps;++i){ // loop of steps
// performing a time step
         tem.step_(dt);
         t+=dt;
      //   R nu=(tem.energy()-en0)*sca;
      //   R nu=(tem.energy()-en0)/en0;
         R nu=tem.energy();
         R eps=(tem.dissipation()-dis0)*sca;
         //  R nu=tem.energy();
         // R eps=tem.dissipation();
// storing data concerning the state reached by the time step
         nu_scaled[j][i]=R2(t,nu);
         eps_scaled[j][i]=R2(t,eps);
         R L=tem.getL();
         R xL=L*xViewRelLower;
         R xU=L*xViewRelUpper;
         Iv yRange(-yView,yView);
         Iv xRange(xL,xU);
         if (j==1){
            if (show()||i==nSteps){ // visualizing the evolution
               if (autoScale) g.setAutoScale(1);
               else g.setXY(xRange,yRange);
               tem.showAsCurves(g);
            }
            if (i==nSteps){
               g.write(Word("finalFrame_")&app);
               cpmwait(tWait);
            }
         }
         cpmprogress("loop",i*nStepsInv);
      } // loop of steps
// storing results from run j
      vRes[j]=tem.getXr();
      V< V<R2> > vvr=tem.getEH();
      vE[j]=vvr[1];
      vH[j]=vvr[2];
// setting the parameters for run j+1
      nSteps*=facSteps;
      nStepsInv*=facStepsInv;
      dt*=facStepsInv;
      sca*=facSca;
   } // loop of runs
// putting the results from the various runs together

   V<R_Vector> diff(nRuns);

   for (j=1;j<=nRuns;++j){
      diff[j]=vRes[j]-vRes[nRuns];
   }

   R_Matrix diffs(nRuns);

   if (nRuns>1){
      Z dimx=tem.getxr().dim();
      Z dimy=diff[1].dim();
      diffs[1] = dimx==dimy ? tem.getxr() : tem.getxr()&tem.getxr();
      R fac=1;
      for (j=1;j<nRuns;++j){
         diffs[j+1]=diff[j]*fac;
         fac*=facSca;
      }
   }

// showing documentation on screen

   g.setAutoScale(1);
   g.mark(nu_scaled);
   g.addText("nu_scaled");
   g.vis();
   cpmwait(tWait);

   g.setAutoScale(1);
   g.mark(eps_scaled);
   g.addText("eps_scaled");
  // g.display();
  // cpmwait(tWait);

   g.clear();
   g.setAutoScale(1);
   g.mark(vE);
   g.addText("final E");
  // g.display();
 //  cpmwait(tWait);

   g.clear();
   g.setAutoScale(1);
   g.mark(vH);
   g.addText("final H");
 //  g.display();
  // cpmwait(tWait);

   if (nRuns>1){
      g.clear();
      g.setAutoScale(1);
      g.addText("scaled differences between runs");
      g.show(diffs);
      cpmwait(2*nRuns*tWait);
   }

// filing documentation data

   R normFinal=vRes[nRuns].abs();
   R normInv=1./normFinal;
   R_Vector res(nRuns);
   for (i=1;i<=nRuns;i++){
      res[i]=diff[i].abs()*normInv;
   }

   OFileStream of1(outFiles[1]);
   of1()<<"Type of initial state is "<<typeOfIniState<<endl;
   of1()<<"Normalized distance dj from final state of run j to final\
    state\
    of final run:"<<endl;
   for (j=1;j<=nRuns;j++){
      of1()<<"j="<<j<<" dt="<<vdt[j]<<" dj="<<res[j]<<endl;
   }

   OFileStream of2(outFiles[2]);
   Z nr=nu_scaled[1].dim();
   for (i=1;i<=nr;++i){
      of2()<<nu_scaled[1][i][1]<<" "<<nu_scaled[1][i][2]<<"\
       "<<eps_scaled[1][i][2]<<endl;
   }
   CPM_MZ
}

////////////////////// App15 /////////////////////////////

void CpmVQM2::App15::doTheWork()
// Derived from App7, now aiming at testing the Chebyshev-method
{
   Z mL=2;
   Word loc("CpmVQM2::App15::doTheWork()");
   CPM_MA
//////////////////////////////////////////////////////////////////////
   Word sec("run control");
//////////////////////////////////////////////////////////////////////
   Z viewingPeriod;
   R travelRel, tWait=0;
   cpmrh(travelRel);
   cpmrh(viewingPeriod);
   cpmrh(tWait);

//////////////////////////////////////////////////////////////////////
   sec="system data";
//////////////////////////////////////////////////////////////////////

   Z d1;
   R size1=1.,massRel1,vRel1;
   R dtRel=0, xRel1=0, wRel1=0;
   B useSI,noRel;

   cpmrh(d1);
   cpmrh(size1);
   cpmrh(noRel);
   cpmrh(massRel1);
   cpmrh(xRel1);
   cpmrh(wRel1);
   cpmrh(vRel1);
   cpmrh(useSI);
   cpmrh(dtRel);
// start working
   Word units=useSI ? "SI":"1";
   OFileStream out1(outFiles[1]);
   Greg date;
   date.now();
   out1()<<"// ";
   date.writeFormatted(out1());
   out1()<<"// output of CpmVQM2::App7::doTheWork()"<<endl;
   out1()<<"units="<<units;
   QMBase::setUnits(units);
   QMBase::setSize(size1);
      // to get the right lattice related data for particle 1
   R m1= noRel ? massRel1 : massRel1*QMBase::mMax(d1);
   // That boosting instances of State correctly, the class
// QMBase has to be adjusted, needs e.g. member mass
   QMBase::setMass(m1);
   out1()<<"d1="<<d1<<endl;
   out1()<<"size1="<<size1<<endl;
   out1()<<"m1="<<m1<<endl;

// kinetic energy as spectral object, which is the most efficient
// type of object for implementing dynamics

   bool ipt=CpmRootX::isPowOfTwo(d1);
// two Hamiltonians to select (indirectly via d1)
   Observable ekin= ipt? Observable(Word("eKin"),d1) :
      Observable(Word("eKinF"),d1);
// initial state:
   bool unitHeg=true; // aha heg = height
   R_Func gpsi1=R_Func::Gauss(xRel1,wRel1,unitHeg);
   State s1(gpsi1,d1);
   R enr=(s1|ekin*s1).real();
   out1()<<"Kinetic energy befor boost: "<<enr<<endl;
   R v1=noRel ? vRel1 : vRel1*0.5*QMBase::vMax(d1);
   out1()<<"v1="<<v1<<endl;
      // 0.5 inserted since without this vRel1=1
      // corresponds to a wave function mainly at
      // Nyquist frequency. Our representation of the Hamilton
      // operator is not very suitable for such strongly
      // varying wave functions
   s1=s1.boost(v1);
   enr=(s1|ekin*s1).real();
   out1()<<"Kinetic energy after boost: "<<enr<<endl;
   enr=0.5*m1*v1*v1;
   out1()<<"Kinetic energy from mass and velocity: "<<enr<<endl;
   Graph gr;
   R dt=noRel ? dtRel : dtRel*QMBase::tRef(d1);
   out1()<<"dt direct ="<<dt<<endl;
   R travel=noRel ? travelRel : travelRel*size1;
   cpmassert(v1!=0.,loc);
   R tTot=travel/v1;
   Z nSteps=cpmrnd(tTot/dt);
   out1()<<"nSteps ="<<nSteps<<endl;
   s1.showAsCurves(gr);
   cpmwait(1+tWait);
      // inspecting the initial state
   if (nSteps>0){
      dt=tTot/nSteps;
      out1()<<"dt adjusted ="<<dt<<endl;
      CyclicAlarm cyc(viewingPeriod);
      R nInv=1./nSteps;
      SpectralObject so;
      {  // encapsulating the heavy initialization of so
         bool enforceSymmetry=false;
         // our H2 is known to be symmetric
         bool test=false;
         // true would consume a lot of time but worked in my test cases
         // perfectly
         Z iterMax=128;
         R eps0=0;
         R eps1=1e-7,eps2=1e-6; // test values from the past
         so=ekin.spectralRepresentation(enforceSymmetry,
         test,iterMax,eps0,eps1,eps2); // this is a big beast
      }
   //   so.expi(-dt);
         // very efficient way of letting free Hamiltonian
         // dynamics act on the boosted state
      R t=0; // exp
      Chebyshev<Observable,State> ch(ekin*C::I,s1);
      for (Z i=0; i<nSteps; ++i){
         t+=dt; // exp
         R ip=i*nInv;
         cpmprogress("loop",ip,1);
         SpectralObject sot=so;
         sot.expi(-t);
         State sSpec=sot*s1;
         State sCheb=ch.step(-t);
         State diff=sSpec-sCheb;

         if (cyc()||i==(nSteps-1)){
            R acc=1e-10;
            gr.setY(Iv(-acc,acc));
         //   s1.showAsCurves(gr);
          //  sCheb.showAsCurves(gr);
            diff.showAsCurves(gr);
            cpmwait(tWait);
         }
      }
   }
   cpmwait(tWaitWorkScreen_);
   CPM_MZ
}


////////////////////// App16 /////////////////////////////


namespace{
//   const Z dHam=100;
//   R omegaHam=2000; // 50,2000
   R wobble(R const& t, R const& omega)
   {
      R y=cpmcos(omega*t);
      return y*y;
   }


   State dFunc(X2<State,R> const& x, Z const& n, R const& omega)
   {
      R lambda=QMBase::lambdaOsc(n);
      R_Func f2=F1<R,R,R>(omega)(wobble);
      R2_Func pot(R_Func::x2,f2);
      State psi=x.c1();
      R t=x.c2();
      return psi.sinParHam(pot,t,lambda)*C(0,-1);
   }
}


class Ham{ // Hamiltonian
// independent data
   Z n_;
   R omega_;
// dependent data
   R lambda_;
   R2_Func pot_;
public:
#ifdef CPM_Fn
   Ham(Z n, R omega):n_(n),omega_(omega),lambda_(QMBase::lambdaOsc(n_)),
      pot_(R_Func::x2,F1<R,R,R>(omega_)(wobble)){}
#else
   Ham(Z n, R omega):n_(n),omega_(omega),lambda_(QMBase::lambdaOsc(n_)),
      pot_(R_Func::x2,F<R,R>(bind(wobble,_1,omega_))){}
#endif

   State operator()(State const& psi, R t)const
      // acting as D not as H
   {
      return psi.sinParHam(pot_,t,lambda_)*C(0,-1);
   }

   bool timeIndependent()const{ return omega_==0.;}
   bool enforceNormalization()const{ return false;}
};

class Inh{
   State st_;
   R omega_;
   R ampl_;
   R phase_;
public:
   Inh(State const& st, R omega, R ampl, R phase):
      st_(st),omega_(omega),ampl_(ampl),phase_(phase){}
   State operator()(R t)const{
      R x=ampl_*cpmcos(omega_*t+phase_);
      return st_*x;
   }
};


void CpmVQM2::App16::doTheWork()
{
   Z mL=2;
   Word loc("CpmVQM2::App16::doTheWork()");
   CPM_MA
   Z d=100;
   R omega=2000,omega0=4000,ampl0=20,phase0=0.1,x=-0.2,sigma=0.05;
   R dtRel=4e-2;
   Z nSteps=20000;
   Z nRunsForOrder=0;
   Z viewingPeriod=40;
   Z integrationMethod=1;
   R tWait=10;
   R tWaitTest1=3;
   R tWaitTest2=6;
   R tWaitTest3=0;
   R tWaitTest4=0;
   R fracRun=0.5;
   B symmetric{true};

   Word sec("system data");
   cpmrh(d);
   cpmrh(omega0);
   cpmrh(ampl0);
   cpmrh(phase0);

   cpmrh(omega);
   cpmrh(x);
   cpmrh(sigma);
   cpmrh(symmetric);

   sec="run control";
   cpmrh(dtRel);
   cpmrh(nSteps);
   cpmrh(nRunsForOrder);
   cpmrh(integrationMethod);
   cpmrh(fracRun);
   cpmrh(viewingPeriod);
   cpmrh(tWait);
   cpmrh(tWaitTest1);
   cpmrh(tWaitTest2);
   cpmrh(tWaitTest3);
   cpmrh(tWaitTest4);

   State psi0(d);
   psi0=psi0.ran();
   psi0=psi0.smooth();

   R_Func f=R_Func::Gauss(x,sigma);
   if (symmetric) f+=R_Func::Gauss(-x,sigma);
   State psi(f,d);
   R t=0;

   Inh inh(psi0,omega0,ampl0,phase0);
   Ham ham(d,omega);

   GMPM<Ham,Inh,State> mp(ham,inh,psi,t);
   mp.setMethod(integrationMethod);

   R dt=QMBase::dtStabilityLimit(d)*dtRel;
   R tSpan=nSteps*dt;
   R t1=0;
   R t2=t1+tSpan*0.45;
   R t3=t1+tSpan;
   R errorSum=0;

   if (tWaitTest1>0){
      R error=GMPM<Ham,Inh,State>::testOfOrder(ham,inh,psi,t1,t2,
      nSteps,nRunsForOrder);
      errorSum+=error;
      cpmvalue("errComposition",error,1);
      cpmwait(tWaitTest1);
   }
   if (fracRun>0){
      R nFrac=fracRun*nSteps;
      nSteps=cpmrnd(nFrac);
      Graph gr;
      R_Vector times(nSteps);
  // R_Matrix norms(3,nSteps);
      R_Matrix norms(2,nSteps);
      CyclicAlarm show(viewingPeriod);
      R nInv=1./nSteps;
      C ene0=mp.getEnergy();
      R ea=ene0[2];
      R eb=ene0[1];
      R facInv=-1./ea;
      mp.getPsi().showAsCurves(gr);
      cpmwait(1);
      for (Z i=1;i<=nSteps;++i){
         mp.step(dt);
         R t=mp.getTime();
         times[i]=t;
         norms[1][i]=mp.getNorm()-1.;
         C ene=(mp.getEnergy()-ene0)*facInv;
     // C ene=mp.getEnergy();
         norms[2][i]=ene[1];
    //  norms[3][i]=ene[2];

         if (show()){
            mp.getPsi().showAsCurves(gr);
            cpmprogress("loop",i*nInv);
         }
      }
      gr.clr();
      gr.showGL(times,norms,rainBow,false);
      cpmwait(tWait);
   }
   CPM_MZ
}

////////////////// void CpmVQM2::function(void) //////////////

// functions of signature void *f(void) for final association with a menu
// selection

void CpmVQM2::app11()
{
   cpmmessage("app11() started");
   App11 a("app11");
   a.run();
}

void CpmVQM2::app12()
{
   cpmmessage("app12() started");
   App12 a("app12");
   a.run();
}

void CpmVQM2::app13()
{
   cpmmessage("app13() started");
   App13 a("app13");
   a.run();
}

void CpmVQM2::app14()
{
   cpmmessage("app14() started");
   App14 a("app14");
   a.run();
}


void CpmVQM2::app15()
{
   cpmmessage("app15() started");
   App15 a("app15");
   a.run();
}

void CpmVQM2::app16()
{
   cpmmessage("app16() started");
   App16 a("app16");
   a.run();
}
